﻿// <copyright file="CommandThread.cs" company="Redpoint Apps">
// Copyright (c) Redpoint Apps. All rights reserved.
// </copyright>

namespace RedPoint.ReefStatus.Common.Commands
{
    using System;
    using System.Collections.ObjectModel;
    using System.Drawing;
    using System.Threading;
    using RedPoint.ReefStatus.Common;
    using RedPoint.ReefStatus.Common.ProfiLux;
    using RedPoint.ReefStatus.Common.UI;

    /// <summary>
    /// The command thread
    /// </summary>
    public class CommandThread : IDisposable, IProtocolCommands
    {
        private IProfilux protocol;
        private readonly ICommandReply replys;
        private readonly object protocolLock = new object();
        private System.Threading.Timer connectionTimer;


        /// <summary>
        /// Initializes a new instance of the <see cref="CommandThread"/> class.
        /// </summary>
        /// <param name="replys">The replys.</param>
        /// <param name="controler">The controler.</param>
        public CommandThread(ICommandReply replys, Controler controler)
        {
            this.replys = replys;
            this.Controler = controler;
        }

        /// <summary>
        /// Gets or sets the controler.
        /// </summary>
        /// <value>The controler.</value>
        public Controler Controler
        {
            get; private set;
        }

        /// <summary>
        /// Gets or sets the progress.
        /// </summary>
        /// <value>The progress.</value>
        public IUpdateProgress Progress { get; set; }

        /// <summary>
        /// Gets a value indicating whether this instance is connected.
        /// </summary>
        /// <value>
        ///     <c>true</c> if this instance is connected; otherwise, <c>false</c>.
        /// </value>
        public bool IsConnected
        {
            get { return this.protocol != null && this.protocol.IsConnected; }
        }

        /// <summary>
        /// Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources.
        /// </summary>
        public void Dispose()
        {
            GC.SuppressFinalize(this);
            this.Stop();
            if (this.protocol != null)
            {
                this.protocol.Disconnect();
            }
        }

        private bool inUpdateDisplayText;

        /// <summary>
        /// Gets the display text.
        /// </summary>
        public void UpdateDisplayText()
        {
            new Thread(() =>
            {
                if (inUpdateDisplayText)
                {
                    return;
                }

                inUpdateDisplayText = true;
                try
                {
                    lock (this.protocolLock)
                    {
                        try
                        {
                            this.Connect();
                            if (this.Controler.Info.IsP3)
                            {
                                this.replys.UpdateDisplayText(
                                    string.Empty, this.GetViewString(), this.GetDisplayImage(), Controler);
                            }
                            else
                            {
                                this.replys.UpdateDisplayText(
                                    this.GetDisplayString(), this.GetViewString(), null, Controler);
                            }
                        }
                        catch (ReefStatusException ex)
                        {
                            this.HandelException(ex);
                        }
                    }
                }
                finally 
                {
                    inUpdateDisplayText = false;
                }
            }).Start();
        }

        /// <summary>
        /// Gets the display image.
        /// </summary>
        /// <returns></returns>
        private Image GetDisplayImage()
        {
            lock (this.protocolLock)
            {
                this.Connect();
                return this.protocol.DisplayImage;
            }
        }

        /// <summary>
        /// Handels the exception.
        /// </summary>
        /// <param name="ex">The ex.</param>
        private void HandelException(ReefStatusException ex)
        {
            try
            {
                if (this.protocol != null && this.protocol.IsConnected)
                {
                    this.Disconnect();
                    this.Connect();
                }
            }
            catch (ReefStatusException)
            {
                this.Disconnect();
            }

            Logger.Instance.LogError(ex);
        }

        private bool inUpdateAll;

        /// <summary>
        /// Updates all.
        /// </summary>
        public void UpdateAll()
        {
            new Thread(() =>
            {
                if (inUpdateAll)
                {
                    return;
                }

                inUpdateAll = true;
                try
                {
                    lock (this.protocolLock)
                    {
                        try
                        {
                            this.Connect();
                            if(Controler.UpdateAll(this.protocol, this.Progress, false))
                            {
                                this.replys.UpdateStatus(Controler);
                                this.replys.LogData(DateTime.Now, Controler);
                            }                           
                        }
                        catch (ReefStatusException ex)
                        {
                            this.HandelException(ex);
                        }
                    }
                }
                finally
                {
                    inUpdateAll = false;
                }
            }).Start();
        }

        /// <summary>
        /// Logs the parameters.
        /// </summary>
        public void LogParameters()
        {
            new Thread(
                () =>
                    {
                        if (this.inLog)
                        {
                            return;
                        }

                        this.inLog = true;
                        try
                        {
                            lock (this.protocolLock)
                            {
                                bool processedCommand = false;
                                ReefStatusException lastErrorMessage = null;
                                int retryCount = 0;

                                // try the command 5 times until it fails
                                while (!processedCommand && retryCount < 5)
                                {
                                    retryCount++;
                                    try
                                    {
                                        this.Connect();

                                        if (Controler.UpdateValues(this.protocol, this.Progress))
                                        {
                                            this.replys.UpdateStatus(Controler);
                                            this.replys.LogData(DateTime.Now, Controler);
                                        }

                                        processedCommand = true;
                                    }
                                    catch (ReefStatusException ex)
                                    {
                                        lastErrorMessage = ex;
                                        try
                                        {
                                            if (this.protocol != null && this.protocol.IsConnected)
                                            {
                                                this.Disconnect();
                                                this.Connect();
                                            }
                                        }
                                        catch (ReefStatusException excpt)
                                        {
                                            lastErrorMessage = excpt;
                                            this.Disconnect();
                                        }

                                        Thread.Sleep(1000);
                                    }
                                }

                                if (!processedCommand)
                                {
                                    if (lastErrorMessage != null)
                                    {
                                        Logger.Instance.LogError(lastErrorMessage);
                                        if (!isConnectionError)
                                        {
                                            this.replys.OnErrorInConnection(lastErrorMessage);
                                            isConnectionError = true;
                                        }
                                    }
                                }
                                else
                                {
                                    isConnectionError = false;
                                }
                            }
                        }
                        finally
                        {
                            this.inLog = false;
                        }
                    }).Start();
        }

        /// <summary>
        /// Sends the key command.
        /// </summary>
        /// <param name="key">The key.</param>
        public void SendKeyCommand(FaceplateKey key)
        {
            new Thread(() =>
            {
                lock (this.protocolLock)
                {
                    try
                    {
                        this.Connect();
                        this.KeyCommand(key);
                    }
                    catch (ReefStatusException ex)
                    {
                        this.HandelException(ex);
                    }
                }
            }).Start();
        }

        /// <summary>
        /// Resets the reminder.
        /// </summary>
        /// <param name="reminder">The reminder.</param>
        public void SendResetReminder(Reminder reminder)
        {
            new Thread(() =>
            {
                lock (this.protocolLock)
                {
                    try
                    {
                        this.Connect();
                        Controler.ResetReminder(this.protocol, reminder);
                        Logger.Instance.Log(new LogMessage(0, Language.GetResource("strReminder") + " " + reminder.Text + " " + Language.GetResource("strReset")));
                    }
                    catch (ReefStatusException ex)
                    {
                        this.HandelException(ex);
                    }
                }
            }).Start();
        }

        /// <summary>
        /// Sends the state of the socket.
        /// </summary>
        /// <param name="port">The port.</param>
        /// <param name="value">if set to <c>true</c> [value].</param>
        public void SendSocketState(SPort port, bool value)
        {
            new Thread(() =>
            {
                lock (this.protocolLock)
                {
                    try
                    {
                        this.Connect();
                        Controler.SetSocketState(this.protocol, port, value);
                        Controler.UpdateSPortValue(this.protocol, port);
                    }
                    catch (ReefStatusException ex)
                    {
                        this.HandelException(ex);
                    }
                }
            }).Start();
        }

        private bool isSendingLightCommand;

        /// <summary>
        /// Sends the state of the light.
        /// </summary>
        /// <param name="light">The light.</param>
        /// <param name="value">The value.</param>
        public void SendLightState(Light light, double value)
        {
            new Thread(() =>
            {
                if (this.isSendingLightCommand)
                {
                    return;
                }

                this.isSendingLightCommand = true;
                try
                {
                    lock (this.protocolLock)
                    {
                        try
                        {
                            this.Connect();
                            Controler.SetLightState(this.protocol, light, value);
                            Controler.UpdateLightValue(this.protocol, light);
                        }
                        catch (ReefStatusException ex)
                        {
                            this.HandelException(ex);
                        }
                    }
                }
                finally 
                {
                    this.isSendingLightCommand = false;
                }
            }).Start();
        }

        /// <summary>
        /// Sends the start water change.
        /// </summary>
        /// <param name="levelSensor">The level sensor.</param>
        public void SendStartWaterChange(LevelSensor levelSensor)
        {
            new Thread(() =>
            {
                lock (this.protocolLock)
                {
                    try
                    {
                        this.Connect();
                        Controler.WaterChange(this.protocol, levelSensor);
                        Logger.Instance.Log(new LogMessage(0, Language.GetResource("strWaterChangefor") + levelSensor.DisplayName + Language.GetResource("strStarted")));
                    }
                    catch (ReefStatusException ex)
                    {
                        this.HandelException(ex);
                    }
                }
            }).Start();
        }

        /// <summary>
        /// Sends the update dosing rate.
        /// </summary>
        /// <param name="pump">The pump.</param>
        /// <param name="rate">The rate.</param>
        /// <param name="perDay"> the rate per day</param>
        public void SendUpdateDosingRate(DosingPump pump, int rate, int perDay)
        {
            new Thread(() =>
            {
                lock (this.protocolLock)
                {
                    try
                    {
                        this.Connect();
                        Controler.UpdateDosingRate(this.protocol, pump, rate, perDay);
                        Logger.Instance.Log(new LogMessage(0, Language.GetResource("strUpdated") + " " + pump.DisplayName + " " + Language.GetResource("strDosingRateTo") + (rate * perDay) + pump.Units));

                    }
                    catch (ReefStatusException ex)
                    {
                        this.HandelException(ex);
                    }
                }
            }).Start();
        }

        /// <summary>
        /// Sends the clear level alarm.
        /// </summary>
        /// <param name="levelSensor">The level sensor.</param>
        public void SendClearLevelAlarm(LevelSensor levelSensor)
        {
            new Thread(() =>
            {
                lock (this.protocolLock)
                {
                    try
                    {
                        this.Connect();
                        Controler.ClearLevelAlarm(this.protocol, levelSensor);
                        Logger.Instance.Log(new LogMessage(0, Language.GetResource("strLevelSensorAlarm") + levelSensor.DisplayName + Language.GetResource("strCleared")));


                    }
                    catch (ReefStatusException ex)
                    {
                        this.HandelException(ex);
                    }
                }
            }).Start();
        }

        /// <summary>
        /// Starts a Thunder Storm.
        /// </summary>
        /// <param name="duration">The duration.</param>
        public void ThunderStorm(int duration)
        {
            new Thread(() =>
            {
                lock (this.protocolLock)
                {
                    try
                    {
                        this.Connect();
                        this.protocol.Thunderstorm(duration);
                        Logger.Instance.Log(new LogMessage(0, Language.GetResource("strThunderstormStarted") + duration));
                    }
                    catch (ReefStatusException ex)
                    {
                        this.HandelException(ex);
                    }
                }
            }).Start();
        }

        /// <summary>
        /// Sends the operation mode.
        /// </summary>
        /// <param name="mode">The mode.</param>
        public void SendOperationMode(OperationMode mode)
        {
            new Thread(
                () =>
                    {
                        lock (this.protocolLock)
                        {
                            try
                            {
                                this.Connect();
                                OperationMode lastMode = this.protocol.OpMode;
                                if (lastMode != mode)
                                {
                                    this.protocol.OpMode = mode;
                                    Logger.Instance.Log(
                                        new LogMessage(
                                            0,
                                            Language.GetResource("strSetOperationMode") + Language.GetFrendyName(mode)));
                                    Controler.UpdateOperationMode(this.protocol.OpMode, null);
                                    if (lastMode == OperationMode.ManualSockets)
                                    {
                                        Controler.UpdateSPortsValues(this.protocol, null);
                                    }
                                    else if (lastMode == OperationMode.ManualIllumination)
                                    {
                                        Controler.UpdateLPortValues(this.protocol, null);
                                        Controler.UpdateLightsValues(this.protocol, null);
                                    }
                                }
                            }
                            catch (ReefStatusException ex)
                            {
                                this.HandelException(ex);
                            }
                        }
                    }).Start();
        }

        /// <summary>
        /// Sends the maintenance.
        /// </summary>
        public void SendMaintenance(bool enable)
        {
            new Thread(() =>
            {
                lock (this.protocolLock)
                {
                    try
                    {
                        this.Connect();
                        this.protocol.Maintenace(enable);
                        Logger.Instance.Log(new LogMessage(0, Language.GetResource("strMaintenanceMode") + (enable ? Language.GetResource("strEnabled") : Language.GetResource("strDisabled"))));
                        Controler.UpdateOperationMode(this.protocol.OpMode, null);
                    }
                    catch (ReefStatusException ex)
                    {
                        this.HandelException(ex);
                    }
                }
            }).Start();
        }

        /// <summary>
        /// Sends the feed.
        /// </summary>
        public void SendFeed(bool enable)
        {
            new Thread(() =>
            {
                lock (this.protocolLock)
                {
                    try
                    {
                        this.Connect();
                        this.protocol.FeedPause(enable);
                        Logger.Instance.Log(new LogMessage(0, Language.GetResource("strFeedPauseSet")));
                    }
                    catch (ReefStatusException ex)
                    {
                        this.HandelException(ex);
                    }
                }
            }).Start();
        }

        /// <summary>
        /// Stops this instance.
        /// </summary>
        public void Stop()
        {
            if (this.Progress != null)
            {
                this.Progress.StopProcessing = true;
            }

            this.Disconnect();
        }

        /// <summary>
        /// Gets the info.
        /// </summary>
        /// <param name="fullUpdate">if set to <c>true</c> [full update].</param>
        public void UpdateInfo(bool fullUpdate)
        {
            lock (this.protocolLock)
            {
                this.Connect();
                if (fullUpdate)
                {
                    Controler.UpdateInfo(this.protocol, null);
                }
                else
                {
                    Controler.UpdateInfoValues(this.protocol, null);
                }
            }
        }

        /// <summary>
        /// Gets the sensors.
        /// </summary>
        /// <param name="fullUpdate">if set to <c>true</c> [full update].</param>
        public void UpdateProbes(bool fullUpdate)
        {
            lock (this.protocolLock)
            {
                this.Connect();
                if (fullUpdate)
                {
                    Controler.UpdateProbes(this.protocol, null, false);
                }
                else
                {
                    Controler.UpdateProbeValues(this.protocol, null);
                }
            }
        }

        /// <summary>
        /// Gets the sockets.
        /// </summary>
        /// <param name="fullUpdate">if set to <c>true</c> [full update].</param>
        public void UpdateSPorts(bool fullUpdate)
        {
            lock (this.protocolLock)
            {
                this.Connect();
                if (fullUpdate)
                {
                    Controler.UpdateSPorts(this.protocol, null);
                }
                else
                {
                    Controler.UpdateSPortsValues(this.protocol, null);
                }
            }
        }

        /// <summary>
        /// Gets the sensors.
        /// </summary>
        /// <param name="fullUpdate">if set to <c>true</c> [full update].</param>
        public void UpdateLevelSensors(bool fullUpdate)
        {
            lock (this.protocolLock)
            {
                this.Connect();
                if (fullUpdate)
                {
                    Controler.UpdateLevelSensors(this.protocol, null);
                }
                else
                {
                    Controler.UpdateLevelSensorsValues(this.protocol, null);
                }
            }
        }

        /// <summary>
        /// Gets the sockets.
        /// </summary>
        /// <param name="fullUpdate">if set to <c>true</c> [full update].</param>
        public void UpdateLPorts(bool fullUpdate)
        {
            lock (this.protocolLock)
            {
                this.Connect();
                if (fullUpdate)
                {
                    Controler.UpdateLPorts(this.protocol, null);
                }
                else
                {
                    Controler.UpdateLPortValues(this.protocol, null);
                }
            }
        }

        /// <summary>
        /// Updates the digital input.
        /// </summary>
        /// <param name="fullUpdate">if set to <c>true</c> [full update].</param>
        public void UpdateDigitalInput(bool fullUpdate)
        {
            lock (this.protocolLock)
            {
                this.Connect();
                if (fullUpdate)
                {
                    Controler.UpdateDigitalInputs(this.protocol, null);
                }
                else
                {
                    Controler.UpdateDigitalInputValues(this.protocol, null);
                }
            }
        }

        /// <summary>
        /// Keys the command.
        /// </summary>
        /// <param name="facePlateKey">The face plate key.</param>
        public void KeyCommand(FaceplateKey facePlateKey)
        {
            lock (this.protocolLock)
            {
                this.Connect();
                this.protocol.KeyCommand(facePlateKey);
            }
        }

        /// <summary>
        /// Gets the display string.
        /// </summary>
        /// <returns>The display String</returns>
        public string GetDisplayString()
        {
            lock (this.protocolLock)
            {
                this.Connect();
                return this.protocol.DisplayText;
            }
        }

        /// <summary>
        /// Gets the display string.
        /// </summary>
        /// <returns>The display String</returns>
        public string[] GetDisplay()
        {
            lock (this.protocolLock)
            {
                this.Connect();
                return this.protocol.Display;
            }
        }

        /// <summary>
        /// Gets the view string.
        /// </summary>
        /// <returns>The View string</returns>
        public string GetViewString()
        {
            lock (this.protocolLock)
            {
                this.Connect();
                return this.protocol.ViewText;
            }
        }

        /// <summary>
        /// Gets the view text.
        /// </summary>
        /// <returns></returns>
        public string[] GetViewText()
        {
            lock (this.protocolLock)
            {
                this.Connect();
                return this.protocol.View;
            }
        }

        /// <summary>
        /// Gets the data points.
        /// </summary>
        /// <param name="callback">The callback.</param>
        /// <returns>List of Data Points</returns>
        public Collection<ItemDataRow> GetDataPoints(IProgressCallback callback)
        {
            lock (this.protocolLock)
            {
                this.Connect();
                return this.protocol.GetDataPoints(callback, Controler.ProbeCollection);
            }
        }

        /// <summary>
        /// Handles the Tick event of the connectionTimer control.
        /// </summary>
        /// <param name="state">The state.</param>
        private void ConnectionTimerTick(object state)
        {
            this.Disconnect();
        }

        /// <summary>
        /// The max time for a connection
        /// </summary>
        private const int ConnectonTimeout = 1000;

        /// <summary>
        /// Connects this instance.
        /// </summary>
        private void Connect()
        {
            lock (this.protocolLock)
            {
                if (this.protocol == null || !this.protocol.IsConnected)
                {
                    this.protocol = Protocol.Connect(Controler.Connection);
                }

                if (this.connectionTimer != null)
                {
                    this.connectionTimer.Change(ConnectonTimeout, System.Threading.Timeout.Infinite);
                }
                else
                {
                    this.connectionTimer = new System.Threading.Timer(this.ConnectionTimerTick, null, ConnectonTimeout, System.Threading.Timeout.Infinite);
                }
            }
        }

        /// <summary>
        /// Disconnects this instance.
        /// </summary>
        private void Disconnect()
        {
            lock (this.protocolLock)
            {
                if (this.protocol != null)
                {
                    try
                    {
                        this.protocol.Disconnect();
                    }
                    catch (ReefStatusException ex)
                    {
                        Logger.Instance.LogError(ex);
                    }
                    
                    this.protocol = null;
                }

                if (this.connectionTimer != null)
                {
                    this.connectionTimer.Dispose();
                    this.connectionTimer = null;
                }
            }
        }

        private bool isConnectionError;
        private bool inLog;

        public void SendResetOperationalHours(BaseInfo baseInfo)
        {
            new Thread(() =>
            {
                lock (this.protocolLock)
                {
                    try
                    {
                        this.Connect();
                        Controler.ResetOperationalHours(this.protocol, baseInfo);
                        Logger.Instance.Log(new LogMessage(0, string.Format(Language.GetResource("strResetOperationalHoursEvent"), baseInfo.DisplayName)));
                    }
                    catch (ReefStatusException ex)
                    {
                        this.HandelException(ex);
                    }
                }
            }).Start();
        }
    }
}