﻿// --------------------------------------------------------------------------------------------------------------------
// <copyright file="MemoryDataAccess.cs" company="Redpoint">
//      2010
// </copyright>
// --------------------------------------------------------------------------------------------------------------------

namespace RedPoint.ReefStatus.Common.Database
{
    using System;
    using System.Collections;
    using System.Collections.Generic;
    using System.Collections.ObjectModel;
    using System.Linq;

    using RedPoint.ReefStatus.Common.ProfiLux;
    using RedPoint.ReefStatus.Common.UI;

    /// <summary>
    /// The memory data access.
    /// </summary>
    internal class MemoryDataAccess : DataAccess, IDataAccess
    {
        #region Constants and Fields

        /// <summary>
        /// The data logs.
        /// </summary>
        private readonly Collection<DataLog> dataLogs = new Collection<DataLog>();

        /// <summary>
        /// The data types.
        /// </summary>
        private readonly Collection<DataType> dataTypes = new Collection<DataType>();

        #endregion

        #region Properties

        /// <summary>
        /// Gets the data logs.
        /// </summary>
        /// <value>The data logs.</value>
        private Collection<DataLog> DataLogs
        {
            get
            {
                return this.dataLogs;
            }
        }

        /// <summary>
        /// Gets the data types.
        /// </summary>
        /// <value>The data types.</value>
        private Collection<DataType> DataTypes
        {
            get
            {
                return this.dataTypes;
            }
        }

        #endregion

        #region Public Methods

        /// <summary>
        /// Gets the data points.
        /// </summary>
        /// <param name="type">
        /// The type.
        /// </param>
        /// <param name="count">
        /// The count.
        /// </param>
        /// <param name="descending">
        /// if set to <c>true</c> [descending].
        /// </param>
        /// <param name="controler">
        /// The controler.
        /// </param>
        /// <returns>
        /// A list of data points
        /// </returns>
        public IEnumerable<DataLog> GetDataPoints(int type, int count, bool descending, int controler)
        {
            try
            {
                IEnumerable<DataLog> logs;
                if (descending)
                {
                    logs = (from item in this.DataLogs
                            where item.Type == type && item.Controller == controler
                            orderby item.Time descending
                            select item).Take(count);
                }
                else
                {
                    logs = (from item in this.DataLogs
                            where item.Type == type && item.Controller == controler
                            orderby item.Time ascending
                            select item).Take(count);
                }

                return logs;
            }
            catch (Exception ex)
            {
                throw new DataAccessException(103, "Unable to Read Data Points", ex);
            }
        }

        /// <summary>
        /// Inserts the item.
        /// </summary>
        /// <param name="log">
        /// The log.
        /// </param>
        public void InsertItem(DataLog log)
        {
            try
            {
                this.WriteCount++;
                this.Insert(log);
            }
            catch (Exception ex)
            {
                throw new DataAccessException(111, "Unable To insert Item into database", ex);
            }
        }

        /// <summary>
        /// Inserts the items.
        /// </summary>
        /// <param name="log">
        /// The log.
        /// </param>
        public void InsertItems(Collection<DataLog> log)
        {
            try
            {
                this.WriteCount += log.Count;
                this.InserAll(log);
            }
            catch (Exception ex)
            {
                throw new DataAccessException(112, "Unable To insert Item into database", ex);
            }
        }

        #endregion

        #region Implemented Interfaces

        #region IDataAccess

        /// <summary>
        /// Backups the specified connection string.
        /// </summary>
        /// <param name="connectionString">
        /// The connection string.
        /// </param>
        /// <param name="archiveLocation">
        /// The archive location.
        /// </param>
        public void Backup(string connectionString, string archiveLocation)
        {
        }

        /// <summary>
        /// Cleanups the data set.
        /// </summary>
        /// <param name="type">
        /// The type.
        /// </param>
        /// <param name="controler">
        /// the controler
        /// </param>
        public void CleanupDataSet(string type, int controler)
        {
            try
            {
                int typeIndex = this.GetTypeIndex(type);
                List<DataLog> items = (from item in this.DataLogs
                                       where item.Type == typeIndex && item.Controller == controler
                                       orderby item.Time ascending
                                       select item).ToList();

                var toDelete = new Collection<DataLog>();

                for (int i = 2; i < items.Count; i++)
                {
                    DataLog log0 = items[i - 2];
                    DataLog log1 = items[i - 1];
                    DataLog log2 = items[i];

                    if (log1.Value == log2.Value && log0.Value == log2.Value)
                    {
                        toDelete.Add(log1);
                    }
                }

                this.DeleteCount += toDelete.Count;
                this.DeleteAll(toDelete);
            }
            catch (Exception ex)
            {
                throw new DataAccessException(109, "Unable to delete old logs from database", ex);
            }
        }

        /// <summary>
        /// The get data points.
        /// </summary>
        /// <param name="graph">The graph.</param>
        /// <param name="descending">The descending.</param>
        /// <param name="controler">The controler.</param>
        /// <returns> a list of data points</returns>
        /// <exception cref="DataAccessException">
        /// </exception>
        public Collection<DataPoint> GetDataPoints(string graph, bool descending, int controler)
        {
            try
            {
                var data = new Collection<DataPoint>();
                IEnumerable<DataLog> logs;
                int type = this.GetTypeIndex(graph);
                if (descending)
                {
                    logs = from item in this.DataLogs where item.Type == type orderby item.Time descending select item;
                }
                else
                {
                    logs = from item in this.DataLogs
                           where item.Type == type && item.Controller == controler
                           orderby item.Time ascending
                           select item;
                }

                foreach (DataLog log in logs)
                {
                    data.Add(new DataPoint(graph, log.Time, log.Value, log.Index));
                }

                return data;
            }
            catch (Exception ex)
            {
                throw new DataAccessException(101, "Unable to Read Data Points", ex);
            }
        }

        /// <summary>
        /// Gets the data points.
        /// </summary>
        /// <param name="graph">
        /// The graph.
        /// </param>
        /// <param name="count">
        /// The count.
        /// </param>
        /// <param name="descending">
        /// if set to <c>true</c> [descending].
        /// </param>
        /// <param name="controler">
        /// The controler.
        /// </param>
        /// <returns>
        /// list of data points
        /// </returns>
        public Collection<DataPoint> GetDataPoints(string graph, int count, bool descending, int controler)
        {
            try
            {
                var data = new Collection<DataPoint>();
                IEnumerable<DataLog> logs = this.GetDataPoints(this.GetTypeIndex(graph), count, descending, controler);

                foreach (DataLog log in logs)
                {
                    data.Add(new DataPoint(graph, log.Time, log.Value, log.Index));
                }

                return data;
            }
            catch (Exception ex)
            {
                throw new DataAccessException(102, "Unable to Read Data Points", ex);
            }
        }

        /// <summary>
        /// Gets the data points.
        /// </summary>
        /// <param name="graph">
        /// The graph.
        /// </param>
        /// <param name="startTime">
        /// The start time.
        /// </param>
        /// <param name="descending">
        /// if set to <c>true</c> [descending].
        /// </param>
        /// <param name="contorler">
        /// The contorler.
        /// </param>
        /// <returns>
        /// List of data points
        /// </returns>
        public Collection<DataPoint> GetDataPoints(string graph, DateTime startTime, bool descending, int contorler)
        {
            try
            {
                var data = new Collection<DataPoint>();
                IEnumerable<DataLog> logs;
                int type = this.GetTypeIndex(graph);
                if (descending)
                {
                    logs = from item in this.DataLogs
                           where item.Type == type && item.Time > startTime && item.Controller == contorler
                           orderby item.Time descending
                           select item;
                }
                else
                {
                    logs = from item in this.DataLogs
                           where item.Type == type && item.Time > startTime && item.Controller == contorler
                           orderby item.Time ascending
                           select item;
                }

                foreach (DataLog log in logs)
                {
                    data.Add(new DataPoint(graph, log.Time, log.Value, log.Index));
                }

                return data;
            }
            catch (Exception ex)
            {
                throw new DataAccessException(104, "Unable to Read Data Points", ex);
            }
        }

        /// <summary>
        /// Gets the data points.
        /// </summary>
        /// <param name="startTime">
        /// The start time.
        /// </param>
        /// <param name="endTime">
        /// The end time.
        /// </param>
        /// <param name="graph">
        /// The graph.
        /// </param>
        /// <param name="controler">
        /// The controler.
        /// </param>
        /// <returns>
        /// a list of data points for the range
        /// </returns>
        public Collection<DataPoint> GetDataPoints(DateTime startTime, DateTime endTime, string graph, int controler)
        {
            try
            {
                var data = new Collection<DataPoint>();
                int type = this.GetTypeIndex(graph);

                IEnumerable<DataLog> logs = from item in this.DataLogs
                                            where
                                                item.Type == type && item.Time > startTime && item.Time <= endTime &&
                                                item.Controller == controler
                                            orderby item.Time descending
                                            select item;

                foreach (DataLog log in logs)
                {
                    data.Add(new DataPoint(graph, log.Time, log.Value, log.Index));
                }

                return data;
            }
            catch (Exception ex)
            {
                throw new DataAccessException(105, "Unable to Read Data Points", ex);
            }
        }

        /// <summary>
        /// Gets the data points.
        /// </summary>
        /// <param name="typeIndex">Index of the type.</param>
        /// <param name="newTypeIndex">New index of the type.</param>
        /// <param name="callback">The callback.</param>
        /// <param name="controler">contorler index</param>
        /// <returns>The Data points</returns>
        public Collection<DataLog> GetDataPoints(
            int typeIndex, int newTypeIndex, IProgressCallback callback, int controler)
        {
            try
            {
                var data = new Collection<DataLog>();

                IEnumerable<DataLog> logs = from item in this.DataLogs
                                            where item.Type == typeIndex && item.Controller == controler
                                            select item;

                foreach (DataLog log in logs)
                {
                    if (callback.IsAborting)
                    {
                        return data;
                    }

                    var dataLog = new DataLog
                        {
                           Time = log.Time, Value = log.Value, Type = newTypeIndex, Controller = (short)controler 
                        };
                    data.Add(dataLog);
                }

                return data;
            }
            catch (Exception ex)
            {
                throw new DataAccessException(115, "Unable to Read Data Points", ex);
            }
        }

        /// <summary>
        /// Gets the data points.
        /// </summary>
        /// <param name="type">The type.</param>
        /// <param name="controler">controler Index</param>
        /// <returns>The Data points</returns>
        public IEnumerable GetDataPoints(string type, int controler)
        {
            try
            {
                int typeIndex = this.GetTypeIndex(type);
                return from item in this.DataLogs
                       where item.Type == typeIndex && item.Controller == controler
                       select new { item.Time, item.Value };
            }
            catch (Exception ex)
            {
                throw new DataAccessException(116, "Unable to Read Data Points", ex);
            }
        }

        /// <summary>
        /// Gets the stats.
        /// </summary>
        /// <param name="graph">The graph.</param>
        /// <param name="endTime">The end time.</param>
        /// <param name="startTime">The start time.</param>
        /// <param name="controler">The controler.</param>
        /// <returns>
        /// the stats for the graph
        /// </returns>
        public Stats GetStats(string graph, DateTime endTime, DateTime startTime, int controler, bool getStdDev)
        {
            return new Stats
                {
                    Max = this.MaxDataPoint(graph, endTime, startTime, controler), 
                    Min = this.MinDataPoint(graph, endTime, startTime, controler), 
                    Average = this.AverageDataPoint(graph, endTime, startTime, controler), 
                    StdDeviation = this.StdDevDataPoint(graph, endTime, startTime, controler)
                };
        }

        /// <summary>
        /// Gets the index of the type.
        /// </summary>
        /// <param name="type">
        /// The type.
        /// </param>
        /// <returns>
        /// the index of the type
        /// </returns>
        public int GetTypeIndex(string type)
        {
            try
            {
                IEnumerable<int> typeList = from types in this.DataTypes where types.Type == type select types.Index;

                if (typeList.Count() == 0)
                {
                    this.InsertType(type);
                    return (from types in this.DataTypes where types.Type == type select types.Index).Single();
                }

                return typeList.First();
            }
            catch (Exception ex)
            {
                throw new DataAccessException(106, "Unable get the type index", ex);
            }
        }

        /// <summary>
        /// Gets the types.
        /// </summary>
        /// <returns>
        /// a dictionary of types and there indexes
        /// </returns>
        public override Dictionary<int, string> GetTypes()
        {
            try
            {
                return this.DataTypes.ToDictionary(dataType => dataType.Index, dataType => dataType.Type);
            }
            catch (Exception ex)
            {
                throw new DataAccessException(114, "get the types from the database", ex);
            }
        }

        /// <summary>
        /// Inserts the item.
        /// </summary>
        /// <param name="value">
        /// The value.
        /// </param>
        /// <param name="time">
        /// The time.
        /// </param>
        /// <param name="type">
        /// The type.
        /// </param>
        /// <param name="optimize">
        /// if set to <c>true</c> [optimize].
        /// </param>
        /// <param name="controler">
        /// The controler.
        /// </param>
        public override void InsertItem(double value, DateTime time, string type, bool optimize, int controler)
        {
            try
            {
                int typeIndex = this.GetTypeIndex(type);

                var log = new DataLog { Time = time, Type = typeIndex, Value = value, Controller = (short)controler };

                if (optimize)
                {
                    List<DataLog> datapoints = this.GetDataPoints(typeIndex, 2, true, controler).ToList();

                    if (datapoints.Count != 2 || datapoints[0].Value != log.Value)
                    {
                        // it is a new value we must add it
                        this.InsertItem(log);
                    }
                    else
                    {
                        if (datapoints[1].Value != log.Value)
                        {
                            this.InsertItem(log);
                        }
                        else
                        {
                            datapoints[0].Time = log.Time;
                        }
                    }
                }
                else
                {
                    this.InsertItem(log);
                }
            }
            catch (Exception ex)
            {
                throw new DataAccessException(110, "Unable To insert Item into database", ex);
            }
        }

        /// <summary>
        /// Gets the number of data entries.
        /// </summary>
        /// <returns>
        /// the number of dial entreies
        /// </returns>
        public override int RecordCount()
        {
            try
            {
                return this.DataLogs.Count();
            }
            catch (Exception ex)
            {
                throw new DataAccessException(113, "Unable get the type index", ex);
            }
        }

        /// <summary>
        /// Removes the data set.
        /// </summary>
        /// <param name="type">
        /// The type.
        /// </param>
        /// <param name="controler">
        /// The controler.
        /// </param>
        public void RemoveDataSet(string type, int controler)
        {
            try
            {
                int typeIndex = this.GetTypeIndex(type);
                IEnumerable<DataLog> logs = from item in this.DataLogs
                                            where item.Type == typeIndex && item.Controller == controler
                                            select item;

                this.DeleteCount += logs.Count();
                this.DeleteAll(logs);
            }
            catch (Exception ex)
            {
                throw new DataAccessException(108, "Unable to delete old logs from database", ex);
            }
        }

        /// <summary>
        /// Removes the logs.
        /// </summary>
        /// <param name="timefrom">
        /// The timefrom.
        /// </param>
        public void RemoveLogs(DateTime timefrom)
        {
            try
            {
                IEnumerable<DataLog> logs = from item in this.DataLogs where item.Time < timefrom select item;

                this.DeleteAll(logs);
                this.DeleteCount++;
            }
            catch (Exception ex)
            {
                throw new DataAccessException(107, "Unable to delete old logs from database", ex);
            }
        }

        #endregion

        #region IDisposable

        /// <summary>
        /// Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources.
        /// </summary>
        public void Dispose()
        {
            this.OnDispose();
        }

        #endregion

        #endregion

        #region Methods

        /// <summary>
        /// Gets the data points.
        /// </summary>
        /// <param name="time">
        /// The time.
        /// </param>
        /// <param name="controler">
        /// The controler.
        /// </param>
        /// <returns>
        /// A list of points
        /// </returns>
        protected override Dictionary<int, double> GetDataPoints(DateTime time, int controler)
        {
            try
            {
                IEnumerable<DataLog> logs = from item in this.DataLogs
                                            where item.Time == time && item.Controller == controler
                                            select item;

                var data = new Dictionary<int, double>();
                foreach (DataLog item in logs)
                {
                    if (!data.ContainsKey(item.Type))
                    {
                        data.Add(item.Type, item.Value);
                    }
                }

                return data;
            }
            catch (Exception ex)
            {
                throw new DataAccessException(117, "Unable get a data point", ex);
            }
        }

        /// <summary>
        /// Gets the times.
        /// </summary>
        /// <returns>
        /// List of Times
        /// </returns>
        protected override IEnumerable<DateTime> GetTimes()
        {
            return (from item in this.DataLogs select item.Time).Distinct().OrderBy(n => n);
        }

        /// <summary>
        /// Called when [dispose].
        /// </summary>
        protected virtual void OnDispose()
        {
        }

        /// <summary>
        /// Averages the data point.
        /// </summary>
        /// <param name="graph">The graph.</param>
        /// <param name="endTime">The end time.</param>
        /// <param name="startTime">The start time.</param>
        /// <param name="controler">The controler.</param>
        /// <returns>
        /// avrage data point
        /// </returns>
        private double AverageDataPoint(string graph, DateTime endTime,  DateTime startTime, int controler)
        {
            try
            {
                int type = this.GetTypeIndex(graph);
                return (from item in this.DataLogs
                        where item.Type == type && item.Time <= endTime && item.Time > startTime && item.Controller == controler
                        select item.Value).Average();
            }
            catch (Exception ex)
            {
                throw new DataAccessException(119, "Unable to Read Data Points", ex);
            }
        }

        /// <summary>
        /// Deletes all.
        /// </summary>
        /// <param name="logs">
        /// The logs.
        /// </param>
        private void DeleteAll(IEnumerable<DataLog> logs)
        {
            var items = new Collection<DataLog>();
            foreach (DataLog dataLog in logs)
            {
                items.Add(dataLog);
            }

            this.RemoveAll(items);
        }

        private int lastIndex;

        /// <summary>
        /// Insers all.
        /// </summary>
        /// <param name="log">
        /// The log.
        /// </param>
        private void InserAll(IEnumerable<DataLog> log)
        {
            foreach (var dataLog in log)
            {
                this.Insert(dataLog);
            }
        }

        /// <summary>
        /// Inserts the specified log.
        /// </summary>
        /// <param name="log">
        /// The log.
        /// </param>
        private void Insert(DataLog log)
        {
            log.Index = this.lastIndex;
            this.lastIndex++;
            this.DataLogs.Add(log);
        }

        /// <summary>
        /// Inserts the specified data type.
        /// </summary>
        /// <param name="dataType">
        /// Type of the data.
        /// </param>
        private void Insert(DataType dataType)
        {
            this.DataTypes.Add(dataType);
        }

        /// <summary>
        /// Inserts the type.
        /// </summary>
        /// <param name="type">
        /// The type.
        /// </param>
        private void InsertType(string type)
        {
            try
            {
                this.Insert(new DataType { Type = type });
            }
            catch (Exception ex)
            {
                throw new DataAccessException(123, "Unable To insert type into database", ex);
            }
        }

        /// <summary>
        /// Maxes the data point.
        /// </summary>
        /// <param name="graph">The graph.</param>
        /// <param name="endTime">The end time.</param>
        /// <param name="startTime">The start time.</param>
        /// <param name="controler">The controler.</param>
        /// <returns>
        /// the largeset data point
        /// </returns>
        private double MaxDataPoint(string graph, DateTime endTime,  DateTime startTime, int controler)
        {
            try
            {
                int type = this.GetTypeIndex(graph);

                IEnumerable<double> items = from item in this.DataLogs
                                            where
                                                item.Type == type && item.Time <= endTime && item.Time > startTime &&
                                                item.Controller == controler
                                            select item.Value;
                return items.Max();
            }
            catch (Exception ex)
            {
                throw new DataAccessException(121, "Unable to Read Data Points", ex);
            }
        }

        /// <summary>
        /// Mins the data point.
        /// </summary>
        /// <param name="graph">The graph.</param>
        /// <param name="endTime">The end time.</param>
        /// <param name="startTime">The start time.</param>
        /// <param name="controler">The controler.</param>
        /// <returns>
        /// smallest data point
        /// </returns>
        private double MinDataPoint(string graph,DateTime endTime, DateTime startTime, int controler)
        {
            try
            {
                int type = this.GetTypeIndex(graph);
                return (from item in this.DataLogs
                        where item.Type == type && item.Time <= endTime && item.Time > startTime && item.Controller == controler
                        select item.Value).Min();
            }
            catch (Exception ex)
            {
                throw new DataAccessException(120, "Unable to Read Data Points", ex);
            }
        }

        public DateTime GetMinDate(string graph, int controler)
        {
            try
            {
                int type = this.GetTypeIndex(graph);
                return (from item in this.DataLogs
                        where item.Type == type && item.Controller == controler
                        select item.Time).Min();
            }
            catch (Exception ex)
            {
                throw new DataAccessException(120, "Unable to Read Data Points", ex);
            }
        }

        /// <summary>
        /// Removes all.
        /// </summary>
        /// <param name="items">
        /// The items.
        /// </param>
        private void RemoveAll(Collection<DataLog> items)
        {
            if (items.Count != 0)
            {
                this.DataLogs.Remove(items[0]);
                items.RemoveAt(0);
                this.RemoveAll(items);
            }
        }

        /// <summary>
        /// Sums the data point.
        /// </summary>
        /// <param name="graph">The graph.</param>
        /// <param name="endTime">The end time.</param>
        /// <param name="startTime">The start time.</param>
        /// <param name="controler">The controler.</param>
        /// <returns>
        /// the sum of the data points
        /// </returns>
        private double StdDevDataPoint(string graph, DateTime endTime, DateTime startTime, int controler)
        {
            try
            {
                int type = this.GetTypeIndex(graph);
                return CalculateStdDev(
                    from item in this.DataLogs
                    where item.Type == type && item.Time <= endTime && item.Time > startTime && item.Controller == controler
                    select item.Value);
            }
            catch (Exception ex)
            {
                throw new DataAccessException(118, "Unable to Read Data Points", ex);
            }
        }

        #endregion

        #region IDataAccess Members

        public void RemoveDataPoint(int index)
        {
            var data = this.DataLogs.FirstOrDefault(item => item.Index == index);
            this.DataLogs.Remove(data);
        }

        #endregion
    }
}