﻿// <copyright file="Dump.cs" company="Redpoint Apps">
// Copyright (c) Redpoint Apps. All rights reserved.
// </copyright>

namespace RedPoint.ReefStatus.Common
{
    using System;
    using System.Diagnostics;
    using System.IO;
    using System.Net;
    using System.Net.Mail;
    using System.Reflection;
    using System.Runtime.InteropServices;
    using System.Windows.Forms;

    using RedPoint.ReefStatus.Common.UI.ViewModel;

    using UI;

    /// <summary>
    /// Dump Helper Class
    /// </summary>
    public static class Dump
    {
        [Flags]
        public enum DumpType : uint
        {
            DumpNormal = 0x00000000,
            DumpWithDataSegs = 0x00000001,
            DumpWithFullMemory = 0x00000002,
            DumpWithHandleData = 0x00000004,
            DumpFilterMemory = 0x00000008,
            DumpScanMemory = 0x00000010,
            DumpWithUnloadedModules = 0x00000020,
            DumpWithIndirectlyReferencedMemory = 0x00000040,
            DumpFilterModulePaths = 0x00000080,
            DumpWithProcessThreadData = 0x00000100,
            DumpWithPrivateReadWriteMemory = 0x00000200,
            DumpWithoutOptionalData = 0x00000400,
            DumpWithFullMemoryInfo = 0x00000800,
            DumpWithThreadInfo = 0x00001000,
            DumpWithCodeSegs = 0x00002000,
            DumpWithoutAuxiliaryState = 0x00004000,
            DumpWithFullAuxiliaryState = 0x00008000,
            DumpWithPrivateWriteCopyMemory = 0x00010000,
            DumpIgnoreInaccessibleMemory = 0x00020000,
            DumpWithTokenInformation = 0x00040000,
            DumpValidTypeFlags = 0x0007ffff,
        }

        /// <summary>
        /// Handles this instance.
        /// </summary>
        public static void Handle()
        {
            AppDomain.CurrentDomain.UnhandledException += UnhandledException;
        }

        /// <summary>
        /// Writes the normal.
        /// </summary>
        /// <param name="fileName">Name of the file.</param>
        /// <returns>true if the write is succsfull</returns>
        private static bool WriteNormal(string fileName)
        {
            return Write(fileName, Dump.DumpType.DumpNormal);
        }

        /// <summary>
        /// Writes the large.
        /// </summary>
        /// <param name="fileName">Name of the file.</param>
        /// <returns>true if the write is succsfull</returns>
        private static bool WriteLarge(string fileName)
        {
            return Write(fileName, Dump.DumpType.DumpWithDataSegs | Dump.DumpType.DumpWithProcessThreadData);
        }

        /// <summary>
        /// Writes the full.
        /// </summary>
        /// <param name="fileName">Name of the file.</param>
        /// <returns>true if the write is succsfull</returns>
        private static bool WriteFull(string fileName)
        {
            return Write(fileName, DumpType.DumpWithFullMemory);
        }

        /// <summary>
        /// Writes the specified file name.
        /// </summary>
        /// <param name="fileName">Name of the file.</param>
        /// <param name="miniDumpType">Type of the mini dump.</param>
        /// <returns>true if the write is succsfull</returns>
        private static bool Write(string fileName, DumpType miniDumpType)
        {
            using (var stream = new FileStream(fileName, FileMode.Create, FileAccess.Write, FileShare.None))
            {
                MiniDumpExceptionInformation mei;
                mei.ThreadId = GetCurrentThreadId();
                mei.ClientPointers = false;
                mei.ExceptioonPointers = Marshal.GetExceptionPointers();

                if (stream.SafeFileHandle != null)
                {
                    bool ret = MiniDumpWriteDump(
                        GetCurrentProcess(), 
                        GetCurrentProcessId(), 
                        stream.SafeFileHandle.DangerousGetHandle(), 
                        (uint)miniDumpType, 
                        ref mei, 
                        IntPtr.Zero, 
                        IntPtr.Zero);

                    return ret;
                }

                return false;
            }
        }

        /// <summary>
        /// Unhandleds the exception.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.UnhandledExceptionEventArgs"/> instance containing the event data.</param>
        private static void UnhandledException(object sender, UnhandledExceptionEventArgs e)
        {
            var exception = e.ExceptionObject as Exception;
            if (exception != null)
            {
                if (new ErrorWindow
                    {
                        Discription =
                            "Sorry but and error was encounted in " + Application.ProductName +
                            " and will need to close.",
                        Details = exception.ToString()
                    }

                    .ShowDialog() == DialogResult.OK)
                {
                    try
                    {
                        var mailClient = new SmtpClient
                            {
                                Host = "plus.smtp.mail.yahoo.com",
                                Port = 25,
                                Credentials = new NetworkCredential("cjburchell", "redpoint")
                            };
   
                        var message = new MailMessage
                            {
                                From = new MailAddress("cjburchell@yahoo.com", "Reef Status"),
                                Subject =
                                    "Exception Report " + Application.ProductName + " " +
                                    Assembly.GetEntryAssembly().GetName().Version,
                                Body =
                                    string.Format(
                                        "Name: {0}\nE-mail: {1}\n OS: {2} {3}\n\nException:\n {4}",
                                        RegistrationViewModel.Instance.Name,
                                        RegistrationViewModel.Instance.Email,
                                        Environment.OSVersion,
                                        Environment.Is64BitOperatingSystem ? "64 Bit" : "32 Bit",
                                        exception)
                            };

                        message.To.Add("cjburchell@yahoo.com");
                        mailClient.Send(message);
                    }
                    catch (Exception ex)
                    {
                        Trace.WriteLine(ex);
                    }
                }
            }

            ////string timestamp = DateTime.Now.ToString("s");
            ////timestamp = timestamp.Replace(":", "-");
            ////string path = System.IO.Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData), "ReefStatus\\Dumps");

            ////if (!Directory.Exists(path))
            ////{
            ////    Directory.CreateDirectory(path);
            ////}

            ////string baseFileName = System.IO.Path.Combine(path, timestamp);

            ////if (Dump.WriteNormal(string.Format("{0}_normal.dmp", baseFileName)))
            ////{
            ////    Dump.WriteLarge(string.Format("{0}_large.dmp", baseFileName));
            ////    Dump.WriteFull(string.Format("{0}_full.dmp", baseFileName));
            ////}
        }

        [DllImport("dbghelp.dll", EntryPoint = "MiniDumpWriteDump", CallingConvention = CallingConvention.StdCall,
          CharSet = CharSet.Unicode, ExactSpelling = true, SetLastError = true)]
        private static extern bool MiniDumpWriteDump(
            IntPtr process, 
            uint processId, 
            IntPtr file, 
            uint dumpType,
            ref MiniDumpExceptionInformation expParam, 
            IntPtr userStreamParam, 
            IntPtr callbackParam);

        [DllImport("kernel32.dll", EntryPoint = "GetCurrentThreadId", ExactSpelling = true)]
        private static extern uint GetCurrentThreadId();

        [DllImport("kernel32.dll", EntryPoint = "GetCurrentProcess", ExactSpelling = true)]
        private static extern IntPtr GetCurrentProcess();

        [DllImport("kernel32.dll", EntryPoint = "GetCurrentProcessId", ExactSpelling = true)]
        private static extern uint GetCurrentProcessId();

        /// <summary>
        /// Mini Dump Exception Information
        /// </summary>
        [StructLayout(LayoutKind.Sequential, Pack = 4)]  // Pack = 4 for x64
        private struct MiniDumpExceptionInformation
        {
            /// <summary>
            /// Thread Id
            /// </summary>
            public uint ThreadId;

            /// <summary>
            /// Exception Pointers
            /// </summary>
            public IntPtr ExceptioonPointers;

            /// <summary>
            /// Client Pointers
            /// </summary>
            [MarshalAs(UnmanagedType.Bool)]
            public bool ClientPointers;
        }
    }
}
