﻿// <copyright file="BaseInfo.cs" company="Redpoint Apps">
// Copyright (c) Redpoint Apps. All rights reserved.
// </copyright>

namespace RedPoint.ReefStatus.Common.ProfiLux
{

#if (PocketPC != true)
    using System.Drawing;
#endif

#if !PocketPC && !Mono
    using System.Windows.Input;
#endif

#if (PocketPC != true)
    using RedPoint.ReefStatus.Common.Commands;
    using RedPoint.ReefStatus.Common.Settings;
#endif
    using RedPoint.ReefStatus.Common.UI.ViewModel;

    /// <summary>
    /// The range of the graph
    /// </summary>
    public enum GraphRange
    {
        /// <summary>
        /// Show all the graph
        /// </summary>
        All  = 0,

        /// <summary>
        /// Show datapoints from the last year
        /// </summary>
        Year = 4,

        /// <summary>
        /// Show datapoints from the last month
        /// </summary>
        Month = 3,

        /// <summary>
        /// Show datapoints from the last day
        /// </summary>
        Day = 1,

        /// <summary>
        /// Show datapoints from the last week
        /// </summary>
        Week = 2
    }

    /// <summary>
    /// Protocol item
    /// </summary>
    public abstract class BaseInfo : ViewModelBase
    {
#if (PocketPC != true && Mono != true)
        /// <summary>
        /// Default Units Command
        /// </summary>
        private ICommand defaultUnitsCommand;

        /// <summary>
        /// Default Name Command
        /// </summary>
        private ICommand defaultNameCommand;

#endif
        
        /// <summary>
        /// Colour Value
        /// </summary>
        private int colourValue;

        /// <summary>
        /// Gets or sets a value indicating whether this <see cref="Properties"/> is display.
        /// </summary>
        /// <value><c>true</c> if display; otherwise, <c>false</c>.</value>
        private bool display;

        /// <summary>
        /// Gets or sets the display name.
        /// </summary>
        /// <value>The display name.</value>
        private string displayName;

        /// <summary>
        /// Gets or sets the units.
        /// </summary>
        /// <value>The units.</value>
        private string units;

        /// <summary>
        /// Gets or sets the range.
        /// </summary>
        /// <value>The range.</value>
        private GraphRange range;

        /// <summary>
        /// Id of the object
        /// </summary>
        private string id;

        /// <summary>
        /// Default Units
        /// </summary>
        private string defaultUnits;

        /// <summary>
        /// the current value of the item
        /// </summary>
        private object value;

        /// <summary>
        /// info value
        /// </summary>
        private Info info;

        /// <summary>
        /// Initializes a new instance of the <see cref="BaseInfo"/> class.
        /// </summary>
        protected BaseInfo()
        {
            this.Range = GraphRange.Week;
#if (PocketPC != true)
            this.Colour = Color.Red;
#endif
            this.Display = false;
        }

        /// <summary>
        /// Gets or sets the info.
        /// </summary>
        /// <value>The info.</value>
        [System.Xml.Serialization.XmlIgnore]
        public Info Info
        {
            get
            {
                return this.info;
            }

            set
            {
                if (this.info != value)
                {
                    this.info = value;
                    OnPropertyChanged("Info");
                }
            }
        }

#if (PocketPC != true && Mono != true )
        /// <summary>
        /// Gets or sets the default name command.
        /// </summary>
        /// <value>The default name command.</value>
        [System.Xml.Serialization.XmlIgnore]
        public ICommand DefaultNameCommand
        {
            get
            {
                return this.defaultNameCommand ?? (this.defaultNameCommand = new DelegateCommand(this.SetDefaultName));
            }

            set
            {
                this.defaultNameCommand = value;
            }
        }

        /// <summary>
        /// Gets or sets the default units command.
        /// </summary>
        /// <value>The default units command.</value>
        [System.Xml.Serialization.XmlIgnore]
        public ICommand DefaultUnitsCommand
        {
            get
            {
                return this.defaultUnitsCommand ??
                       (this.defaultUnitsCommand = new DelegateCommand(this.SetDefaultUnits));
            }

            set
            {
                this.defaultUnitsCommand = value;
            }
        }
#endif

        /// <summary>
        /// Gets the commands.
        /// </summary>
        /// <value>The commands.</value>
        [System.Xml.Serialization.XmlIgnore]
        public CommandThread Commands
        {
            get
            {
                return Controler.Commands;
            }
        }

        /// <summary>
        /// Gets the controler.
        /// </summary>
        /// <value>The controler.</value>
        [System.Xml.Serialization.XmlIgnore]
        public Controler Controler
        {
            get
            {
                return ReefStatusSettings.Instance.GetControler(this);
            }
        }

        public virtual string GraphId
        {
            get
            {
                return this.Id;
            }
        }

        /// <summary>
        /// Gets or sets the name.
        /// </summary>
        /// <value>The name.</value>
        public string Id
        {
            get
            {
                return this.id;
            }

            set
            {
                if (this.id != value)
                {
                    this.id = value;
                    if (string.IsNullOrEmpty(this.DisplayName))
                    {
                        this.DisplayName = value;
                    }

                    OnPropertyChanged("Id");
                    OnPropertyChanged("GraphId");
                }
            }
        }

        /// <summary>
        /// Gets or sets the units.
        /// </summary>
        /// <value>The units.</value>
        public string DefaultUnits
        {
            get
            {
                return this.defaultUnits;
            }

            set
            {
                if (this.defaultUnits != value)
                {
                    this.defaultUnits = value;
                    if (string.IsNullOrEmpty(this.Units))
                    {
                        this.Units = value;
                    }

                    OnPropertyChanged("DefaultUnits");
                }
            }
        }

        /// <summary>
        /// Gets or sets the colour.
        /// </summary>
        /// <value>The colour.</value>
        public int ColourValue
        {
            get
            {
                return this.colourValue;
            }

            set
            {
                if (this.colourValue != value)
                {
                    this.colourValue = value;
                    OnPropertyChanged("ColourValue");
                    OnPropertyChanged("Colour");
                }
            }
        }

#if (PocketPC != true)
        /// <summary>
        /// Gets or sets the colour.
        /// </summary>
        /// <value>The colour.</value>
        [System.Xml.Serialization.XmlIgnore]
        public Color Colour
        {
            get
            {
                return Color.FromArgb(this.ColourValue);
            }

            set
            {
                this.ColourValue = value.ToArgb();
            }
        }
#endif

        /// <summary>
        /// Gets or sets a value indicating whether this <see cref="BaseInfo"/> is display.
        /// </summary>
        /// <value><c>true</c> if display; otherwise, <c>false</c>.</value>
        public bool Display
        {
            get
            {
                return this.display;
            }

            set
            {
                if (value != this.display)
                {
                    this.display = value;
                    OnPropertyChanged("Display");
                }
            }
        }

        /// <summary>
        /// Gets or sets the display name.
        /// </summary>
        /// <value>The display name.</value>
        public string DisplayName
        {
            get
            {
                return this.displayName;
            }

            set
            {
                if (this.displayName != value)
                {
                    this.displayName = value;
                    OnPropertyChanged("DisplayName");
                }
            }
        }

        /// <summary>
        /// Gets or sets the units.
        /// </summary>
        /// <value>The units.</value>
        public string Units
        {
            get
            {
                return this.units;
            }

            set
            {
                if (value != this.units)
                {
                    this.units = value;
                    OnPropertyChanged("Units");
                    OnPropertyChanged("Mode");
                    OnPropertyChanged("ValueWithUnits");
                }
            }
        }

        /// <summary>
        /// Gets or sets the range.
        /// </summary>
        /// <value>The range.</value>
        public GraphRange Range
        {
            get
            {
                return this.range;
            }

            set
            {
                if (this.range != value)
                {
                    this.range = value;
                    OnPropertyChanged("Range");
                }
            }
        }

        public abstract object ConvertedValue
        { get; }

        /// <summary>
        /// Gets the value with units.
        /// </summary>
        /// <value>The value with units.</value>
        public virtual string ValueWithUnits
        {
            get { return this.ConvertedValue + this.Units; }
        }

        /// <summary>
        /// Gets the mode.
        /// </summary>
        /// <value>The mode.</value>
        public virtual string Mode
        {
            get
            {
                return this.Units;
            }
        }

        /// <summary>
        /// Gets the double value.
        /// </summary>
        /// <value>The double value.</value>
        public abstract double DoubleValue { get; }

        /// <summary>
        /// Gets or sets the value.
        /// </summary>
        /// <value>The value.</value>
        public object Value
        {
            get
            {
                return this.value;
            }

            set
            {
                this.value = value;
                OnPropertyChanged("Value");
                OnPropertyChanged("DoubleValue");
                OnPropertyChanged("ValueWithUnits");
                OnPropertyChanged("IsActive");
                OnPropertyChanged("LightValue");
                OnPropertyChanged("IsLightOn");
                OnPropertyChanged("ConvertedValue");
            }
        }

        /// <summary>
        /// Gets a value indicating whether this instance is constant.
        /// </summary>
        /// <value>
        ///     <c>true</c> if this instance is constant; otherwise, <c>false</c>.
        /// </value>
        public virtual bool IsConstant
        {
            get { return false; }
        }

        /// <summary>
        /// Returns a <see cref="T:System.String"/> that represents the current <see cref="T:System.Object"/>.
        /// </summary>
        /// <returns>
        /// A <see cref="T:System.String"/> that represents the current <see cref="T:System.Object"/>.
        /// </returns>
        public override string ToString()
        {
            return this.DisplayName;
        }

        /// <summary>
        /// Defaults the units.
        /// </summary>
        private void SetDefaultUnits()
        {
            this.Units = this.DefaultUnits;
        }

        /// <summary>
        /// Defaults the name.
        /// </summary>
        private void SetDefaultName()
        {
            this.DisplayName = this.Id;
        }
    }
}
