﻿// --------------------------------------------------------------------------------------------------------------------
// <copyright file="LoggingSettings.cs" company="Redpoint Apps">
//   2010
// </copyright>
// --------------------------------------------------------------------------------------------------------------------

namespace RedPoint.ReefStatus.Common.Settings
{
    using System;
    using System.Collections.Generic;
    using System.Diagnostics;
    using System.Windows.Input;
    using System.Xml.Serialization;

    using RedPoint.ReefStatus.Common.Database;
    using RedPoint.ReefStatus.Common.ProfiLux;
    using RedPoint.ReefStatus.Common.UI.ViewModel;

#if !Mono
#endif

    /// <summary>
    /// The logging settings.
    /// </summary>
    public class LoggingSettings : ViewModelBase
    {
        /// <summary>
        /// The archive database.
        /// </summary>
        private bool archiveDatabase;

        /// <summary>
        /// The archive database duration.
        /// </summary>
        private int archiveDatabaseDuration;

        /// <summary>
        /// The archive database mode.
        /// </summary>
        private DateRangeMode archiveDatabaseMode;

        /// <summary>
        /// The database connection.
        /// </summary>
        private string databaseConnection;

        /// <summary>
        /// The database type.
        /// </summary>
        private DatabaseType databaseType;

        /// <summary>
        /// The last archive time.
        /// </summary>
        private DateTime lastArchiveTime;

        /// <summary>
        /// The limit database.
        /// </summary>
        private bool limitDatabase;

        /// <summary>
        /// The limit database duration.
        /// </summary>
        private int limitDatabaseDuration;

        /// <summary>
        /// The limit database mode.
        /// </summary>
        private DateRangeMode limitDatabaseMode;

        /// <summary>
        /// The log interval.
        /// </summary>
        private int logInterval;

#if !Mono

        /// <summary>
        /// The clean up command.
        /// </summary>
        private ICommand cleanUpCommand;
#endif

        /// <summary>
        /// Initializes a new instance of the <see cref="LoggingSettings"/> class.
        /// </summary>
        public LoggingSettings()
        {
            this.LogInterval = 5;
            this.DatabaseConnection = ReefStatusSettings.AppDataDir + "\\ReefStatus\\ReefStatus.sqlite";
            this.DatabaseType = DatabaseType.Sqlite;
            this.ArchiveDatabase = false;
            this.ArchiveDatabaseDuration = 1;
            this.ArchiveDatabaseMode = DateRangeMode.Days;

            this.LimitDatabase = false;
            this.LimitDatabaseDuration = 1;
            this.LimitDatabaseMode = DateRangeMode.Days;

            this.LastArchiveTime = DateTime.Now;
        }

        /// <summary>
        /// Gets or sets the last archive time.
        /// </summary>
        /// <value>The last archive time.</value>
        public DateTime LastArchiveTime
        {
            get
            {
                return this.lastArchiveTime;
            }

            set
            {
                if (this.lastArchiveTime != value)
                {
                    this.lastArchiveTime = value;
                    this.OnPropertyChanged("LastArchiveTime");
                }
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether [archive database].
        /// </summary>
        /// <value><c>true</c> if [archive database]; otherwise, <c>false</c>.</value>
        public bool ArchiveDatabase
        {
            get
            {
                return this.archiveDatabase;
            }

            set
            {
                if (this.archiveDatabase != value)
                {
                    this.archiveDatabase = value;
                    this.OnPropertyChanged("ArchiveDatabase");
                }
            }
        }

        /// <summary>
        /// Gets or sets the duration of the archive database.
        /// </summary>
        /// <value>The duration of the archive database.</value>
        public int ArchiveDatabaseDuration
        {
            get
            {
                return this.archiveDatabaseDuration;
            }

            set
            {
                if (this.archiveDatabaseDuration != value)
                {
                    this.archiveDatabaseDuration = value;
                    this.OnPropertyChanged("ArchiveDatabaseDuration");
                }
            }
        }

        /// <summary>
        /// Gets or sets the archive database mode.
        /// </summary>
        /// <value>The archive database mode.</value>
        public DateRangeMode ArchiveDatabaseMode
        {
            get
            {
                return this.archiveDatabaseMode;
            }

            set
            {
                if (this.archiveDatabaseMode != value)
                {
                    this.archiveDatabaseMode = value;
                    this.OnPropertyChanged("ArchiveDatabaseMode");
                }
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether [limit database].
        /// </summary>
        /// <value><c>true</c> if [limit database]; otherwise, <c>false</c>.</value>
        public bool LimitDatabase
        {
            get
            {
                return this.limitDatabase;
            }

            set
            {
                if (this.limitDatabase != value)
                {
                    this.limitDatabase = value;
                    this.OnPropertyChanged("LimitDatabase");
                }
            }
        }

        /// <summary>
        /// Gets or sets the duration of the limit database.
        /// </summary>
        /// <value>The duration of the limit database.</value>
        public int LimitDatabaseDuration
        {
            get
            {
                return this.limitDatabaseDuration;
            }

            set
            {
                if (this.limitDatabaseDuration != value)
                {
                    this.limitDatabaseDuration = value;
                    this.OnPropertyChanged("LimitDatabaseDuration");
                }
            }
        }

        /// <summary>
        /// Gets or sets the limit database mode.
        /// </summary>
        /// <value>The limit database mode.</value>
        public DateRangeMode LimitDatabaseMode
        {
            get
            {
                return this.limitDatabaseMode;
            }

            set
            {
                if (this.limitDatabaseMode != value)
                {
                    this.limitDatabaseMode = value;
                    this.OnPropertyChanged("LimitDatabaseMode");
                }
            }
        }

        /// <summary>
        /// Gets or sets the log interval.
        /// </summary>
        /// <value>The log interval.</value>
        public int LogInterval
        {
            get
            {
                return this.logInterval;
            }

            set
            {
                if (this.logInterval != value)
                {
                    this.logInterval = value;
                    this.OnPropertyChanged("LogInterval");
                }
            }
        }

#if !Mono
        /// <summary>
        /// Gets the clean up command.
        /// </summary>
        /// <value>The clean up command.</value>
        [XmlIgnore]
        public ICommand CleanUpCommand
        {
            get
            {
                return this.cleanUpCommand ?? (this.cleanUpCommand = new DelegateCommand(this.CleanUp));
            }
        }
#endif

        /// <summary>
        /// Gets or sets the database connection.
        /// </summary>
        /// <value>The database connection.</value>
        public string DatabaseConnection
        {
            get
            {
                return this.databaseConnection;
            }

            set
            {
                if (this.databaseConnection != value)
                {
                    this.databaseConnection = value;
                    this.OnPropertyChanged("DatabaseConnection");
                }
            }
        }

        /// <summary>
        /// Gets or sets the type of the database.
        /// </summary>
        /// <value>The type of the database.</value>
        public DatabaseType DatabaseType
        {
            get
            {
                return this.databaseType;
            }

            set
            {
                if (this.databaseType != value)
                {
                    this.databaseType = value;
                    this.OnPropertyChanged("DatabaseType");
                }
            }
        }

        /// <summary>
        /// Gets the database settings.
        /// </summary>
        /// <returns></returns>
        [XmlIgnore]
        public DatabaseConnection Connection
        {
            get
            {
                return new DatabaseConnection(this.DatabaseConnection, this.DatabaseType);
            }
        }

        /// <summary>
        /// Gets the record count.
        /// </summary>
        /// <value>The record count.</value>
        [XmlIgnore]
        public int RecordCount
        {
            get
            {
                using (IDataAccess database = this.Connection.Create())
                {
                    if (database != null)
                    {
                        return database.RecordCount();
                    }

                    return 0;
                }
            }
        }

        /// <summary>
        /// Gets the records write.
        /// </summary>
        /// <value>The records write.</value>
        [XmlIgnore]
        public int RecordsWrite
        {
            get
            {
                using (IDataAccess database = this.Connection.Create())
                {
                    if (database != null)
                    {
                        return database.WriteCount;
                    }

                    return 0;
                }
            }
        }

        /// <summary>
        /// Gets the records delete.
        /// </summary>
        /// <value>The records delete.</value>
        [XmlIgnore]
        public int RecordsDelete
        {
            get
            {
                using (IDataAccess database = this.Connection.Create())
                {
                    if (database != null)
                    {
                        return database.DeleteCount;
                    }

                    return 0;
                }
            }
        }

        /// <summary>
        /// Cleans up the database.
        /// </summary>
        public void CleanUp()
        {
            try
            {
                using (IDataAccess database = this.Connection.Create())
                {
                    if (database != null)
                    {
                        Dictionary<int, string> types = database.GetTypes();
                        foreach (string type in types.Values)
                        {
                            foreach (Controler controler in ReefStatusSettings.Instance.Controlers)
                            {
                                database.CleanupDataSet(type, controler.Id);
                            }
                        }
                    }
                }
            }
            catch (ReefStatusException ex)
            {
                Trace.WriteLine(ex);
            }

            this.OnPropertyChanged("RecordCount");
            this.OnPropertyChanged("RecordsWrite");
            this.OnPropertyChanged("RecordsDelete");
        }
    }
}