﻿// <copyright file="MailSettings.cs" company="Redpoint Apps">
// Copyright (c) Redpoint Apps. All rights reserved.
// </copyright>
using RedPoint.ReefStatus.Common.UI.ViewModel;

namespace RedPoint.ReefStatus.Common.Settings
{
    using System;
    using System.Collections.ObjectModel;
    using System.Diagnostics;
    using System.Net;
    using System.Net.Mail;
#if !Mono
    using System.Windows.Input;
#endif

    /// <summary>
    /// Mail Settings
    /// </summary>
    public class MailSettings : ViewModelBase
    {
        private bool sendShortMessage;
        private bool sendOnAlarm;
        private bool sendStatus;
        private int sendStatusDuration;
        private DateTime lastStatusTime;
        private DateRangeMode sendStatusMode;
        private string server;
        private int port;
        private string to;
        private string from;
        private bool usePassword;
        private string userName;
        private string password;

        /// <summary>
        /// Initializes a new instance of the <see cref="MailSettings"/> class.
        /// </summary>
        public MailSettings()
        {
            SendOnAlarm = false;
            Port = 25;
            UsePassword = false;
            SendStatus = false;
            SendStatusDuration = 1;
            SendStatusMode = DateRangeMode.Days;
            LastStatusTime = DateTime.Now;
            SendShortMessage = false;
        }

        /// <summary>
        /// Gets or sets a value indicating whether [send short message].
        /// </summary>
        /// <value><c>true</c> if [send short message]; otherwise, <c>false</c>.</value>
        public bool SendShortMessage
        {
            get { return sendShortMessage; }
            set
            {
                sendShortMessage = value; 
                OnPropertyChanged("SendShortMessage");
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether [send on alarm].
        /// </summary>
        /// <value><c>true</c> if [send on alarm]; otherwise, <c>false</c>.</value>
        public bool SendOnAlarm
        {
            get { return sendOnAlarm; }
            set 
            { 
                sendOnAlarm = value;
                OnPropertyChanged("SendOnAlarm"); 
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether [send status].
        /// </summary>
        /// <value><c>true</c> if [send status]; otherwise, <c>false</c>.</value>
        public bool SendStatus
        {
            get { return sendStatus; }
            set
            {
                sendStatus = value;
                OnPropertyChanged("SendStatus"); 
            }
        }

        /// <summary>
        /// Gets or sets the duration of the send status.
        /// </summary>
        /// <value>The duration of the send status.</value>
        public int SendStatusDuration
        {
            get { return sendStatusDuration; }
            set
            {
                sendStatusDuration = value; 
                OnPropertyChanged("SendStatusDuration");
            }
        }

        /// <summary>
        /// Gets or sets the last status time.
        /// </summary>
        /// <value>The last status time.</value>
        public DateTime LastStatusTime
        {
            get { return lastStatusTime; }
            set
            {
                lastStatusTime = value;
                OnPropertyChanged("LastStatusTime");
            }
        }

        /// <summary>
        /// Gets or sets the send status mode.
        /// </summary>
        /// <value>The send status mode.</value>
        public DateRangeMode SendStatusMode
        {
            get { return sendStatusMode; }
            set 
            { 
                sendStatusMode = value;
                OnPropertyChanged("SendStatusMode");
            }
        }

        /// <summary>
        /// Gets or sets the server.
        /// </summary>
        /// <value>The server.</value>
        public string Server
        {
            get { return server; }
            set 
            { 
                server = value;
                OnPropertyChanged("Server");
            }
        }

        /// <summary>
        /// Gets or sets the port.
        /// </summary>
        /// <value>The port.</value>
        public int Port
        {
            get { return port; }
            set
            {
                port = value;
                OnPropertyChanged("Port");
            }
        }

        /// <summary>
        /// Gets or sets the To time.
        /// </summary>
        /// <value>To time.</value>
        public string To
        {
            get { return to; }
            set
            {
                to = value;
                OnPropertyChanged("To");
            }
        }

        /// <summary>
        /// Gets or sets from time.
        /// </summary>
        /// <value>From time.</value>
        public string From
        {
            get { return from; }
            set
            {
                from = value;
                OnPropertyChanged("From");
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether [use password].
        /// </summary>
        /// <value><c>true</c> if [use password]; otherwise, <c>false</c>.</value>
        public bool UsePassword
        {
            get { return usePassword; }
            set
            {
                usePassword = value;
                OnPropertyChanged("UsePassword");
            }
        }

        /// <summary>
        /// Gets or sets the name of the user.
        /// </summary>
        /// <value>The name of the user.</value>
        public string UserName
        {
            get { return userName; }
            set
            {
                userName = value;
                OnPropertyChanged("UserName");
            }
        }

        /// <summary>
        /// Gets or sets the password.
        /// </summary>
        /// <value>The password.</value>
        public string Password
        {
            get { return password; }
            set
            {
                password = value;
                OnPropertyChanged("Password");
            }
        }

#if !Mono
        private ICommand testEmailCommand;
        [System.Xml.Serialization.XmlIgnore]
        public ICommand TestEmailCommand
        {
            get { return this.testEmailCommand ?? (this.testEmailCommand = new DelegateCommand(this.TestEmail)); }
        }
#endif

        /// <summary>
        /// Tests this instance.
        /// </summary>
        public void TestEmail()
        {
            try
            {
                const string TestEmail = "This is a test of the Reef status e-mail";
                this.SendMailMessage("Test Reef Status E-mail", TestEmail, TestEmail, null);
            }
            catch (ReefStatusException ex)
            {
                Trace.WriteLine(ex);
            }
        }

        /// <summary>
        /// Sends the mail message.
        /// </summary>
        /// <param name="subject">The subject.</param>
        /// <param name="htmlBody">The HTML body.</param>
        /// <param name="textBody">The text body.</param>
        /// <param name="attachments">The attachments.</param>
        public void SendMailMessage(string subject, string htmlBody, string textBody, Collection<Attachment> attachments)
        {
            try
            {
                SmtpClient mailClient = new SmtpClient {Host = this.Server, Port = this.Port};

                if (this.UsePassword)
                {
                    mailClient.Credentials = new NetworkCredential(this.UserName, this.Password);
                }

                MailMessage message = new MailMessage
                                          {
                                              From = new MailAddress(this.From, Language.GetResource("strReefStatus"))
                                          };

                foreach (string address in this.To.Split(new[] { ',', ';' }))
                {
                    message.To.Add(address.Trim());
                }

                message.Subject = subject;

                if (SendShortMessage)
                {
                    message.Body = textBody;
                }
                else
                {
                    AlternateView plainView = AlternateView.CreateAlternateViewFromString(textBody, null, System.Net.Mime.MediaTypeNames.Text.Plain);
                    AlternateView htmlView = AlternateView.CreateAlternateViewFromString(htmlBody, null, System.Net.Mime.MediaTypeNames.Text.Html);

                    if (attachments != null)
                    {
                        foreach (Attachment item in attachments)
                        {
                            message.Attachments.Add(item);
                        }
                    }

                    message.AlternateViews.Add(plainView);
                    message.AlternateViews.Add(htmlView);
                }

                mailClient.Send(message);
            }
            catch (ArgumentException ex)
            {
                throw new ReefStatusException(5000, "Error Sending E-mail", ex);
            }
            catch (FormatException ex)
            {
                throw new ReefStatusException(5001, "Error Sending E-mail", ex);
            }
            catch (InvalidOperationException ex)
            {
                throw new ReefStatusException(5002, "Error Sending E-mail", ex);
            }
            catch (SmtpException ex)
            {
                throw new ReefStatusException(5003, "Error Sending E-mail", ex);
            }
        }
    }
}
