﻿// --------------------------------------------------------------------------------------------------------------------
// <copyright file="MainWindowViewModel.cs" company="Redpoint Apps ">
//   2010
// </copyright>
// <summary>
//   The main window view model.
// </summary>
// --------------------------------------------------------------------------------------------------------------------

namespace RedPoint.ReefStatus.Gui.ViewModels
{
    using System;
    using System.Collections.ObjectModel;
    using System.Diagnostics;
    using System.IO;
    using System.Linq;
    using System.Windows.Input;
    using System.Windows.Threading;
    using Common.Core;
    using Microsoft.Win32;
    using RedPoint.ReefStatus.Common;
    using RedPoint.ReefStatus.Common.ProfiLux;
    using RedPoint.ReefStatus.Common.Settings;
    using RedPoint.ReefStatus.Common.UI;
    using RedPoint.ReefStatus.Common.UI.ViewModel;

    using Timer = System.Windows.Forms.Timer;

    /// <summary>
    /// The main window view model.
    /// </summary>
    public class MainWindowViewModel : ViewModelBase, IUpdateProgress
    {
        /// <summary>
        /// The clear last error timer.
        /// </summary>
        private readonly Timer clearLastErrorTimer = new Timer();

        /// <summary>
        /// The dispatcher.
        /// </summary>
        private readonly Dispatcher dispatcher;

        /// <summary>
        /// The current progress.
        /// </summary>
        private double currentProgress;

        /// <summary>
        /// The display progress.
        /// </summary>
        private bool displayProgress;

        /// <summary>
        /// The progress steps.
        /// </summary>
        private double progressSteps;

        /// <summary>
        /// The selected document.
        /// </summary>
        private ViewModelBase selectedDocument;

        /// <summary>
        /// The update progress.
        /// </summary>
        private double updateProgress;

        /// <summary>
        /// Initializes static members of the <see cref="MainWindowViewModel"/> class. 
        /// </summary>
        static MainWindowViewModel()
        {
            Instance = new MainWindowViewModel();
        }

        /// <summary>
        /// Prevents a default instance of the <see cref="MainWindowViewModel"/> class from being created. 
        /// Initializes a new instance of the <see cref="MainWindowViewModel"/> class.
        /// </summary>
        private MainWindowViewModel()
        {
            this.Progress = new ProgressViewModel();

            this.dispatcher = Dispatcher.CurrentDispatcher;

            Documents = new ObservableCollection<ViewModelBase>();
            Logger.Instance.OnError += this.LoggerOnError;
            DataService.Instance.Start();

            WindowClosingCommand = new DelegateCommand(WindowClosing);
            ImportFromFileCommand = new DelegateCommand(ImportFromFile);
            ImportFromProfiluxCommand = new DelegateCommand(ImportFromProfilux);
            ExportDataMenuItemCommand = new DelegateCommand(ExportData);
            RefreshCommand = new DelegateCommand(UpdateAll, ()=> HasControler);
            LogNowCommand = new DelegateCommand(LogParameters, () => HasControler);
            ShowProfiluxControlCommand = new DelegateCommand(ShowProfiluxControl, ()=> HasProfilux);

            ShowRemoteCommand = new DelegateCommand(ShowRemote);

            ShowCalendarCommand = new DelegateCommand(ShowCalendar);

            DataService.Instance.OnLogItem += AddToGraphDisplay;

            this.UpdateAll();

            this.clearLastErrorTimer.Tick += this.ClearLastErrorTimerTick;
            this.clearLastErrorTimer.Interval = 20000;

            ErrorLogViewModel.Starup();
        }

        protected bool HasControler
        {
            get
            {
                return this.Settings.Controlers.Count != 0;
            }
        }

        /// <summary>
        /// Gets or sets the show remote command.
        /// </summary>
        /// <value>The show remote command.</value>
        public ICommand ShowRemoteCommand { get; set; }

        /// <summary>
        /// Gets or sets the calendar command.
        /// </summary>
        /// <value>
        /// The calendar command.
        /// </value>
        public ICommand ShowCalendarCommand { get; set; }

        /// <summary>
        /// Shows the remote.
        /// </summary>
        private void ShowRemote()
        {
            var remote = new RemoteViewModel();
            this.Documents.Add(remote);
            this.SelectedDocument = remote;
        }

        /// <summary>
        /// Shows the calendar.
        /// </summary>
        private void ShowCalendar()
        {
            var remote = new CalendarViewModel();
            this.Documents.Add(remote);
            this.SelectedDocument = remote;
        }

        /// <summary>
        /// Logs the parameters.
        /// </summary>
        private void LogParameters()
        {
            foreach (Controler controler in ReefStatusSettings.Instance.Controlers)
            {
                controler.Commands.Progress = this;
                controler.Commands.LogParameters();
            }
        }

        /// <summary>
        /// Updates all.
        /// </summary>
        private void UpdateAll()
        {
            foreach (Controler controler in ReefStatusSettings.Instance.Controlers)
            {
                controler.Commands.Progress = this;
                controler.Commands.UpdateAll();
            }
        }

        /// <summary>
        /// Gets Progress.
        /// </summary>
        /// <value>The progress.</value>
        public ProgressViewModel Progress { get; private set; }

        /// <summary>
        /// Gets or sets the instance.
        /// </summary>
        /// <value>The instance.</value>
        public static MainWindowViewModel Instance { get; private set; }

        /// <summary>
        /// Gets the settings.
        /// </summary>
        /// <value>The settings.</value>
        public ReefStatusSettings Settings
        {
            get
            {
                return ReefStatusSettings.Instance;
            }
        }

        /// <summary>
        /// Gets the window closing command.
        /// </summary>
        /// <value>The window closing command.</value>
        public ICommand WindowClosingCommand { get; private set; }

        /// <summary>
        /// Gets the import from file command.
        /// </summary>
        /// <value>The import from file command.</value>
        public ICommand ImportFromFileCommand { get; private set; }

        /// <summary>
        /// Gets the import from profilux command.
        /// </summary>
        /// <value>The import from profilux command.</value>
        public ICommand ImportFromProfiluxCommand { get; private set; }


        /// <summary>
        /// Gets the export data menu item command.
        /// </summary>
        /// <value>The export data menu item command.</value>
        public ICommand ExportDataMenuItemCommand { get; private set; }

        /// <summary>
        /// Gets the refresh command.
        /// </summary>
        /// <value>The refresh command.</value>
        public ICommand RefreshCommand { get; private set; }

        /// <summary>
        /// Gets the log now command.
        /// </summary>
        /// <value>The log now command.</value>
        public ICommand LogNowCommand { get; private set; }

        /// <summary>
        /// Gets the show profilux control command.
        /// </summary>
        /// <value>The show profilux control command.</value>
        public ICommand ShowProfiluxControlCommand { get; private set; }

        /// <summary>
        /// Gets the documents.
        /// </summary>
        /// <value>The documents.</value>
        public ObservableCollection<ViewModelBase> Documents { get; private set; }

        /// <summary>
        /// Gets or sets the selected document.
        /// </summary>
        /// <value>The selected document.</value>
        public ViewModelBase SelectedDocument
        {
            get
            {
                return this.selectedDocument;
            }

            set
            {
                if (this.selectedDocument != value)
                {
                    this.selectedDocument = value;
                    OnPropertyChanged("SelectedDocument");
                }
            }
        }

        /// <summary>
        /// Gets or sets the last error message.
        /// </summary>
        /// <value>The last error message.</value>
        public string LastLogMessage
        {
            get
            {
                if (LastLog != null)
                {
                    return LastLog.Code + " " + LastLog.Message;
                }

                return string.Empty;
            }
        }

        private LogMessage lastLog;

        LogMessage LastLog
        {
            get { return this.lastLog; }

            set
            {
                if (this.lastLog != value)
                {
                    this.lastLog = value;
                    OnPropertyChanged("LastLogCode");
                    OnPropertyChanged("LastLogMessage");
                }
            }
        }

        /// <summary>
        /// Gets or sets the last log.
        /// </summary>
        /// <value>The last log.</value>
        public int LastLogCode
        {
            get
            {
                if (this.LastLog != null)
                {
                    return this.LastLog.Code;
                }

                return 0;
            }
        }

        /// <summary>
        /// Gets or sets the update progress.
        /// </summary>
        /// <value>The update progress.</value>
        public double UpdateProgress
        {
            get
            {
                return this.updateProgress;
            }

            set
            {
                if (this.updateProgress != value)
                {
                    this.updateProgress = value;
                    OnPropertyChanged("UpdateProgress");
                }
            }
        }

        #region IUpdateProgress Members

        /// <summary>
        /// Gets or sets a value indicating whether DisplayProgress.
        /// </summary>
        public bool DisplayProgress
        {
            get
            {
                return this.displayProgress;
            }

            set
            {
                if (this.displayProgress != value)
                {
                    this.displayProgress = value;
                    OnPropertyChanged("DisplayProgress");
                }
            }
        }

        /// <summary>
        /// Gets or sets the progress text.
        /// </summary>
        /// <value>The progress text.</value>
        public string ProgressText
        {
            get
            {
                return this.progressText;
            }

            set
            {
                if (this.progressText != value)
                {
                    this.progressText = value;
                    OnPropertyChanged("ProgressText");
                }
            }
        }

        /// <summary>
        /// Sets the progress steps.
        /// </summary>
        /// <param name="steps">
        /// The steps.
        /// </param>
        public void SetProgressSteps(double steps)
        {
            this.progressSteps = steps;
            this.UpdateProgress = 0;
            this.currentProgress = 0;
        }

        /// <summary>
        /// Increments the progress.
        /// </summary>
        public void IncrementProgress()
        {
            this.currentProgress++;
            this.UpdateProgress = this.currentProgress / this.progressSteps * 100;
        }

        #endregion

        /// <summary>
        /// Updates the graph.
        /// </summary>
        /// <param name="item">
        /// The item.
        /// </param>
        public void UpdateGraph(BaseInfo item)
        {
            try
            {
                if (item.Display)
                {
                    this.RefreshGraphItems(item);
                }
                else
                {
                    this.CloseGraph(item);
                }
            }
            catch (ReefStatusException ex)
            {
                Logger.Instance.LogError(ex);
            }
        }

        /// <summary>
        /// Updates the graphs.
        /// </summary>
        public void UpdateGraphs()
        {
            try
            {
                foreach (Controler controler in ReefStatusSettings.Instance.Controlers)
                {
                    foreach (BaseInfo graph in controler.Items)
                    {
                        if (graph.Display)
                        {
                            this.RefreshGraphItems(graph);
                        }
                        else
                        {
                            this.CloseGraph(graph);
                        }
                    }
                }
            }
            catch (ReefStatusException ex)
            {
                Logger.Instance.LogError(ex);
            }
        }


        /// <summary>
        /// Creates the graph.
        /// </summary>
        /// <param name="item">The item.</param>
        /// <param name="update">if set to <c>true</c> [update].</param>
        /// <returns></returns>
        public GraphViewModel CreateGraph(BaseInfo item, bool update)
        {
            GraphViewModel graph;
            if (item is Probe)
            {
                graph = new ProbeGraphViewModel(item);
            }
            else if (item is SPort)
            {
                graph = new SPortGraphViewModel(item);
            }
            else
            {
                graph = new GraphViewModel(item);
            }

            Documents.Add(graph);
            SelectedDocument = graph;

            graph.Settings.Display = true;
            if (update)
            {
                UpdateGraph(item);
            }

            return graph;
        }

        /// <summary>
        /// Creates the report view.
        /// </summary>
        /// <param name="item">
        /// The item.
        /// </param>
        public void CreateReportView(BaseInfo item)
        {
            ReportViewModel graph = new ReportViewModel(item);
            Documents.Add(graph);
            SelectedDocument = graph;
        }

        /// <summary>
        /// Creates the data points view.
        /// </summary>
        /// <param name="item">
        /// The item.
        /// </param>
        public void CreateDataPointsView(BaseInfo item)
        {
            DataPointsViewModel graph = new DataPointsViewModel(item);
            Documents.Add(graph);
            SelectedDocument = graph;
        }

        /// <summary>
        /// The clear last error timer tick.
        /// </summary>
        /// <param name="sender">
        /// The sender.
        /// </param>
        /// <param name="e">
        /// The e.
        /// </param>
        private void ClearLastErrorTimerTick(object sender, EventArgs e)
        {
            this.LastLog = null;
            this.clearLastErrorTimer.Stop();
        }

        /// <summary>
        /// Closes the graph.
        /// </summary>
        /// <param name="baseInfo">
        /// The base info.
        /// </param>
        private void CloseGraph(BaseInfo baseInfo)
        {
            GraphViewModel graph = Documents.FirstOrDefault(item => item is GraphViewModel && ((GraphViewModel)item).Settings.Id == baseInfo.Id) as GraphViewModel;
            if (graph != null)
            {
                Documents.Remove(graph);
                graph.Settings.Display = false;
            }
        }

        private ICommand closeWindowCommand;

        private string progressText;

        private void CloseWindow(ViewModelBase item)
        {
            if (item is RemoteViewModel)
            {
                ((RemoteViewModel)item).Stop();
            }

            if (item is GraphViewModel)
            {
                GraphViewModel graph = (GraphViewModel)item;
                graph.Settings.Display = false;
            }

            this.Documents.Remove(item);
        }

        public ICommand CloseWindowCommand
        {
            get
            {
                if (this.closeWindowCommand == null)
                {
                    this.closeWindowCommand = new DelegateCommand<ViewModelBase>(this.CloseWindow);
                }
                return this.closeWindowCommand;
            }
        }

        /// <summary>
        /// Adds to graph display.
        /// </summary>
        /// <param name="timeStamp">The time stamp.</param>
        /// <param name="controler">The controler.</param>
        private void AddToGraphDisplay(DateTime timeStamp, Controler controler)
        {
            if (!this.dispatcher.CheckAccess())
            {
                this.dispatcher.BeginInvoke(new Action(() => AddToGraphDisplay(timeStamp, controler)));
            }
            else
            {
                bool alarm = false;
                foreach (Controler item in ReefStatusSettings.Instance.Controlers)
                {
                    if (item.Info.Alarm == CurrentState.On)
                    {
                        alarm = true;
                        break;
                    }
                }

                this.IsInAlarm = alarm;

                foreach (BaseInfo sensor in controler.Items)
                {
                    if (!(sensor is UserInfo))
                    {
                        AddPointToGraph(sensor, timeStamp);
                    }
                }
            }
        }

        private bool isInAlarm;

        /// <summary>
        /// Gets or sets a value indicating whether this instance is in alarm.
        /// </summary>
        /// <value>
        ///     <c>true</c> if this instance is in alarm; otherwise, <c>false</c>.
        /// </value>
        public bool IsInAlarm
        {
            get
            {
                return this.isInAlarm;
            }
            set
            {
                if (this.isInAlarm != value)
                {
                    this.isInAlarm = value;
                    this.OnPropertyChanged("IsInAlarm");
                }
            }
        }

        /// <summary>
        /// Adds the point to graph.
        /// </summary>
        /// <param name="settings">
        /// The settings.
        /// </param>
        /// <param name="timeStamp">
        /// The time stamp.
        /// </param>
        private void AddPointToGraph(BaseInfo settings, DateTime timeStamp)
        {
            if (settings.Display)
            {
                var graph = (GraphViewModel)this.Documents.FirstOrDefault(view => view is GraphViewModel && ((GraphViewModel)view).Settings.Id == settings.Id) ??
                                       this.CreateGraph(settings, true);

                graph.UpdatePoint(timeStamp);
            }
        }

        /// <summary>
        /// Adds the item to graph.
        /// </summary>
        /// <param name="settings">
        /// The settings.
        /// </param>
        private void RefreshGraphItems(BaseInfo settings)
        {
            if (settings.Display)
            {
                GraphViewModel graph = (GraphViewModel)Documents.FirstOrDefault(view => view is GraphViewModel && ((GraphViewModel)view).Settings.Id == settings.Id);
                if (graph == null)
                {
                    graph = CreateGraph(settings, false);
                }
                else
                {
                    SelectedDocument = graph;
                }

                graph.Refresh();
            }
        }



        /// <summary>
        /// Loggers the on error.
        /// </summary>
        /// <param name="message">
        /// The message.
        /// </param>
        private void LoggerOnError(LogMessage message)
        {
            this.dispatcher.BeginInvoke(new Action(() =>
                                                       {
                                                           LastLog = message;
                                                           this.clearLastErrorTimer.Start();
                                                       }));
        }

        /// <summary>
        /// Imports from file.
        /// </summary>
        private void ImportFromFile()
        {
            if (ReefStatusSettings.Instance.SelectedControler != null)
            {
                OpenFileDialog dialog = new OpenFileDialog { Filter = "Text files (*.txt)|*.txt|All files (*.*)|*.*" };

                if (dialog.ShowDialog().Value)
                {
                    Common.UI.ImportFromFile.Start(
                        dialog.FileName, this.Progress, ReefStatusSettings.Instance.SelectedControler.Id);
                    this.UpdateGraphs();
                }
            }
        }

        /// <summary>
        /// Imports from profilux.
        /// </summary>
        private void ImportFromProfilux()
        {
            if (ReefStatusSettings.Instance.SelectedControler != null)
            {
                Common.UI.ImportFromProfilux.Start(this.Progress, ReefStatusSettings.Instance.SelectedControler);

                this.UpdateGraphs();
            }
        }

        /// <summary>
        /// Exports the data.
        /// </summary>
        private void ExportData()
        {
            if (ReefStatusSettings.Instance.SelectedControler != null)
            {
                SaveFileDialog dialog = new SaveFileDialog { Filter = "Text files (*.txt)|*.txt|All files (*.*)|*.*" };

                if (dialog.ShowDialog().Value)
                {
                    ExportToFile.Start(dialog.FileName, this.Progress, ReefStatusSettings.Instance.SelectedControler.Id);
                }
            }
        }

        /// <summary>
        /// Windows the closing.
        /// </summary>
        private void WindowClosing()
        {
            // shut down the data service and save its state
            ReefStatusSettings.Instance.Save();
            foreach (ViewModelBase document in this.Documents)
            {
                if (document is RemoteViewModel)
                {
                    RemoteViewModel remoteViewModel = (RemoteViewModel)document;
                    remoteViewModel.Stop();
                }
            }

            DataService.Instance.Stop();
        }

        public bool HasProfilux
        {
            get
            {
                const string appname = "ProfiLuxControl II.exe";
                const string appname2 = "ProfiLuxControl.exe";
                const string appname3 = "ProfiLuxControl6.exe";
                string path = this.ProfiluxPath;

                string appLocation3 = path + "\\" + appname3;
                if (File.Exists(appLocation3))
                {
                    return true;
                }


                string appLocation = path + "\\" + appname;
                if (File.Exists(appLocation))
                {
                    return true;
                }

                string appLocation2 = path + "\\" + appname2;
                if (File.Exists(appLocation2))
                {
                    return true;
                }
                return false;
            }
        }

        public string ProfiluxPath
        {
            get
            {
                string path = null;

                RegistryKey ghlKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\GHL\\ProfiLuxControl II");
                if (ghlKey != null)
                {
                    string[] keyNames = ghlKey.GetSubKeyNames();

                    if (keyNames.Length != 0)
                    {
                        try
                        {
                            RegistryKey dirKey = ghlKey.OpenSubKey(keyNames[0]);
                            if (dirKey != null)
                            {
                                 path = (string)dirKey.GetValue("BinDir");      
                            }
                        }
                        catch (Exception)
                        {            
                        }
                    }
                }

                if (string.IsNullOrEmpty(path))
                {
                    string progfiles = Environment.GetFolderPath(Environment.SpecialFolder.ProgramFilesX86);
                    if (string.IsNullOrEmpty(progfiles))
                    {
                        progfiles = Environment.GetFolderPath(Environment.SpecialFolder.ProgramFiles);
                    }

                    try
                    {
                        const string appname2 = "ProfiLuxControl6.exe";
                        foreach (string directory in Directory.GetDirectories(progfiles, "ProfiLuxControl*"))
                        {
                            if (File.Exists(directory + "\\" + appname2))
                            {
                                path = directory;
                            }
                        }
                    }
                    catch (ArgumentException)
                    {
                        path = string.Empty;
                    }

                    if (string.IsNullOrEmpty(path))
                    {
                        try
                        {
                            const string appname2 = "ProfiLuxControl.exe";
                            foreach (string directory in Directory.GetDirectories(progfiles, "ProfiLuxControl_*"))
                            {
                                if (File.Exists(directory + "\\" + appname2))
                                {
                                    path = directory;
                                }
                            }
                        }
                        catch (ArgumentException)
                        {
                            path = string.Empty;
                        }
                    }
                }

                return path;
            }
        }

        /// <summary>
        /// Shows the profilux control.
        /// </summary>
        private void ShowProfiluxControl()
        {
            const string appname = "ProfiLuxControl II.exe";
            const string appname2 = "ProfiLuxControl.exe";
            const string appname3 = "ProfiLuxControl6.exe";
            string path = this.ProfiluxPath;
            
            try
            {
                string appLocation3 = path + "\\" + appname3;
                if (File.Exists(appLocation3))
                {
                    Process app = new Process
                    {
                        StartInfo =
                        {
                            WorkingDirectory = path,
                            FileName = appLocation3
                        }
                    };
                    app.Start();
                }

                string appLocation = path + "\\" + appname;
                if (File.Exists(appLocation))
                {
                    Process app = new Process
                    {
                        StartInfo =
                        {
                            WorkingDirectory = path,
                            FileName = appLocation
                        }
                    };
                    app.Start();
                }

                string appLocation2 = path + "\\" + appname2;
                if (File.Exists(appLocation2))
                {
                    Process app = new Process
                    {
                        StartInfo =
                        {
                            WorkingDirectory = path,
                            FileName = appLocation2
                        }
                    };
                    app.Start();
                }
               
            }
            catch (Exception)
            {
            }
        }

        private ICommand stopCommand;

        /// <summary>
        /// Gets the stop command.
        /// </summary>
        /// <value>The stop command.</value>
        public ICommand StopCommand
        {
            get
            {
                if (this.stopCommand == null)
                {
                    this.stopCommand = new DelegateCommand(this.StopRefresh);
                }
                return this.stopCommand;
            }
        }

        /// <summary>
        /// Stops the refresh.
        /// </summary>
        private void StopRefresh()
        {
            StopProcessing = true;
        }


        /// <summary>
        /// Gets or sets a value indicating whether [stop processing].
        /// </summary>
        /// <value><c>true</c> if [stop processing]; otherwise, <c>false</c>.</value>
        public bool StopProcessing
        {
            get;
            set;
        }

        private StatusViewModel currentStatusView;

        /// <summary>
        /// Gets or sets the current status view.
        /// </summary>
        /// <value>The current status view.</value>
        public StatusViewModel CurrentStatusView
        {
            get
            {
                return this.currentStatusView;
            }
            set
            {
                if (this.currentStatusView != value)
                {
                    this.currentStatusView = value;
                    this.OnPropertyChanged("CurrentStatusView");
                }
            }
        }
    }
}