﻿// --------------------------------------------------------------------------------------------------------------------
// <copyright file="StatusViewModel.cs" company="RedpointGames">
//   2010
// </copyright>
// <summary>
//   The status view model.
// </summary>
// --------------------------------------------------------------------------------------------------------------------

namespace RedPoint.ReefStatus.Gui.ViewModels
{
    using System;
    using System.Globalization;
    using System.Linq;
    using System.Windows.Input;
    using Common.Settings;
    using RedPoint.ReefStatus.Common;
    using RedPoint.ReefStatus.Common.Database;
    using RedPoint.ReefStatus.Common.ProfiLux;
    using RedPoint.ReefStatus.Common.UI.ViewModel;

    /// <summary>
    /// The status view model.
    /// </summary>
    public class StatusViewModel : ViewModelBase
    {
        /// <summary>
        /// The add user value command.
        /// </summary>
        private ICommand addUserValueCommand;

        /// <summary>
        /// The current selection.
        /// </summary>
        private BaseInfo currentSelection;

        /// <summary>
        /// The new user value.
        /// </summary>
        private double newUserValue;

        /// <summary>
        /// The new user value date.
        /// </summary>
        private DateTime newUserValueDate;


        /// <summary>
        /// Prevents a default instance of the <see cref="StatusViewModel"/> class from being created.
        /// </summary>
        /// <param name="controler">The controler.</param>
        public StatusViewModel(Controler controler)
        {
            this.UpdateCommand = new DelegateCommand(this.UpdateItem, () => (this.CurrentSelection != null && this.CurrentSelection is UserInfo));
            this.DeleteCommand = new DelegateCommand(this.DeleteItem, () => (this.CurrentSelection != null && this.CurrentSelection is UserInfo));
            this.AddDataItemCommand = new DelegateCommand(this.AddDataItem);
            this.ShowGraphCommand = new DelegateCommand(this.ShowGraph, () => (this.CurrentSelection != null));

            this.Controler = controler;
        }


        /// <summary>
        /// Gets the controler.
        /// </summary>
        /// <value>The controler.</value>
        public Controler Controler
        {
            get;
            private set;
        }

        private int newDosingRate;

        /// <summary>
        /// Gets or sets the new dosing rate.
        /// </summary>
        /// <value>The new dosing rate.</value>
        public int NewDosingRate
        {
            get
            {
                return this.newDosingRate;
            }

            set
            {
                this.newDosingRate = value;
                this.OnPropertyChanged("NewDosingRate");
            }
        }

        private int newDosingPerDay;

        /// <summary>
        /// Gets or sets the new dosing per day.
        /// </summary>
        /// <value>The new dosing per day.</value>
        public int NewDosingPerDay
        {
            get { return this.newDosingPerDay; }
            set
            {
                this.newDosingPerDay = value;
                this.OnPropertyChanged("NewDosingPerDay");
            }
        }

        private ICommand clearLevelAlarmCommand;

        /// <summary>
        /// Gets the new dosing command.
        /// </summary>
        /// <value>The new dosing command.</value>
        public ICommand ClearLevelAlarmCommand
        {
            get
            {
                if (this.clearLevelAlarmCommand == null)
                {
                    this.clearLevelAlarmCommand = new DelegateCommand(this.ClearLevelAlarm, () => (this.CurrentSelection != null && this.CurrentSelection is LevelSensor && ((LevelSensor)this.CurrentSelection).IsAlarmOn == CurrentState.On));
                }

                return this.clearLevelAlarmCommand;
            }
        }


        /// <summary>
        /// Clears the level alarm.
        /// </summary>
        private void ClearLevelAlarm()
        {
            this.CurrentSelection.Commands.SendClearLevelAlarm((LevelSensor)this.CurrentSelection);
        }

        /// <summary>
        /// Command to reset the hours
        /// </summary>
        private ICommand resetOperationalHoursCommand;

        /// <summary>
        /// Gets the reset operational hours command.
        /// </summary>
        /// <value>The reset operational hours command.</value>
        public ICommand ResetOperationalHoursCommand
        {
            get
            {
                if (this.resetOperationalHoursCommand == null)
                {
                    this.resetOperationalHoursCommand = new DelegateCommand(this.ResetOperationalHours, () => (this.CurrentSelection != null && (this.CurrentSelection is Probe || this.CurrentSelection is Light)));
                }

                return this.resetOperationalHoursCommand;
            }
        }

        /// <summary>
        /// Resets the operational hours.
        /// </summary>
        private void ResetOperationalHours()
        {
            this.CurrentSelection.Commands.SendResetOperationalHours(this.CurrentSelection);
        }

        private ICommand startWaterChangeCommand;

        /// <summary>
        /// Gets the new dosing command.
        /// </summary>
        /// <value>The new dosing command.</value>
        public ICommand StartWaterChangeCommand
        {
            get
            {
                if (this.startWaterChangeCommand == null)
                {
                    this.startWaterChangeCommand = new DelegateCommand(this.StartWaterChange, () => (
                        this.CurrentSelection != null &&
                        this.CurrentSelection is LevelSensor && 
                        ((LevelSensor)this.CurrentSelection).CanDoWaterChange));
                }

                return this.startWaterChangeCommand;
            }
        }

        private void StartWaterChange()
        {
            this.CurrentSelection.Commands.SendStartWaterChange((LevelSensor)this.CurrentSelection);
        }

        private ICommand newDosingCommand;

        /// <summary>
        /// Gets the new dosing command.
        /// </summary>
        /// <value>The new dosing command.</value>
        public ICommand NewDosingCommand
        {
            get
            {
                if (this.newDosingCommand == null)
                {
                    this.newDosingCommand = new DelegateCommand(this.NewDosing, () => (this.CurrentSelection != null && this.CurrentSelection is DosingPump));
                }

                return this.newDosingCommand;
            }
        }

        /// <summary>
        /// News the dosing.
        /// </summary>
        private void NewDosing()
        {
            DosingPump pump = (DosingPump)this.CurrentSelection;
            pump.Commands.SendUpdateDosingRate(pump, this.NewDosingRate, this.NewDosingPerDay);
        }

        private ICommand updateDosingCommand;

        public ICommand UpdateDosingCommand
        {
            get
            {
                if (this.updateDosingCommand == null)
                {
                    this.updateDosingCommand = new DelegateCommand(this.UpdateDosing, () => (this.CurrentSelection != null && this.CurrentSelection is DosingPump));
                }

                return this.updateDosingCommand;
            }
        }

        private void UpdateDosing()
        {
            DosingPump pump = (DosingPump)this.CurrentSelection;
            this.NewDosingPerDay = pump.PerDay;
            this.NewDosingRate = pump.Rate;
        }

        /// <summary>
        /// Gets UpdateCommand.
        /// </summary>
        public ICommand UpdateCommand { get; private set; }

        /// <summary>
        /// Gets DeleteCommand.
        /// </summary>
        public ICommand DeleteCommand { get; private set; }

        /// <summary>
        /// Gets AddDataItemCommand.
        /// </summary>
        public ICommand AddDataItemCommand { get; private set; }

        /// <summary>
        /// Gets ShowGraphCommand.
        /// </summary>
        public ICommand ShowGraphCommand { get; private set; }

        /// <summary>
        /// Gets or sets CurrentSelection.
        /// </summary>
        public BaseInfo CurrentSelection
        {
            get
            {
                return this.currentSelection;
            }

            set
            {
                if (this.currentSelection != value)
                {
                    this.currentSelection = value;
                    MainWindowViewModel.Instance.CurrentStatusView = this;
                    OnPropertyChanged("CurrentSelection");
                    OnPropertyChanged("ProbeSelection");
                    OnPropertyChanged("LevelSelection");
                    OnPropertyChanged("DigitalInputSelection");
                    OnPropertyChanged("SPortSelection");
                    OnPropertyChanged("LPortSelection");
                    OnPropertyChanged("UserValueSelection");
                    OnPropertyChanged("LightSelection");
                    OnPropertyChanged("DosingPumpSelection");
                    OnPropertyChanged("CurrentPumpSelection");
                    this.OnChangeAllGraphStates();
                }
            }
        }

        /// <summary>
        /// Gets or sets the light selection.
        /// </summary>
        /// <value>The light selection.</value>
        public BaseInfo LightSelection
        {
            get
            {
                return this.CurrentSelection is Light ? this.CurrentSelection : null;
            }

            set
            {
                this.CurrentSelection = value;
            }
        }

        /// <summary>
        /// Gets or sets the dosing pump selection.
        /// </summary>
        /// <value>The dosing pump selection.</value>
        public BaseInfo DosingPumpSelection
        {
            get
            {
                return this.CurrentSelection is DosingPump ? this.CurrentSelection : null;
            }

            set
            {
                this.CurrentSelection = value;
            }
        }

        /// <summary>
        /// Gets or sets the probe selection.
        /// </summary>
        /// <value>The probe selection.</value>
        public BaseInfo ProbeSelection
        {
            get
            {
                return this.CurrentSelection is Probe?this.CurrentSelection : null;
            }

            set
            {
                this.CurrentSelection = value;
            }
        }

        /// <summary>
        /// Gets or sets the level selection.
        /// </summary>
        /// <value>The level selection.</value>
        public BaseInfo LevelSelection
        {
            get
            {
                return this.CurrentSelection is LevelSensor ? this.CurrentSelection : null;
            }

            set
            {
                this.CurrentSelection = value;
            }
        }

        /// <summary>
        /// Gets or sets the digital input selection.
        /// </summary>
        /// <value>The digital input selection.</value>
        public BaseInfo DigitalInputSelection
        {
            get
            {
                return this.CurrentSelection is DigitalInput ? this.CurrentSelection : null;
            }

            set
            {
                this.CurrentSelection = value;
            }
        }

        /// <summary>
        /// Gets or sets the S port selection.
        /// </summary>
        /// <value>The S port selection.</value>
        public BaseInfo SPortSelection
        {
            get
            {
                return this.CurrentSelection is SPort ? this.CurrentSelection : null;
            }

            set
            {
                this.CurrentSelection = value;
            }
        }

        public BaseInfo CurrentPumpSelection
        {
            get
            {
                return this.CurrentSelection is CurrentPump ? this.CurrentSelection : null;
            }

            set
            {
                this.CurrentSelection = value;
            }
        }

        /// <summary>
        /// Gets or sets the L port selection.
        /// </summary>
        /// <value>The L port selection.</value>
        public BaseInfo LPortSelection
        {
            get
            {
                return this.CurrentSelection is LPort ? this.CurrentSelection : null;
            }

            set
            {
                this.CurrentSelection = value;
            }
        }

        /// <summary>
        /// Gets or sets the user value selection.
        /// </summary>
        /// <value>The user value selection.</value>
        public BaseInfo UserValueSelection
        {
            get
            {
                return this.CurrentSelection is UserInfo ? this.CurrentSelection : null;
            }

            set
            {
                this.CurrentSelection = value;
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether this <see cref="StatusViewModel"/> is report.
        /// </summary>
        /// <value><c>true</c> if report; otherwise, <c>false</c>.</value>
        public bool Report
        {
            get
            {
                if (this.CurrentSelection != null)
                {
                    BaseInfo item = this.CurrentSelection;
                    ReportViewModel report =
                        MainWindowViewModel.Instance.Documents.FirstOrDefault(
                            view => view is ReportViewModel && ((ReportViewModel)view).Item.Id == item.Id) as ReportViewModel;

                    return report != null;
                }

                return false;
            }

            set
            {
                if (this.CurrentSelection != null)
                {
                    BaseInfo item = this.CurrentSelection;
                    if (value)
                    {
                        if (MainWindowViewModel.Instance.Settings.Layout == ReportLayout.None)
                        {
                            MainWindowViewModel.Instance.Settings.Layout = ReportLayout.Tabbed;
                        }

                        MainWindowViewModel.Instance.CreateReportView(item);
                    }
                    else
                    {
                        ReportViewModel report = MainWindowViewModel.Instance.Documents.FirstOrDefault(view => view is ReportViewModel && ((ReportViewModel)view).Item.Id == item.Id) as ReportViewModel;
                        if (report != null)
                        {
                            MainWindowViewModel.Instance.Documents.Remove(report);
                        }
                    }

                    OnPropertyChanged("Report");
                }
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether [data points].
        /// </summary>
        /// <value><c>true</c> if [data points]; otherwise, <c>false</c>.</value>
        public bool DataPoints
        {
            get
            {
                if (CurrentSelection != null)
                {
                    BaseInfo item = CurrentSelection;
                    DataPointsViewModel report =
                        MainWindowViewModel.Instance.Documents.FirstOrDefault(
                            view => view is DataPointsViewModel && ((DataPointsViewModel)view).Item.Id == item.Id) as DataPointsViewModel;

                    return report != null;
                }

                return false;
            }

            set
            {
                if (CurrentSelection != null)
                {

                    BaseInfo item = CurrentSelection;
                    if (value)
                    {
                        if (MainWindowViewModel.Instance.Settings.Layout == ReportLayout.None)
                        {
                            MainWindowViewModel.Instance.Settings.Layout = ReportLayout.Tabbed;
                        }

                        MainWindowViewModel.Instance.CreateDataPointsView(item);
                    }
                    else
                    {
                        DataPointsViewModel report =
                            MainWindowViewModel.Instance.Documents.FirstOrDefault(
                                view => view is DataPointsViewModel && ((DataPointsViewModel)view).Item.Id == item.Id) as
                            DataPointsViewModel;

                        if (report != null)
                        {
                            MainWindowViewModel.Instance.Documents.Remove(report);
                        }
                    }

                    OnPropertyChanged("DataPoints");
                }
            }
        }

        /// <summary>
        /// Gets or sets the graph range.
        /// </summary>
        /// <value>The graph range.</value>
        public GraphRange GraphRange
        {
            get
            {
                if (this.CurrentSelection != null)
                {
                    return this.CurrentSelection.Range;
                }

                return GraphRange.All;
            }

            set
            {
                if (this.CurrentSelection != null && this.CurrentSelection.Range != value)
                {
                    this.CurrentSelection.Range = value;
                    MainWindowViewModel.Instance.UpdateGraph(this.CurrentSelection);
                    this.OnChangeAllGraphStates();
                }
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether [graph none].
        /// </summary>
        /// <value><c>true</c> if [graph none]; otherwise, <c>false</c>.</value>
        public bool IsGraphShown
        {
            get
            {
                if (this.CurrentSelection != null)
                {
                    return this.CurrentSelection.Display;
                }

                return false;
            }

            set
            {
                if (this.CurrentSelection != null && this.CurrentSelection.Display != value)
                {
                    if (value && MainWindowViewModel.Instance.Settings.Layout == ReportLayout.None)
                    {
                        MainWindowViewModel.Instance.Settings.Layout = ReportLayout.Tabbed;
                    }

                    this.CurrentSelection.Display = value;
                    MainWindowViewModel.Instance.UpdateGraph(this.CurrentSelection);
                    this.OnChangeAllGraphStates();
                }
            }
        }

        public bool IsCurrentGraphShown
        {
            get
            {
                if (this.CurrentSelection != null && this.CurrentSelection is SPort)
                {

                    return ((SPort)this.CurrentSelection).DisplayCurrentGraph;
                }

                return false;
            }

            set
            {
                if (this.CurrentSelection != null && this.CurrentSelection is SPort && ((SPort)this.CurrentSelection).DisplayCurrentGraph != value)
                {
                    if (value && MainWindowViewModel.Instance.Settings.Layout == ReportLayout.None)
                    {
                        MainWindowViewModel.Instance.Settings.Layout = ReportLayout.Tabbed;
                    }

                    this.CurrentSelection.Display = value;
                    MainWindowViewModel.Instance.UpdateGraph(this.CurrentSelection);
                    this.OnChangeAllGraphStates();
                }
            }
        }

        /// <summary>
        /// Gets or sets the new user value date.
        /// </summary>
        /// <value>The new user value date.</value>
        public DateTime NewUserValueDate
        {
            get
            {
                return this.newUserValueDate;
            }

            set
            {
                if (this.newUserValueDate != value)
                {
                    this.newUserValueDate = value;
                    OnPropertyChanged("NewUserValueDate");
                }
            }
        }

        /// <summary>
        /// Gets or sets the new user value.
        /// </summary>
        /// <value>The new user value.</value>
        public double NewUserValue
        {
            get
            {
                return this.newUserValue;
            }

            set
            {
                if (this.newUserValue != value)
                {
                    this.newUserValue = value;
                    OnPropertyChanged("NewUserValue");
                }
            }
        }

        /// <summary>
        /// Gets the add user value command.
        /// </summary>
        /// <value>The add user value command.</value>
        public ICommand AddUserValueCommand
        {
            get
            {
                if (this.addUserValueCommand == null)
                {
                    this.addUserValueCommand = new DelegateCommand(AddUserValue);
                }

                return this.addUserValueCommand;
            }
        }

        /// <summary>
        /// Called when [change all graph states].
        /// </summary>
        public void OnChangeAllGraphStates()
        {
            OnPropertyChanged("DataPoints");
            OnPropertyChanged("Report");
            this.OnPropertyChanged("IsGraphShown");
            this.OnPropertyChanged("GraphRange");
            this.OnPropertyChanged("IsCurrentGraphShown");
        }

        /// <summary>
        /// Adds the user value.
        /// </summary>
        private void AddUserValue()
        {
            UserInfo userInfo = CurrentSelection as UserInfo;
            if (userInfo != null)
            {
                // only display the newest value
                if (userInfo.Time < NewUserValueDate)
                {
                    userInfo.Value = NewUserValue;
                    userInfo.Time = NewUserValueDate;
                }

                try
                {
                    using (IDataAccess access = ReefStatusSettings.Instance.Logging.Connection.Create())
                    {
                        access.InsertItem(this.NewUserValue, this.NewUserValueDate, userInfo.Id, false, userInfo.Controler.Id);
                    }
                }
                catch (ReefStatusException ex)
                {
                    Logger.Instance.LogError(ex);
                }

                MainWindowViewModel.Instance.UpdateGraph(userInfo);
                ReefStatusSettings.Instance.Save();
            }
        }

        /// <summary>
        /// Updates the item.
        /// </summary>
        private void UpdateItem()
        {
            this.NewUserValueDate = DateTime.Now;
            if (this.CurrentSelection.Value == null)
            {
                this.NewUserValue = 0;
            }
            else
            {
                this.NewUserValue = (double)this.CurrentSelection.Value;
            }
        }

        /// <summary>
        /// Shows the graph.
        /// </summary>
        private void ShowGraph()
        {
            if (this.CurrentSelection != null)
            {
                BaseInfo item = this.CurrentSelection;

                if (MainWindowViewModel.Instance.Settings.Layout == ReportLayout.None)
                {
                    MainWindowViewModel.Instance.Settings.Layout = ReportLayout.Tabbed;
                }

                GraphViewModel graph = MainWindowViewModel.Instance.Documents.FirstOrDefault(view => view is GraphViewModel && ((GraphViewModel)view).Settings.Id == item.Id) as GraphViewModel;
                if (graph == null)
                {
                    item.Display = true;
                    MainWindowViewModel.Instance.CreateGraph(item, true);
                }
                else
                {
                    MainWindowViewModel.Instance.SelectedDocument = graph;
                }
            }
        }

        /// <summary>
        /// Deletes the item.
        /// </summary>
        private void DeleteItem()
        {
            BaseInfo item = this.CurrentSelection;
            item.Display = false;
            MainWindowViewModel.Instance.UpdateGraph(item);
            Controler.Items.Remove(item);
            Controler.ChangeUserValues();
            ReefStatusSettings.Instance.Save();
        }

        /// <summary>
        /// Adds the data item.
        /// </summary>
        private void AddDataItem()
        {
            int userIndex = 0;
            foreach (UserInfo item in Controler.UserValues)
            {
                userIndex = Math.Max(int.Parse(item.Id.Substring(3), CultureInfo.InvariantCulture), userIndex);
            }

            userIndex += 1;

            BaseInfo settings = new UserInfo
                                    {
                                        Id = "USR" + userIndex, 
                                        DisplayName = "USR" + userIndex, 
                                        Units = string.Empty
                                    };

            Controler.Items.Add(settings);
            Controler.ChangeUserValues();
            ReefStatusSettings.Instance.Save();
        }
    }
}