// <copyright file="ProgressWindow.cs" company="Redpoint Apps">
// Copyright (c) Redpoint Apps. All rights reserved.
// </copyright>

namespace RedPoint.ReefStatus.Common.UI
{
    using System.Diagnostics;
    using System.Globalization;
    using System.Windows.Forms;
    using RedPoint.ReefStatus.Common;

    /// <summary>
    /// The Progress window
    /// </summary>
    public class ProgressWindow : System.Windows.Forms.Form, IProgressCallback
    {
        private System.Windows.Forms.Button cancelButton;
        private System.Windows.Forms.Label label;
        private System.Windows.Forms.ProgressBar progressBar;

        private string titleRoot = string.Empty;
        private readonly System.Threading.ManualResetEvent initEvent = new System.Threading.ManualResetEvent(false);
        private readonly System.Threading.ManualResetEvent abortEvent = new System.Threading.ManualResetEvent(false);
        private bool requiresClose = true;

        /// <summary>
        /// Initializes a new instance of the <see cref="ProgressWindow"/> class.
        /// </summary>
        public ProgressWindow()
        {
            // Required for Windows Form Designer support
            InitializeComponent();
            label.Text = "Starting operation...";
        }

        private delegate void SetTextInvoker(string text);

        private delegate void IncrementInvoker(int val);

        private delegate void StepToInvoker(int val);

        private delegate void RangeInvoker(int minimum, int maximum);
        
#region Implementation of IProgressCallback

        /// <summary>
        /// Gets a value indicating whether this instance is aborting.
        /// </summary>
        /// <value>
        ///     <c>true</c> if this instance is aborting; otherwise, <c>false</c>.
        /// </value>
        public bool IsAborting
        {
            get
            {
                return this.abortEvent.WaitOne(0, false);
            }
        }

        /// <summary>
        /// Call this method from the worker thread to initialize
        /// the progress meter.
        /// </summary>
        /// <param name="minimum">The minimum value in the progress range (e.g. 0)</param>
        /// <param name="maximum">The maximum value in the progress range (e.g. 100)</param>
        /// <param name="title"></param>
        public void Begin(int minimum, int maximum, string title)
        {
            initEvent.WaitOne();
            Invoke(new RangeInvoker(DoBegin), new object[] { minimum, maximum });
        }

        /// <summary>
        /// Call this method from the worker thread to initialize
        /// the progress callback, without setting the range
        /// </summary>
        /// <param name="title"></param>
        public void Begin(string title)
        {
            initEvent.WaitOne();
            Invoke(new MethodInvoker(DoBegin));
        }

        /// <summary>
        /// Call this method from the worker thread to reset the range in the progress callback
        /// </summary>
        /// <param name="minimum">The minimum value in the progress range (e.g. 0)</param>
        /// <param name="maximum">The maximum value in the progress range (e.g. 100)</param>
        /// <remarks>You must have called one of the Begin() methods prior to this call.</remarks>
        public void SetRange(int minimum, int maximum)
        {
            initEvent.WaitOne();
            Invoke(new RangeInvoker(DoSetRange), new object[] { minimum, maximum });
        }

        /// <summary>
        /// Call this method from the worker thread to update the progress text.
        /// </summary>
        /// <param name="text">The progress text to display</param>
        public void SetText(string text)
        {
            Invoke(new SetTextInvoker(DoSetText), new object[] { text });
        }

        /// <summary>
        /// Call this method from the worker thread to increase the progress counter by a specified value.
        /// </summary>
        /// <param name="value">The amount by which to increment the progress indicator</param>
        public void Increment(int value)
        {
            Invoke(new IncrementInvoker(DoIncrement), new object[] { value });
        }

        /// <summary>
        /// Call this method from the worker thread to step the progress meter to a particular value.
        /// </summary>
        /// <param name="value">The amount by which to increment the progress indicator</param>
        /// <remarks>You must have called one of the Begin() methods prior to this call.</remarks>
        public void StepTo(int value)
        {
            Invoke(new StepToInvoker(DoStepTo), new object[] { value });
        }

        /// <summary>
        /// Call this method from the worker thread to finalize the progress meter
        /// </summary>
        public void End()
        {
            try
            {
                if (requiresClose)
                {
                    Invoke(new MethodInvoker(DoEnd));
                }
            }
            catch (System.InvalidOperationException ex)
            {
                Trace.WriteLine(ex.ToString());
            }
        }

        #endregion

        /// <summary>
        /// Shows the cancel button.
        /// </summary>
        /// <param name="show">if set to <c>true</c> [show].</param>
        public void ShowCancelButton(bool show)
        {
            cancelButton.Visible = show;
        }

        #region Overrides
        /// <summary>
        /// Handles the form load, and sets an event to ensure that
        /// intialization is synchronized with the appearance of the form.
        /// </summary>
        /// <param name="e">An <see cref="T:System.EventArgs"/> that contains the event data.</param>
        protected override void OnLoad(System.EventArgs e)
        {
            base.OnLoad(e);
            initEvent.Set();
            ControlBox = false;
        }

        /// <summary>
        /// Clean up any resources being used.
        /// </summary>
        /// <param name="disposing">true to release both managed and unmanaged resources; false to release only unmanaged resources.</param>
        protected override void Dispose(bool disposing)
        {
            if (disposing)
            {
                if (this.initEvent != null)
                {
                    this.initEvent.Close();
                }

                if (this.abortEvent != null)
                {
                    this.abortEvent.Close();
                }
            }

            base.Dispose(disposing);
        }

        /// <summary>
        /// Handler for 'Close' clicking
        /// </summary>
        /// <param name="e">A <see cref="T:System.ComponentModel.CancelEventArgs"/> that contains the event data.</param>
        protected override void OnClosing(System.ComponentModel.CancelEventArgs e)
        {
            requiresClose = false;
            this.AbortWork();
            base.OnClosing(e);
        }
        #endregion

        #region Implementation members invoked on the owner thread
        private void DoSetText(string text)
        {
            label.Text = text;
        }

        private void DoIncrement(int val)
        {
            progressBar.Increment(val);
            UpdateStatusText();
        }

        private void DoStepTo(int val)
        {
            progressBar.Value = val;
            UpdateStatusText();
        }

        private void DoBegin(int minimum, int maximum)
        {
            this.DoBegin();
            this.DoSetRange(minimum, maximum);
        }

        private void DoBegin()
        {
            this.cancelButton.Enabled = true;
            this.ControlBox = false;
        }

        private void DoSetRange(int minimum, int maximum)
        {
            this.progressBar.Minimum = minimum;
            this.progressBar.Maximum = maximum;
            this.progressBar.Value = minimum;
            this.titleRoot = Text;
        }

        private void DoEnd()
        {
            Close();
        }
        #endregion

        #region Implementation Utilities
        /// <summary>
        /// Utility function that formats and updates the title bar text
        /// </summary>
        private void UpdateStatusText()
        {
            Text = titleRoot + string.Format(CultureInfo.CurrentCulture, " - {0}% complete", (progressBar.Value * 100) / (progressBar.Maximum - progressBar.Minimum));
        }

        /// <summary>
        /// Utility function to terminate the thread
        /// </summary>
        private void AbortWork()
        {
            this.abortEvent.Set();
        }
        #endregion

        #region Windows Form Designer generated code
        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent()
        {
            this.progressBar = new System.Windows.Forms.ProgressBar();
            this.label = new System.Windows.Forms.Label();
            this.cancelButton = new System.Windows.Forms.Button();
            this.SuspendLayout();
            // 
            // progressBar
            // 
            this.progressBar.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left)
                        | System.Windows.Forms.AnchorStyles.Right)));
            this.progressBar.Location = new System.Drawing.Point(8, 80);
            this.progressBar.Name = "progressBar";
            this.progressBar.Size = new System.Drawing.Size(192, 23);
            this.progressBar.TabIndex = 1;
            // 
            // label
            // 
            this.label.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom)
                        | System.Windows.Forms.AnchorStyles.Left)
                        | System.Windows.Forms.AnchorStyles.Right)));
            this.label.Location = new System.Drawing.Point(8, 8);
            this.label.Name = "label";
            this.label.Size = new System.Drawing.Size(272, 64);
            this.label.TabIndex = 0;
            this.label.Text = "Starting operation...";
            // 
            // cancelButton
            // 
            this.cancelButton.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
            this.cancelButton.DialogResult = System.Windows.Forms.DialogResult.Cancel;
            this.cancelButton.Enabled = false;
            this.cancelButton.Location = new System.Drawing.Point(208, 80);
            this.cancelButton.Name = "cancelButton";
            this.cancelButton.Size = new System.Drawing.Size(75, 23);
            this.cancelButton.TabIndex = 2;
            this.cancelButton.Text = "Cancel";
            // 
            // ProgressWindow
            // 
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.ClientSize = new System.Drawing.Size(290, 114);
            this.ControlBox = false;
            this.Controls.Add(this.cancelButton);
            this.Controls.Add(this.progressBar);
            this.Controls.Add(this.label);
            this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedDialog;
            this.MaximizeBox = false;
            this.MinimizeBox = false;
            this.Name = "ProgressWindow";
            this.ShowIcon = false;
            this.ShowInTaskbar = false;
            this.StartPosition = System.Windows.Forms.FormStartPosition.CenterScreen;
            this.Text = "ProgressWindow";
            this.ResumeLayout(false);

        }
        #endregion

        #region IProgressCallback Members

        /// <summary>
        /// Lock Item
        /// </summary>
        private object lockItem = new object();

        /// <summary>
        /// Gets the lock.
        /// </summary>
        /// <value>The lock.</value>
        public object Lock
        {
            get { return lockItem; }
        }

        #endregion
    }
}
