﻿
// <copyright file="ReefStatusFrame.cs" company="Redpoint Apps">
// Copyright (c) Redpoint Apps. All rights reserved.
// </copyright>

namespace RedPoint.ReefStatus
{
    using System;
    using System.Collections.Generic;
    using System.Collections.ObjectModel;
    using System.Diagnostics;
    using System.Drawing;
    using System.Globalization;
    using System.IO;
    using System.Linq;
    using System.Threading;
    using System.Windows.Forms;
    using Microsoft.Win32;
    using RedPoint.ReefStatus.Common;
    using RedPoint.ReefStatus.Common.Core;
    using RedPoint.ReefStatus.Common.Database;
    using RedPoint.ReefStatus.Common.ProfiLux;
    using RedPoint.ReefStatus.Common.Settings;
    using RedPoint.ReefStatus.Common.UI;

    /// <summary>
    /// The main window for the application
    /// </summary>
    public partial class ReefStatusFrame : Form, IReefStatusFrame
    {
        private readonly System.Windows.Forms.Timer clearLastErrorTimer = new System.Windows.Forms.Timer();

        private readonly Remote remote;
        private readonly ErrorLog errorLog;

        private readonly ObservableCollection<GraphForm> graphs = new ObservableCollection<GraphForm>();
        private readonly ObservableCollection<DataPointsView> dataPiontViews = new ObservableCollection<DataPointsView>();
        private readonly ObservableCollection<Report> reportViews = new ObservableCollection<Report>();

        private readonly CustomGraph customGraph;
        private ReefStatusItem selectedItem;

        /// <summary>
        /// Initializes a new instance of the <see cref="ReefStatusFrame"/> class.
        /// </summary>
        public ReefStatusFrame()
        {
            Logger.Instance.OnError += this.LoggerOnError;

            InitializeComponent();

            this.statusText.Text = string.Empty;
            this.lastUpdate.Text = string.Empty;
            
            InitializeLanguageSettings();

            DataService.Instance.Start();
            DataService.Instance.OnLogItem += this.LogData;
            DataService.Instance.OnUpdateStatus += this.UpdateStatus;

            remote = new Remote();
            this.remote.FormClosing += this.Remote_FormClosing;
            this.remote.MdiParent = this;

            this.errorLog = new ErrorLog();
            this.errorLog.FormClosing += this.ErrorLog_FormClosing;
            this.errorLog.MdiParent = this;

            probeItem.Content.Columns.RemoveAt(1);
            lightItem.Content.Columns.RemoveAt(1);
            dosingPumpItem.Content.Columns.RemoveAt(1);
            userItem.Content.Columns.RemoveAt(1);

            UpdateDisplay();

            customGraph = new CustomGraph(this);
            customGraph.FormClosing += CustomGraph_FormClosing;

            if (!ReefStatusSettings.Instance.Window.Size.IsEmpty)
            {
                Size = ReefStatusSettings.Instance.Window.Size;
                Location = ReefStatusSettings.Instance.Window.Location;
                WindowState = ReefStatusSettings.Instance.Window.WindowState;
            }

            errorText.Text = string.Empty;
            UpdateNetworkStatusText();

            ReefStatusSettings.Instance.FirstControler.Commands.UpdateAll();
            dataTableLayout.Visible = false;
            dataLogToolStripMenuItem.Checked = false;
            Show();
            UpdateGraphs(ReefStatusSettings.Instance.FirstControler);
            LayoutMdi(MdiLayout.TileHorizontal);

            clearLastErrorTimer.Tick += ClearLastErrorTimerTick;
            clearLastErrorTimer.Interval = 20000;

            probeItem.Content.MouseDown += this.ControlMouseDown;
            levelSensorItem.Content.MouseDown += this.ControlMouseDown;
            digitalInputItem.Content.MouseDown += this.ControlMouseDown;
            lportItem.Content.MouseDown += this.ControlMouseDown;
            sportItem.Content.MouseDown += this.ControlMouseDown;
            lightItem.Content.MouseDown += this.ControlMouseDown;
            dosingPumpItem.Content.MouseDown += this.ControlMouseDown;
            userItem.Content.MouseDown += this.ControlMouseDown;
            userItem.Header.MouseDown += this.ControlMouseDown;


            probeItem.Content.DoubleClick += this.ControlDoubleClick;
            levelSensorItem.Content.DoubleClick += this.ControlDoubleClick;
            digitalInputItem.Content.DoubleClick += this.ControlDoubleClick;
            lportItem.Content.DoubleClick += this.ControlDoubleClick;
            sportItem.Content.DoubleClick += this.ControlDoubleClick;
            lightItem.Content.DoubleClick += this.ControlDoubleClick;
            dosingPumpItem.Content.DoubleClick += this.ControlDoubleClick;
            userItem.Content.DoubleClick += this.ControlDoubleClick;


        }

        #region IReefStatusFrame
        /// <summary>
        /// Removes the graph.
        /// </summary>
        /// <param name="item">The item.</param>
        public void RemoveGraph(BaseInfo item)
        {
            GraphForm graphform = this.graphs.FirstOrDefault(graph => graph.Settings == item);
            if (graphform != null)
            {
                this.graphs.Remove(graphform);
            }
        }

        #endregion

        /// <summary>
        /// Clears the last error timer tick.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void ClearLastErrorTimerTick(object sender, EventArgs e)
        {
            this.errorText.Text = string.Empty;
            this.clearLastErrorTimer.Stop();
        }

        /// <summary>
        /// Loggers the on error.
        /// </summary>
        /// <param name="message">The message.</param>
        private void LoggerOnError(LogMessage message)
        {
            if (this.InvokeRequired)
            {
                this.BeginInvoke(new Action(() => this.LoggerOnError(message)));
            }
            else
            {
                if (errorLog != null)
                {
                    errorLog.LogError(string.Format("{0} {1}", message.Code, message.Message));
                }

                errorText.Text = string.Format("{0} {1}", message.Code, message.Message);
                clearLastErrorTimer.Start();
            }
        }

        #region ICommandReply

        /// <summary>
        /// Updates the status.
        /// </summary>
        /// <param name="controler">The controler.</param>
        private void UpdateStatus(Controler controler)
        {
            if (this.InvokeRequired)
            {
                this.BeginInvoke(new Action(() => this.UpdateStatus(controler)));
            }
            else
            {
                this.UpdateDisplay();
            }
        }

        /// <summary>
        /// Logs the data.
        /// </summary>
        /// <param name="now">The now.</param>
        private void LogData(DateTime now, Controler controler)
        {
            if (this.InvokeRequired)
            {
                this.BeginInvoke(new Action(() => this.LogData(now, controler)));
            }
            else
            {
                AddToLogDisplay(now, controler);
                AddToGraphDisplay(now, controler);
            }
        }

        #endregion

        /// <summary>
        /// Adds the item to graph.
        /// </summary>
        /// <param name="item">The item.</param>
        /// <param name="value">The value.</param>
        /// <param name="now">The now.</param>
        /// <param name="controler">The controler.</param>
        private void AddItemToGraph(BaseInfo item, double value, DateTime now, Controler controler)
        {

            if (item is Probe)
            {
                value = ((Probe)item).ConvertValue(value);
            }

            GraphForm graph;
            if (item.Display)
            {
                graph = this.graphs.FirstOrDefault(form => form.Settings == item);
                if (graph == null)
                {
                    graph = CreateGraph(item, true);
                }

                graph.AddPoint(now, value);
            }

            customGraph.AddPoint(item, now, value);
        }

        /// <summary>
        /// Updates the user data display.
        /// </summary>
        private void UpdateUserDataDisplay()
        {
            userItem.Content.Items.Clear();
            foreach (UserInfo settings in ReefStatusSettings.Instance.FirstControler.UserValues)
            {
                ReefStatusItem treeItem = new ReefStatusItem(settings);
                treeItem.SubItems.Add(string.Empty);
                treeItem.SubItems.Add(settings.ValueWithUnits);
                userItem.Content.Items.Add(treeItem);
            }
        }

        /// <summary>
        /// Updates the sensors dispaly.
        /// </summary>
        private void UpdateDisplay()
        {
            statusText.Text = ReefStatusSettings.Instance.FirstControler.Info.InfoText;
            lastUpdate.Text = ReefStatusSettings.Instance.FirstControler.Info.LastUpdate.ToString();
            if (ReefStatusSettings.Instance.FirstControler.Info.Alarm == CurrentState.On)
            {
                this.Text = "Reef Status " + "ALARM!";
            }
            else
            {
                this.Text = "Reef Status";
            }

            probeItem.Content.Items.Clear();
            levelSensorItem.Content.Items.Clear();
            sportItem.Content.Items.Clear();
            lportItem.Content.Items.Clear();
            digitalInputItem.Content.Items.Clear();
            lightItem.Content.Items.Clear();
            dosingPumpItem.Content.Items.Clear();


            probeItem.Visible = ReefStatusSettings.Instance.FirstControler.Probes.Count != 0;

            foreach (Probe probe in ReefStatusSettings.Instance.FirstControler.Probes)
            {
                ReefStatusItem treeItem = new ReefStatusItem(probe);
                treeItem.SubItems.Add(probe.ValueWithUnits);
                probeItem.Content.Items.Add(treeItem);
            }

            levelSensorItem.Visible = ReefStatusSettings.Instance.FirstControler.LevelSensors.Count != 0;

            foreach (LevelSensor sensor in ReefStatusSettings.Instance.FirstControler.LevelSensors)
            {
                ReefStatusItem treeItem = new ReefStatusItem(sensor);
                treeItem.SubItems.Add(sensor.Mode);
                treeItem.SubItems.Add(sensor.ValueWithUnits);
                levelSensorItem.Content.Items.Add(treeItem);
            }

            digitalInputItem.Visible = ReefStatusSettings.Instance.FirstControler.DigitalInputs.Count != 0;

            foreach (DigitalInput sensor in ReefStatusSettings.Instance.FirstControler.DigitalInputs)
            {
                ReefStatusItem treeItem = new ReefStatusItem(sensor);
                treeItem.SubItems.Add(sensor.Mode);
                treeItem.SubItems.Add(sensor.ValueWithUnits);
                digitalInputItem.Content.Items.Add(treeItem);
            }

            sportItem.Visible = ReefStatusSettings.Instance.FirstControler.SPorts.Count != 0;

            foreach (DeviceInfo socket in ReefStatusSettings.Instance.FirstControler.SPorts)
            {
                ReefStatusItem treeItem = new ReefStatusItem(socket);
                treeItem.SubItems.Add(socket.Mode);
                treeItem.SubItems.Add(socket.ValueWithUnits);
                sportItem.Content.Items.Add(treeItem);
            }

            lportItem.Visible = ReefStatusSettings.Instance.FirstControler.LPorts.Count != 0;

            foreach (DeviceInfo socket in ReefStatusSettings.Instance.FirstControler.LPorts)
            {
                ReefStatusItem treeItem = new ReefStatusItem(socket);
                treeItem.SubItems.Add(socket.Mode);
                treeItem.SubItems.Add(socket.ValueWithUnits);
                lportItem.Content.Items.Add(treeItem);
            }

            lightItem.Visible = ReefStatusSettings.Instance.FirstControler.Lights.Count != 0;

            foreach (Light light in ReefStatusSettings.Instance.FirstControler.Lights)
            {
                ReefStatusItem treeItem = new ReefStatusItem(light);
                treeItem.SubItems.Add(light.ValueWithUnits);
                lightItem.Content.Items.Add(treeItem);
            }

            dosingPumpItem.Visible = ReefStatusSettings.Instance.FirstControler.DosingPumps.Count != 0;

            foreach (DosingPump pump in ReefStatusSettings.Instance.FirstControler.DosingPumps)
            {
                ReefStatusItem treeItem = new ReefStatusItem(pump);
                treeItem.SubItems.Add(pump.ValueWithUnits);
                dosingPumpItem.Content.Items.Add(treeItem);
            }

            UpdateUserDataDisplay();
        }

        /// <summary>
        /// Creates the graph.
        /// </summary>
        /// <param name="settings">The settings.</param>
        /// <param name="update">if set to <c>true</c> [update].</param>
        /// <returns>Creates the graph </returns>
        private GraphForm CreateGraph(BaseInfo settings, bool update)
        {
            GraphForm graph = new GraphForm(settings, this);
            this.graphs.Add(graph);
            graph.Visible = true;
            if (update)
            {
                this.UpdateGraph(settings);
            }

            return graph;
        }

        private void InitializeLanguageSettings()
        {
            groupBox2.Text = "Info";
            graphsToolStripMenuItem.Text = "&File";
            loadDataToolStripMenuItem.Text = "Import Data";
            this.fileToolStripMenuItem.Text = "From File...";
            this.profiluxToolStripMenuItem.Text = "From Profilux";
            this.exportDataToolStripMenuItem.Text = "Export Data...";
            exitToolStripMenuItem.Text = "Exit";
            comToolStripMenuItem.Text = "&Commands";
            refreshToolStripMenuItem.Text = "Refresh";
            logToolStripMenuItem.Text = "Log Now";
            thunderstormToolStripMenuItem.Text = "Thunderstorm...";
            connectionToolStripMenuItem.Text = "&Tools";
            this.remoteToolStripMenuItem.Text = "Remote";
            this.errorLogToolStripMenuItem.Text = "Log";
            this.profiLuxControlIIToolStripMenuItem.Text = "ProfiLuxControl II...";
            logSettingsToolStripMenuItem.Text = "Options...";
            windowToolStripMenuItem.Text = "&Window";
            arrangeToolStripMenuItem.Text = "Arrange";
            cascadeToolStripMenuItem1.Text = "Cascade";
            tileHorizontalToolStripMenuItem1.Text = "Tile Horizontal";
            tileVerticalToolStripMenuItem1.Text = "Tile Vertical";
            arrangeIconsToolStripMenuItem1.Text = "Arrange Icons";
            helpToolStripMenuItem.Text = "&Help";
            aboutToolStripMenuItem.Text = "About...";
            this.tabPage1.Text = "Status";
            this.columnHeader6.Text = "Name";
            this.columnHeader7.Text = "Mode";
            this.columnHeader8.Text = "Value";
            this.tabPage3.Text = "Info";
            this.statsToolStripMenuItem.Text = "Report";
            this.dataPointsToolStripMenuItem.Text = "Data Points";
            this.graphToolStripMenuItem.Text = "Graph";
            this.allToolStripMenuItem.Text = "All";
            this.dayToolStripMenuItem.Text = "Day";
            this.weekToolStripMenuItem.Text = "Week";
            this.monthToolStripMenuItem.Text = "Month";
            this.yearToolStripMenuItem.Text = "Year";
            this.noneToolStripMenuItem.Text = "None";
            this.Text = "Reef Status";
            this.addDataItemToolStripMenuItem.Text = "Add Data Item...";

            foreach (Report view in reportViews)
            {
                view.InitializeLanguageSettings();
            }
        }

        /// <summary>
        /// Adds the item to graph.
        /// </summary>
        /// <param name="item">The item.</param>
        /// <param name="points">The points.</param>
        private void AddItemsToGraph(BaseInfo item, Collection<DataPoint> points)
        {
            GraphForm graph;
            if (item.Display)
            {
                graph = this.graphs.FirstOrDefault(form => form.Settings == item);
                if (graph == null)
                {
                    graph = CreateGraph(item, true);
                }

                if (item is Probe)
                {
                    Probe probe = (Probe)item;
                    foreach (var point in points)
                    {
                        point.Value = probe.ConvertValue(point.Value);
                    }
                }

                graph.AddPoints(points);
            }
        }

        /// <summary>
        /// Adds the log sub item.
        /// </summary>
        /// <param name="name">The name.</param>
        /// <param name="value">The value.</param>
        /// <param name="item">The item.</param>
        private void AddLogSubItem(string name, string value, ListViewItem item)
        {
            int column;
            if (!dataLog.Columns.ContainsKey(name))
            {
                column = dataLog.Columns.Count;
                dataLog.Columns.Add(name, name, 60);
            }
            else
            {
                column = dataLog.Columns.IndexOfKey(name);
            }

            while (item.SubItems.Count <= column)
            {
                item.SubItems.Add(string.Empty);
            }

            item.SubItems[column].Text = value;
        }

        /// <summary>
        /// Disconnects this instance.
        /// </summary>
        private void UpdateNetworkStatusText()
        {
            errorText.Text = string.Empty;
            connectionText.Text = ReefStatusSettings.Instance.FirstControler.Connection.NetworkConntectionText;
        }

        private void UpdateGraphs(Controler controler)
        {
            ProgressWindow progress = new ProgressWindow { Text = "Updating" };
            progress.ShowCancelButton(false);
            new Thread(() =>
                {
                    try
                    {
                        using (IDataAccess data = ReefStatusSettings.Instance.Logging.Connection.Create())
                        {
                            if (progress != null)
                            {
                                progress.Begin(0, controler.Items.Count, string.Empty);
                            }

                            int count = 0;
                            foreach (BaseInfo item in controler.Items)
                            {
                                if (item.Display)
                                {
                                    if (progress != null)
                                    {
                                        progress.SetText(Language.GetResource("strUpdateGraph") + " " + item.DisplayName);
                                    }

                                    Collection<Common.ProfiLux.DataPoint> points = new Collection<Common.ProfiLux.DataPoint>();
                                    switch (item.Range)
                                    {
                                        case GraphRange.All:
                                            points = data.GetDataPoints(item.Id, false, controler.Id);
                                            break;
                                        case GraphRange.Year:
                                            points = data.GetDataPoints(item.Id, DateTime.Now.AddYears(-1), false, controler.Id);
                                            break;
                                        case GraphRange.Month:
                                            points = data.GetDataPoints(item.Id, DateTime.Now.AddMonths(-1), false, controler.Id);
                                            break;
                                        case GraphRange.Week:
                                            points = data.GetDataPoints(item.Id, DateTime.Now.AddDays(-7), false, controler.Id);
                                            break;
                                        case GraphRange.Day:
                                            points = data.GetDataPoints(item.Id, DateTime.Now.AddDays(-1), false, controler.Id);
                                            break;
                                    }

                                    BaseInfo info = item;
                                    BeginInvoke(new Action(() => this.AddItemsToGraph(info, points)));
                                }
                                else
                                {
                                    BaseInfo info = item;
                                    BeginInvoke(new Action(() => this.CloseGraph(info)));
                                }

                                if (progress != null)
                                {
                                    progress.StepTo(count++);
                                }
                            }
                        }
                    }
                    catch (ReefStatusException ex)
                    {
                        Logger.Instance.LogError(ex);
                    }
                    finally
                    {
                        if (progress != null)
                        {
                            progress.End();
                        }
                    }
                }).Start();
            progress.ShowDialog();

            UpdateCustomGraph();
        }

        private void CloseGraph(BaseInfo item)
        {
            GraphForm graph = this.graphs.FirstOrDefault(form => form.Settings == item);
            if (graph != null)
            {
                this.graphs.Remove(graph);
                graph.ForceClose();
            }
        }

        /// <summary>
        /// Adds to log display.
        /// </summary>
        /// <param name="now">The now.</param>
        /// <param name="controler">The controler.</param>
        private void AddToLogDisplay(DateTime now, Controler controler)
        {
            if (this.dataLog.Columns.Count == 0)
            {
                this.dataLog.Columns.Add("Time", "Time", 60);
            }

            ListViewItem item = dataLog.Items.Add(now.ToString());

            foreach (BaseInfo param in controler.Items)
            {
                if (!(param is UserInfo))
                {
                    this.AddLogSubItem(param.Id, param.Value.ToString(), item);
                }
            }

            if (dataLog.Items.Count > 100)
            {
                this.dataLog.Items.RemoveAt(0); // only display the last 100 records
            }
        }

        /// <summary>
        /// Adds to graph display.
        /// </summary>
        /// <param name="now">The now.</param>
        private void AddToGraphDisplay(DateTime now, Controler controler)
        {
            foreach (BaseInfo param in controler.Items)
            {
                if (!(param is UserInfo))
                {
                    this.AddItemToGraph(param, param.DoubleValue, now, controler);
                }
            }
        }

        #region FileMenu

        /// <summary>
        /// Handles the Click event of the loadDataToolStripMenuItem control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void LoadDataToolStripMenuItem_Click(object sender, EventArgs e)
        {
            errorText.Text = string.Empty;
            System.Windows.Forms.OpenFileDialog dialog = new System.Windows.Forms.OpenFileDialog
                                                             {
                                                                 Filter = "Text files (*.txt)|*.txt|All files (*.*)|*.*"
                                                             };

            if (dialog.ShowDialog() == DialogResult.OK)
            {
                ProgressWindow progress = new ProgressWindow { Text = "Import Data" };
                ImportFromFile.Start(dialog.FileName, progress, ReefStatusSettings.Instance.FirstControler.Id);
                progress.ShowDialog();
                UpdateGraphs(ReefStatusSettings.Instance.FirstControler);
            }
        }

        /// <summary>
        /// Handles the Click event of the profiluxToolStripMenuItem control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void ProfiluxToolStripMenuItem_Click(object sender, EventArgs e)
        {
            try
            {
                ProgressWindow progress = new ProgressWindow { Text = "Import Data" };
                ImportFromProfilux.Start(progress, ReefStatusSettings.Instance.FirstControler);
                progress.ShowDialog();
                UpdateGraphs(ReefStatusSettings.Instance.FirstControler);
            }
            catch (ReefStatusException ex)
            {
                Logger.Instance.LogError(ex);
            }
        }

        /// <summary>
        /// Handles the Click event of the exportDataToolStripMenuItem control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void ExportDataToolStripMenuItem_Click(object sender, EventArgs e)
        {
            errorText.Text = string.Empty;
            System.Windows.Forms.SaveFileDialog dialog = new System.Windows.Forms.SaveFileDialog
                                                             {
                                                                 Filter = "Text files (*.txt)|*.txt|All files (*.*)|*.*"
                                                             };

            if (dialog.ShowDialog() == DialogResult.OK)
            {
                ProgressWindow progress = new ProgressWindow { Text = "Export Data" };
                ExportToFile.Start(dialog.FileName, progress, ReefStatusSettings.Instance.FirstControler.Id);
                progress.ShowDialog();
            }
        }

        /// <summary>
        /// Handles the Click event of the exitToolStripMenuItem control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void ExitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Close();
        }

        #endregion
        #region CommandMenu

        /// <summary>
        /// Handles the Click event of the refreshToolStripMenuItem control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void RefreshToolStripMenuItem_Click(object sender, EventArgs e)
        {
            errorText.Text = string.Empty;
            ReefStatusSettings.Instance.FirstControler.Commands.UpdateAll();
        }

        /// <summary>
        /// Handles the Click event of the logToolStripMenuItem control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void LogToolStripMenuItem_Click(object sender, EventArgs e)
        {
            errorText.Text = string.Empty;
            ReefStatusSettings.Instance.FirstControler.Commands.LogParameters();
        }

        /// <summary>
        /// Handles the Click event of the thunderstormToolStripMenuItem control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void ThunderstormToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ThunderDialog dialog = new ThunderDialog { duration = { Value = 5 } };

            if (dialog.ShowDialog() == DialogResult.OK)
            {
                errorText.Text = string.Empty;
                ReefStatusSettings.Instance.FirstControler.Commands.ThunderStorm((int)dialog.duration.Value);
            }
        }

        #endregion

        #region ToolMenu

        /// <summary>
        /// Handles the Click event of the optionsToolStripMenuItem control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void OptionsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OptionsDialog dialog = new OptionsDialog();

            if (dialog.ShowDialog(this) == DialogResult.OK)
            {
                lock (ReefStatusSettings.Instance)
                {
                    errorText.Text = string.Empty;
                    dialog.UpdateSettings();

                    InitializeLanguageSettings();

                    connectionText.Text = ReefStatusSettings.Instance.FirstControler.Connection.NetworkConntectionText;

                    UpdateNetworkStatusText();

                    DataService.Instance.UpdateSettings();
                }
            }
        }

        /// <summary>
        /// Handles the Click event of the remoteToolStripMenuItem control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void RemoteToolStripMenuItem_Click(object sender, EventArgs e)
        {
            remote.Visible = !remote.Visible;
            remoteToolStripMenuItem.Checked = remote.Visible;
            remote.buttonLayout.Enabled = true;
        }

        /// <summary>
        /// Handles the Click event of the errorLogToolStripMenuItem control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void ErrorLogToolStripMenuItem_Click(object sender, EventArgs e)
        {
            errorLog.Visible = !errorLog.Visible;
            errorLogToolStripMenuItem.Checked = errorLog.Visible;
        }

        /// <summary>
        /// Handles the Click event of the profiLuxControlIIToolStripMenuItem control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void ProfiLuxControlIIToolStripMenuItem_Click(object sender, EventArgs e)
        {
            RegistryKey ghlKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\GHL\\ProfiLuxControl II");
            if (ghlKey != null)
            {
                string[] keyNames = ghlKey.GetSubKeyNames();

                const string ProfiLuxControlAppName = "ProfiLuxControl II.exe";
                errorText.Text = string.Empty;
                try
                {
                    RegistryKey subKey = ghlKey.OpenSubKey(keyNames[0]);
                    if (subKey == null)
                    {
                        throw new NotImplementedException();
                    }

                    string path = (string)subKey.GetValue("BinDir");
                    if (!string.IsNullOrEmpty(path))
                    {
                        string appLocation = path + "\\" + ProfiLuxControlAppName;
                        if (File.Exists(appLocation))
                        {
                            Process app = new Process
                                              {
                                                  StartInfo =
                                                      {
                                                          WorkingDirectory = path,
                                                          FileName = appLocation
                                                      }
                                              };
                            app.Start();
                        }
                    }
                }
                catch (System.Exception ex)
                {
                    Logger.Instance.Log(
                        new LogMessage(8000, "Error Starting ProfiLuxControl II") { Exception = ex });
                }
            }
        }

        #endregion

        #region WindowMenu
        /// <summary>
        /// Handles the Click event of the cascadeToolStripMenuItem1 control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void CascadeToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            this.LayoutMdi(MdiLayout.Cascade);
        }

        /// <summary>
        /// Handles the Click event of the tileHorizontalToolStripMenuItem1 control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void TileHorizontalToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            this.LayoutMdi(MdiLayout.TileHorizontal);
        }

        /// <summary>
        /// Handles the Click event of the tileVerticalToolStripMenuItem1 control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void TileVerticalToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            this.LayoutMdi(MdiLayout.TileVertical);
        }

        /// <summary>
        /// Handles the Click event of the arrangeIconsToolStripMenuItem1 control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void ArrangeIconsToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            this.LayoutMdi(MdiLayout.ArrangeIcons);
        }

        /// <summary>
        /// Handles the FormClosing event of the ReefStatusControl control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.Forms.FormClosingEventArgs"/> instance containing the event data.</param>
        private void ReefStatusControl_FormClosing(object sender, FormClosingEventArgs e)
        {
            ReefStatusSettings.Instance.Window.Size = Size;
            ReefStatusSettings.Instance.Window.Location = Location;
            ReefStatusSettings.Instance.Window.WindowState = WindowState;
            ReefStatusSettings.Instance.Save();
            DataService.Instance.Stop();
        }

        #endregion

        #region HelpMenu
        /// <summary>
        /// Handles the Click event of the aboutToolStripMenuItem control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void AboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AboutBox box = new AboutBox();
            box.ShowDialog();
        }

        #endregion

        /// <summary>
        /// Handles the FormClosing event of the ErrorLog control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.Forms.FormClosingEventArgs"/> instance containing the event data.</param>
        private void ErrorLog_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (e.CloseReason != CloseReason.FormOwnerClosing && e.CloseReason != CloseReason.MdiFormClosing)
            {
                errorLog.Visible = false;
                errorLogToolStripMenuItem.Checked = errorLog.Visible;
                e.Cancel = true;
            }
        }

        /// <summary>
        /// Handles the FormClosing event of the Remote control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.Forms.FormClosingEventArgs"/> instance containing the event data.</param>
        private void Remote_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (e.CloseReason != CloseReason.FormOwnerClosing && e.CloseReason != CloseReason.MdiFormClosing)
            {
                remote.Visible = false;
                remoteToolStripMenuItem.Checked = remote.Visible;
                e.Cancel = true;
            }
        }

        /// <summary>
        /// Handles the FormClosing event of the customGraph control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.Forms.FormClosingEventArgs"/> instance containing the event data.</param>
        private void CustomGraph_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (e.CloseReason != CloseReason.FormOwnerClosing && e.CloseReason != CloseReason.MdiFormClosing)
            {
                customGraph.Visible = false;
                showToolStripMenuItem.Checked = customGraph.Visible;
                e.Cancel = true;
            }
        }

        /// <summary>
        /// Updates the and show menu.
        /// </summary>
        /// <param name="location">The location.</param>
        private void UpdateAndShowMenu(Point location)
        {
            BaseInfo item = ReefStatusSettings.Instance.FirstControler.GetItem((string)selectedItem.Tag);
            if (item.Display)
            {
                allToolStripMenuItem.Checked = item.Range == GraphRange.All;
                dayToolStripMenuItem.Checked = item.Range == GraphRange.Day;
                weekToolStripMenuItem.Checked = item.Range == GraphRange.Week;
                monthToolStripMenuItem.Checked = item.Range == GraphRange.Month;
                yearToolStripMenuItem.Checked = item.Range == GraphRange.Year;
                noneToolStripMenuItem.Checked = false;
            }
            else
            {
                allToolStripMenuItem.Checked = false;
                dayToolStripMenuItem.Checked = false;
                weekToolStripMenuItem.Checked = false;
                monthToolStripMenuItem.Checked = false;
                yearToolStripMenuItem.Checked = false;
                noneToolStripMenuItem.Checked = true;
            }

            dataPointsToolStripMenuItem.Checked = dataPiontViews.FirstOrDefault(form => form.Item == item) != null;
            statsToolStripMenuItem.Checked = reportViews.FirstOrDefault(form => form.Item == item) != null;

            dataMenu.Items.Remove(deleteToolStripMenuItem);
            dataMenu.Items.Remove(updateValueToolStripMenuItem);

            if (selectedItem.ListView == userItem.Content)
            {
                dataMenu.Items.Add(updateValueToolStripMenuItem);
                dataMenu.Items.Add(deleteToolStripMenuItem);
            }

            customGraphToolStripMenuItem1.Checked = ReefStatusSettings.Instance.CustomGraph.ShowCurves.Contains(item.Id);

            dataMenu.Show(location);
        }

        /// <summary>
        /// Handles the Click event of the statsToolStripMenuItem control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void StatsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            BaseInfo item = ReefStatusSettings.Instance.FirstControler.GetItem((string)selectedItem.Tag);

            Report view = reportViews.FirstOrDefault(form => form.Item == item);
            if (view != null)
            {
                view.UpdateReport();
                view.Visible = true;
                view.WindowState = FormWindowState.Maximized;
            }
            else
            {
                view = new Report(item);
                view.FormClosing += Report_FormClosing;
                view.MdiParent = this;
                view.Text = item.DisplayName + " Report";
                view.Visible = true;
                view.Tag = item.Id;
                view.WindowState = FormWindowState.Maximized;
                reportViews.Add(view);
            }
        }

        /// <summary>
        /// Handles the Click event of the dataPointsToolStripMenuItem control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void DataPointsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            try
            {
                BaseInfo item = ReefStatusSettings.Instance.FirstControler.GetItem((string)selectedItem.Tag);

                DataPointsView view = dataPiontViews.FirstOrDefault(form => form.Item == item);
                if (view != null)
                {
                    view.UpdateData();
                    view.Visible = true;
                    view.WindowState = FormWindowState.Maximized;
                }
                else
                {
                    view = new DataPointsView(ReefStatusSettings.Instance.Logging.Connection, item);
                    view.FormClosing += DataPoint_FormClosing;
                    view.MdiParent = this;
                    view.Text = item.DisplayName + " Data Points";
                    view.Visible = true;
                    view.Tag = item.Id;
                    view.WindowState = FormWindowState.Maximized;
                    dataPiontViews.Add(view);
                }
            }
            catch (ReefStatusException ex)
            {
                Logger.Instance.LogError(ex);
            }
        }

        /// <summary>
        /// Handles the FormClosing event of the Remote control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.Forms.FormClosingEventArgs"/> instance containing the event data.</param>
        private void DataPoint_FormClosing(object sender, FormClosingEventArgs e)
        {
            DataPointsView view = sender as DataPointsView;
            if (view != null)
            {
                if (dataPiontViews.Contains(view))
                {
                    dataPiontViews.Remove(view);
                }
            }
        }

        /// <summary>
        /// Handles the FormClosing event of the Remote control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.Forms.FormClosingEventArgs"/> instance containing the event data.</param>
        private void Report_FormClosing(object sender, FormClosingEventArgs e)
        {
            Report view = sender as Report;
            if (view != null)
            {

                if (reportViews.Contains(view))
                {
                    reportViews.Remove(view);
                }
            }
        }

        /// <summary>
        /// Handles the Click event of the allToolStripMenuItem control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void AllToolStripMenuItem_Click(object sender, EventArgs e)
        {
            BaseInfo item = ReefStatusSettings.Instance.FirstControler.GetItem((string)selectedItem.Tag);
            item.Display = true;
            item.Range = GraphRange.All;
            UpdateGraph(item);
        }

        /// <summary>
        /// Handles the Click event of the dayToolStripMenuItem control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void DayToolStripMenuItem_Click(object sender, EventArgs e)
        {
            BaseInfo item = ReefStatusSettings.Instance.FirstControler.GetItem((string)selectedItem.Tag);
            item.Display = true;
            item.Range = GraphRange.Day;
            UpdateGraph(item);
        }

        /// <summary>
        /// Handles the Click event of the weekToolStripMenuItem control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void WeekToolStripMenuItem_Click(object sender, EventArgs e)
        {
            BaseInfo item = ReefStatusSettings.Instance.FirstControler.GetItem((string)selectedItem.Tag);
            item.Display = true;
            item.Range = GraphRange.Week;
            UpdateGraph(item);
        }

        /// <summary>
        /// Handles the Click event of the monthToolStripMenuItem control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void MonthToolStripMenuItem_Click(object sender, EventArgs e)
        {
            BaseInfo item = ReefStatusSettings.Instance.FirstControler.GetItem((string)selectedItem.Tag);
            item.Display = true;
            item.Range = GraphRange.Month;
            UpdateGraph(item);
        }

        /// <summary>
        /// Handles the Click event of the yearToolStripMenuItem control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void YearToolStripMenuItem_Click(object sender, EventArgs e)
        {
            BaseInfo item = ReefStatusSettings.Instance.FirstControler.GetItem((string)selectedItem.Tag);
            item.Display = true;
            item.Range = GraphRange.Year;
            UpdateGraph(item);
        }

        /// <summary>
        /// Handles the Click event of the noneToolStripMenuItem control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void NoneToolStripMenuItem_Click(object sender, EventArgs e)
        {
            BaseInfo item = ReefStatusSettings.Instance.FirstControler.GetItem((string)selectedItem.Tag);
            item.Display = false;
            UpdateGraph(item);
        }

        private void UpdateCustomGraph()
        {
            if (customGraph.Visible)
            {
                ProgressWindow progress = new ProgressWindow { Text = "Updating" };
                progress.ShowCancelButton(false);
                new Thread(
                    () =>
                        {
                            try
                            {
                                using (IDataAccess data = ReefStatusSettings.Instance.Logging.Connection.Create())
                                {
                                    progress.Begin(
                                        0, ReefStatusSettings.Instance.CustomGraph.ShowCurves.Count, string.Empty);

                                    Dictionary<string, Collection<Common.ProfiLux.DataPoint>> dataPoints =
                                        new Dictionary<string, Collection<DataPoint>>();

                                    foreach (string graphName in ReefStatusSettings.Instance.CustomGraph.ShowCurves)
                                    {
                                        Controler controler = ReefStatusSettings.Instance.FirstControler;
                                        BaseInfo item = controler.GetItem(graphName);
                                        progress.SetText(
                                            Language.GetResource("strUpdateGraph") + " " + item.DisplayName);
                                        Collection<Common.ProfiLux.DataPoint> points =
                                            new Collection<Common.ProfiLux.DataPoint>();
                                        switch (ReefStatusSettings.Instance.CustomGraph.Range)
                                        {
                                            case GraphRange.All:
                                                points = data.GetDataPoints(item.Id, false, controler.Id);
                                                break;
                                            case GraphRange.Year:
                                                points = data.GetDataPoints(
                                                    item.Id, DateTime.Now.AddYears(-1), false, controler.Id);
                                                break;
                                            case GraphRange.Month:
                                                points = data.GetDataPoints(
                                                    item.Id, DateTime.Now.AddMonths(-1), false, controler.Id);
                                                break;
                                            case GraphRange.Week:
                                                points = data.GetDataPoints(
                                                    item.Id, DateTime.Now.AddDays(-7), false, controler.Id);
                                                break;
                                            case GraphRange.Day:
                                                points = data.GetDataPoints(
                                                    item.Id, DateTime.Now.AddDays(-1), false, controler.Id);
                                                break;
                                        }

                                        dataPoints.Add(item.Id, points);
                                        progress.StepTo(1);
                                    }

                                    BeginInvoke(
                                        new Action(
                                            () =>
                                            customGraph.AddPoints(
                                                dataPoints, ReefStatusSettings.Instance.FirstControler)));
                                }
                            }
                            catch (ReefStatusException ex)
                            {
                                Logger.Instance.LogError(ex);
                            }
                            catch (System.Threading.ThreadAbortException)
                            {
                                // We want to exit gracefully here (if we're lucky)
                            }
                            catch (System.Threading.ThreadInterruptedException)
                            {
                                // And here, if we can
                            }
                            finally
                            {
                                progress.End();
                            }
                        }).Start();
                progress.ShowDialog();
            }
        }


        /// <summary>
        /// Updates the graph.
        /// </summary>
        /// <param name="item">The item.</param>
        private void UpdateGraph(BaseInfo item)
        {
            ProgressWindow progress = new ProgressWindow { Text = "Updating" };
            progress.ShowCancelButton(false);
            new Thread(
                () =>
                    {
                        try
                        {
                            using (IDataAccess data = ReefStatusSettings.Instance.Logging.Connection.Create())
                            {
                                progress.Begin(0, 100, string.Empty);
                                if (item.Display)
                                {
                                    progress.SetText(Language.GetResource("strUpdateGraph") + " " + item.DisplayName);
                                    Collection<Common.ProfiLux.DataPoint> points =
                                        new Collection<Common.ProfiLux.DataPoint>();
                                    switch (item.Range)
                                    {
                                        case GraphRange.All:
                                            points = data.GetDataPoints(item.Id, false, item.Controler.Id);
                                            break;
                                        case GraphRange.Year:
                                            points = data.GetDataPoints(
                                                item.Id, DateTime.Now.AddYears(-1), false, item.Controler.Id);
                                            break;
                                        case GraphRange.Month:
                                            points = data.GetDataPoints(
                                                item.Id, DateTime.Now.AddMonths(-1), false, item.Controler.Id);
                                            break;
                                        case GraphRange.Week:
                                            points = data.GetDataPoints(
                                                item.Id, DateTime.Now.AddDays(-7), false, item.Controler.Id);
                                            break;
                                        case GraphRange.Day:
                                            points = data.GetDataPoints(
                                                item.Id, DateTime.Now.AddDays(-1), false, item.Controler.Id);
                                            break;
                                    }

                                    BeginInvoke(new Action(() => this.AddItemsToGraph(item, points)));
                                }
                                else
                                {
                                    BeginInvoke(new Action(() => this.CloseGraph(item)));
                                }

                                progress.StepTo(100);
                            }
                        }
                        catch (ReefStatusException ex)
                        {
                            Logger.Instance.LogError(ex);
                        }
                        finally
                        {
                            progress.End();
                        }
                    }).Start();

            progress.ShowDialog();

            if (ReefStatusSettings.Instance.CustomGraph.ShowCurves.Contains(item.Id))
            {
                UpdateCustomGraph();
            }
        }


        /// <summary>
        /// Handles the DoubleClick event of the treeView1 control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void ControlDoubleClick(object sender, EventArgs e)
        {
            ListView listView = sender as ListView;
            if (listView != null)
            {
                if (listView.SelectedItems[0].Tag != null)
                {
                    string name = (string)listView.SelectedItems[0].Tag;
                    BaseInfo item = ReefStatusSettings.Instance.FirstControler.GetItem(name);
                    if (item != null)
                    {

                        GraphForm graph = this.graphs.FirstOrDefault(form => form.Settings == item);
                        if (graph == null)
                        {
                            item.Display = true;
                            graph = CreateGraph(item, true);
                        }

                        graph.WindowState = FormWindowState.Maximized;
                    }
                }
            }
        }

        /// <summary>
        /// Handles the MouseDown event of the treeView1 control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.Forms.MouseEventArgs"/> instance containing the event data.</param>
        private void ControlMouseDown(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Right)
            {
                ListView listView = sender as ListView;
                if (listView != null)
                {
                    ListViewItem item = listView.GetItemAt(e.Location.X, e.Location.Y);
                    if (item != null && item.Tag != null)
                    {
                        selectedItem = (ReefStatusItem)item;
                        UpdateAndShowMenu(listView.PointToScreen(e.Location));
                    }
                }
                else if (sender == userItem.Header)
                {
                    userDataMenu.Show(userItem.Header.PointToScreen(e.Location));
                }
            }
        }

        /// <summary>
        /// Handles the Click event of the settingsToolStripMenuItem1 control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void SettingsToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            BaseInfo item = selectedItem.Settings;
            DataSettingsForm dialog = new DataSettingsForm
                                          {
                                              color = { BackColor = item.Colour },
                                              displayName = { Text = item.DisplayName },
                                              units = { Text = item.Units },
                                              DefaultName = selectedItem.Settings.Id,
                                              DefaultUnits = selectedItem.Settings.Units,
                                              Text = "Settings for " + selectedItem.Settings.Id
                                          };

            if (dialog.ShowDialog() == DialogResult.OK)
            {
                item.Colour = dialog.color.BackColor;
                item.DisplayName = dialog.displayName.Text;
                item.Units = dialog.units.Text;

                selectedItem.Text = item.DisplayName;
                selectedItem.SubItems[2].Text = item.ValueWithUnits;

                UpdateGraph(item);
            }
        }

        /// <summary>
        /// Handles the Click event of the addDataItemToolStripMenuItem control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void AddDataItemToolStripMenuItem_Click(object sender, EventArgs e)
        {
            int userIndex = 1;

            foreach (UserInfo item in ReefStatusSettings.Instance.FirstControler.UserValues)
            {
                userIndex = Math.Max(int.Parse(item.Id.Substring(3), CultureInfo.InvariantCulture) + 1, userIndex);
            }

            DataSettingsForm dialog = new DataSettingsForm
                                          {
                                              color = { BackColor = Color.Red },
                                              displayName = { Text = "USR" + userIndex },
                                              units = { Text = string.Empty },
                                              DefaultName = "USR" + userIndex,
                                              DefaultUnits = string.Empty,
                                              Text = "Add User Setting"
                                          };

            if (dialog.ShowDialog() == DialogResult.OK)
            {
                BaseInfo settings = new UserInfo
                                        {
                                            Id = "USR" + userIndex,
                                            Colour = dialog.color.BackColor,
                                            DisplayName = dialog.displayName.Text,
                                            Units = dialog.units.Text
                                        };
                ReefStatusSettings.Instance.FirstControler.Items.Add(settings);
                UpdateUserDataDisplay();
            }
        }

        /// <summary>
        /// Handles the Click event of the updateValueToolStripMenuItem control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void UpdateValueToolStripMenuItem_Click(object sender, EventArgs e)
        {
            UpdateDataForm dialog = new UpdateDataForm
                                        {
                                            Text =
                                                "Update Value of " +
                                                selectedItem.Settings.DisplayName,
                                            value = { Value = (decimal)((double)selectedItem.Settings.Value) },
                                            time = { Value = DateTime.Now },
                                            date = { Value = DateTime.Now },
                                            units = { Text = selectedItem.Settings.Units }
                                        };

            if (dialog.ShowDialog() == DialogResult.OK)
            {
                double value = (double)dialog.value.Value;
                DateTime valueTime = new DateTime(dialog.date.Value.Year, dialog.date.Value.Month, dialog.date.Value.Day, dialog.time.Value.Hour, dialog.time.Value.Minute, dialog.time.Value.Second);
                UserInfo userValue = (UserInfo)selectedItem.Settings;
                if (userValue.Time < valueTime)
                {
                    userValue.Value = value;
                    userValue.Time = valueTime;
                    UpdateUserDataDisplay();
                }

                try
                {
                    using (IDataAccess access = ReefStatusSettings.Instance.Logging.Connection.Create())
                    {
                        access.InsertItem(value, valueTime, selectedItem.Settings.Id, false, selectedItem.Settings.Controler.Id);
                    }
                }
                catch (ReefStatusException ex)
                {
                    Logger.Instance.LogError(ex);
                }

                UpdateGraph(selectedItem.Settings);
            }
        }

        /// <summary>
        /// Handles the Click event of the deleteToolStripMenuItem control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void DeleteToolStripMenuItem_Click(object sender, EventArgs e)
        {
            selectedItem.Settings.Display = false;
            UpdateGraph(selectedItem.Settings);
            ReefStatusSettings.Instance.FirstControler.Items.Remove(selectedItem.Settings);
            UpdateUserDataDisplay();
        }

        /// <summary>
        /// Handles the Click event of the DataLogToolStripMenuItem control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void DataLogToolStripMenuItem_Click(object sender, EventArgs e)
        {
            dataTableLayout.Visible = !dataTableLayout.Visible;
            dataLogToolStripMenuItem.Checked = dataTableLayout.Visible;
        }

        /// <summary>
        /// Handles the Click event of the ShowToolStripMenuItem control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void ShowToolStripMenuItem_Click(object sender, EventArgs e)
        {
            customGraph.Visible = !customGraph.Visible;
            showToolStripMenuItem.Checked = customGraph.Visible;
            UpdateCustomGraph();
        }

        /// <summary>
        /// Handles the Click event of the SettingsToolStripMenuItem2 control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void SettingsToolStripMenuItem2_Click(object sender, EventArgs e)
        {
            CustomGraphSettings dialog = new CustomGraphSettings(ReefStatusSettings.Instance);
            if (dialog.ShowDialog() == DialogResult.OK)
            {
                ReefStatusSettings.Instance.CustomGraph.ShowCurves = dialog.SelectedCurves;
                ReefStatusSettings.Instance.CustomGraph.Range = dialog.Range;
                UpdateCustomGraph();
            }
        }

        /// <summary>
        /// Handles the Click event of the customGraphToolStripMenuItem1 control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void CustomGraphToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            string type = (string)selectedItem.Tag;
            if (ReefStatusSettings.Instance.CustomGraph.ShowCurves.Contains(type))
            {
                ReefStatusSettings.Instance.CustomGraph.ShowCurves.Remove(type);
            }
            else
            {
                ReefStatusSettings.Instance.CustomGraph.ShowCurves.Add(type);
            }

            if (!customGraph.Visible)
            {
                customGraph.Visible = true;
                showToolStripMenuItem.Checked = customGraph.Visible;
            }

            UpdateCustomGraph();
        }

        /// <summary>
        /// Reef Status List view item
        /// </summary>
        private class ReefStatusItem : ListViewItem
        {
            /// <summary>
            /// Initializes a new instance of the <see cref="ReefStatusItem"/> class.
            /// </summary>
            /// <param name="settings">The settings.</param>
            public ReefStatusItem(BaseInfo settings)
            {
                Settings = settings;
                Text = settings.DisplayName;
                Tag = settings.Id;
            }

            /// <summary>
            /// Gets the settings.
            /// </summary>
            /// <value>The settings.</value>
            public BaseInfo Settings { get; private set; }
        }
    }
}