﻿// --------------------------------------------------------------------------------------------------------------------
// <copyright file="Controler.cs" company="RedpointGames">
//   2010
// </copyright>
// <summary>
//   Controler Class
// </summary>
// --------------------------------------------------------------------------------------------------------------------

namespace RedPoint.ReefStatus.Common.ProfiLux
{
    using System;
    using System.Collections.ObjectModel;
    using System.Globalization;
    using System.Linq;
    using System.Windows.Data;
    using System.Windows.Forms;
    using System.Windows.Input;
    using System.Windows.Threading;
    using System.Xml.Serialization;

    using RedPoint.ReefStatus.Common.Commands;
    using RedPoint.ReefStatus.Common.Communication;
    using RedPoint.ReefStatus.Common.Core;
    using RedPoint.ReefStatus.Common.Database;
    using RedPoint.ReefStatus.Common.Settings;
    using RedPoint.ReefStatus.Common.UI.ViewModel;
    using RedPoint.ReefStatus.Common.ViewModel;

    /// <summary>
    /// Controler Class
    /// </summary>
    public class Controler : ViewModelBase
    {
        #region Constants and Fields

        /// <summary>
        /// The timer.
        /// </summary>
        private readonly Timer timer;

        /// <summary>
        /// The timer.
        /// </summary>
        private readonly Timer lightTestTimer;

        /// <summary>
        /// The add user value command.
        /// </summary>
        private ICommand addUserValueCommand;

        /// <summary>
        /// The all lights off command.
        /// </summary>
        private ICommand allLightsOffCommand;

        /// <summary>
        /// The all lights on command.
        /// </summary>
        private ICommand allLightsOnCommand;

        /// <summary>
        /// The clear level alarm command.
        /// </summary>
        private ICommand clearLevelAlarmCommand;

        /// <summary>
        /// The command thread
        /// </summary>
        private CommandThread commands;

        /// <summary>
        /// The content changed.
        /// </summary>
        private bool contentChanged = true;

        /// <summary>
        /// The current selection.
        /// </summary>
        private object currentSelection;

        /// <summary>
        /// Feed Command
        /// </summary>
        private ICommand feedCommand;

        /// <summary>
        /// Id of the controler
        /// </summary>
        private int id;

        /// <summary>
        /// The last selection.
        /// </summary>
        private object lastSelection;

        /// <summary>
        /// The light test command.
        /// </summary>
        private ICommand lightTestCommand;

        /// <summary>
        /// mainteance Command
        /// </summary>
        private ICommand maintenanceCommand;

        /// <summary>
        /// Manual Command
        /// </summary>
        private ICommand manualCommand;

        /// <summary>
        /// Manual Illumination Command
        /// </summary>
        private ICommand manualIlluminationCommand;

        /// <summary>
        /// Name of the controler
        /// </summary>
        private string name;

        /// <summary>
        /// The new dosing command.
        /// </summary>
        private ICommand newDosingCommand;

        /// <summary>
        /// The new dosing per day.
        /// </summary>
        private int newDosingPerDay;

        /// <summary>
        /// The new dosing rate.
        /// </summary>
        private int newDosingRate;

        /// <summary>
        /// The new user value.
        /// </summary>
        private double newUserValue;

        /// <summary>
        /// The new user value date.
        /// </summary>
        private DateTime newUserValueDate;

        /// <summary>
        /// Command to reset the hours
        /// </summary>
        private ICommand resetOperationalHoursCommand;

        /// <summary>
        /// The show connection options.
        /// </summary>
        private bool showConnectionOptions;

        /// <summary>
        /// The start water change command.
        /// </summary>
        private ICommand startWaterChangeCommand;

        /// <summary>
        /// The update dosing command.
        /// </summary>
        private ICommand updateDosingCommand;

        #endregion

        #region Constructors and Destructors

        private Dispatcher dispatcher;

        /// <summary>
        /// Initializes a new instance of the <see cref="Controler"/> class.
        /// </summary>
        public Controler()
        {
            this.dispatcher = Dispatcher.CurrentDispatcher;
            this.Connection = new ConnectionSettings();
            this.Info = new Info();
            this.Items = new SafeObservableCollection<BaseInfo>();
            this.Probes = new ListCollectionView(this.Items) { Filter = obj => obj is Probe };
            this.LevelSensors = new ListCollectionView(this.Items) { Filter = obj => obj is LevelSensor };
            this.SPorts = new ListCollectionView(this.Items) { Filter = obj => obj is SPort };
            this.LPorts = new ListCollectionView(this.Items) { Filter = obj => obj is LPort };
            this.UserValues = new ListCollectionView(this.Items) { Filter = obj => obj is UserInfo };
            this.DigitalInputs = new ListCollectionView(this.Items) { Filter = obj => obj is DigitalInput };
            this.Lights = new ListCollectionView(this.Items) { Filter = obj => obj is Light };
            this.DosingPumps = new ListCollectionView(this.Items) { Filter = obj => obj is DosingPump };
            this.CurrentPumps = new ListCollectionView(this.Items) { Filter = obj => obj is CurrentPump };
            this.Remote = new RemoteViewModel(this);

            this.UpdateCommand = new DelegateCommand(
                this.UpdateItem, () => (this.CurrentSelection != null && this.CurrentSelection is UserInfo));
            this.DeleteCommand = new DelegateCommand<BaseInfo>(this.DeleteItem, arg => (arg != null && arg is UserInfo));
            this.AddDataItemCommand = new DelegateCommand(this.AddDataItem);

            this.timer = new Timer();
            this.timer.Tick += this.TimerTick;
            this.timer.Interval = 1000;
            this.timer.Start();

            this.lightTestTimer = new Timer();
            this.lightTestTimer.Tick += this.lightTestTimerTick;
            this.lightTestTimer.Interval = 33;
        }

        void lightTestTimerTick(object sender, EventArgs e)
        {
            if (this.Info.OperationMode != OperationMode.LightTest)
            {
                this.IsPlayingLightTest = false;
            }

            if (!this.IsPlayingLightTest)
            {
                this.lightTestTimer.Stop();
                return;
            }

            var value = this.Info.LightTestTimeValue + 1;
            if (value > 1439)
            {
                this.StopLightTest();
            }

            this.Info.LightTestTimeValue = value;
            this.stopLightTestCommand.RaiseCanExecuteChanged();
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets AddDataItemCommand.
        /// </summary>
        [XmlIgnore]
        public ICommand AddDataItemCommand { get; private set; }

        /// <summary>
        /// Gets the add user value command.
        /// </summary>
        /// <value>The add user value command.</value>
        [XmlIgnore]
        public ICommand AddUserValueCommand
        {
            get
            {
                return this.addUserValueCommand ?? (this.addUserValueCommand = new DelegateCommand(this.AddUserValue));
            }
        }

        /// <summary>
        /// Gets AllLightsOffCommand.
        /// </summary>
        [XmlIgnore]
        public ICommand AllLightsOffCommand
        {
            get
            {
                return this.allLightsOffCommand ??
                       (this.allLightsOffCommand = new DelegateCommand(() => this.SetAllLights(false)));
            }
        }

        /// <summary>
        /// Gets AllLightsOnCommand.
        /// </summary>
        [XmlIgnore]
        public ICommand AllLightsOnCommand
        {
            get
            {
                return this.allLightsOnCommand ??
                       (this.allLightsOnCommand = new DelegateCommand(() => this.SetAllLights(true)));
            }
        }

        /// <summary>
        /// Gets ClearLevelAlarmCommand.
        /// </summary>
        [XmlIgnore]
        public ICommand ClearLevelAlarmCommand
        {
            get
            {
                return this.clearLevelAlarmCommand ??
                       (this.clearLevelAlarmCommand =
                        new DelegateCommand<BaseInfo>(
                            this.ClearLevelAlarm, 
                            arg =>
                            (arg != null && arg is LevelSensor && ((LevelSensor)arg).IsAlarmOn == CurrentState.On)));
            }
        }

        [XmlIgnore]
        public ICommand StopLightTestCommand
        {
            get
            {
                return this.stopLightTestCommand ??
                      (this.stopLightTestCommand =
                       new DelegateCommand(
                           this.StopLightTest,
                           () => this.Info.LightTestTime != 0));
            }
        }

        private void StopLightTest()
        {
            this.IsPlayingLightTest = false;
            this.Info.LightTestTimeValue = 0;
            this.lightTestTimer.Stop();
            stopLightTestCommand.RaiseCanExecuteChanged();
        }

        [XmlIgnore]
        public ICommand PlayLightTestCommand
        {
            get
            {
                return this.playLightTestCommand ??
                      (this.playLightTestCommand =
                       new DelegateCommand(
                           this.PlayLightTest));
            }
        }

        private void PlayLightTest()
        {
            if (this.IsPlayingLightTest)
            {
                this.IsPlayingLightTest = false;
                this.lightTestTimer.Stop();
            }
            else
            {
                this.IsPlayingLightTest = true;
                this.lightTestTimer.Start();
            }
        }

        private bool isPlayingLightTest;

        private DelegateCommand stopLightTestCommand;

        private ICommand playLightTestCommand;

        [XmlIgnore]
        public bool IsPlayingLightTest
        {
            get
            {
                return this.isPlayingLightTest;
            }
            set
            {
                if (this.isPlayingLightTest == value)
                {
                    return;
                }

                this.isPlayingLightTest = value;
                this.OnPropertyChanged("IsPlayingLightTest");
            }
        }

        /// <summary>
        /// Gets the commands.
        /// </summary>
        /// <value>The commands.</value>
        [XmlIgnore]
        public CommandThread Commands
        {
            get
            {
                return this.commands ?? (this.commands = new CommandThread(DataService.Instance, this));
            }
        }

        /// <summary>
        /// Gets or sets the connection.
        /// </summary>
        /// <value>The connection.</value>
        public ConnectionSettings Connection { get; set; }

        /// <summary>
        /// Gets or sets a value indicating whether [content changed].
        /// </summary>
        /// <value>
        ///   <c>true</c> if [content changed]; otherwise, <c>false</c>.
        /// </value>
        [XmlIgnore]
        public bool ContentChanged
        {
            get
            {
                return this.contentChanged;
            }

            set
            {
                if (this.contentChanged == value)
                {
                    return;
                }
                this.contentChanged = value;
                this.OnPropertyChanged("ContentChanged");
            }
        }

        /// <summary>
        /// Gets or sets CurrentPumpSelection.
        /// </summary>
        [XmlIgnore]
        public BaseInfo CurrentPumpSelection
        {
            get
            {
                return this.CurrentSelection is CurrentPump ? (BaseInfo)this.CurrentSelection : null;
            }

            set
            {
                if (this.CurrentSelection is CurrentPump && value == null)
                {
                    this.CurrentSelection = value;
                }
                else if (value != null)
                {
                    this.CurrentSelection = value;
                }
            }
        }

        /// <summary>
        /// Gets the current pumps.
        /// </summary>
        /// <value>The current pumps.</value>
        [XmlIgnore]
        public ListCollectionView CurrentPumps { get; private set; }

        /// <summary>
        /// Gets or sets CurrentSelection.
        /// </summary>
        [XmlIgnore]
        public object CurrentSelection
        {
            get
            {
                return this.currentSelection;
            }

            set
            {
                if (this.currentSelection != value)
                {
                    this.LastSelection = this.currentSelection;
                    this.currentSelection = value;
                    dispatcher.BeginInvoke(
                        new Action(
                            () =>
                                {
                                    this.OnPropertyChanged("CurrentSelection");
                                    this.OnPropertyChanged("ProbeSelection");
                                    this.OnPropertyChanged("LevelSelection");
                                    this.OnPropertyChanged("DigitalInputSelection");
                                    this.OnPropertyChanged("SPortSelection");
                                    this.OnPropertyChanged("LPortSelection");
                                    this.OnPropertyChanged("UserValueSelection");
                                    this.OnPropertyChanged("LightSelection");
                                    this.OnPropertyChanged("DosingPumpSelection");
                                    this.OnPropertyChanged("CurrentPumpSelection");
                                    this.OnPropertyChanged("ReminderSelection");
                                }));
                }
            }
        }

        /// <summary>
        /// Gets DeleteCommand.
        /// </summary>
        [XmlIgnore]
        public ICommand DeleteCommand { get; private set; }

        /// <summary>
        /// Gets or sets the digital input selection.
        /// </summary>
        /// <value>The digital input selection.</value>
        [XmlIgnore]
        public BaseInfo DigitalInputSelection
        {
            get
            {
                return this.CurrentSelection is DigitalInput ? (BaseInfo)this.CurrentSelection : null;
            }

            set
            {
                if (this.CurrentSelection is DigitalInput && value == null)
                {
                    this.CurrentSelection = value;
                }
                else if (value != null)
                {
                    this.CurrentSelection = value;
                }
            }
        }

        /// <summary>
        /// Gets the sensors.
        /// </summary>
        /// <value>The sensors.</value>
        [XmlIgnore]
        public ListCollectionView DigitalInputs { get; private set; }

        /// <summary>
        /// Gets or sets the dosing pump selection.
        /// </summary>
        /// <value>The dosing pump selection.</value>
        [XmlIgnore]
        public BaseInfo DosingPumpSelection
        {
            get
            {
                return this.CurrentSelection is DosingPump ? (BaseInfo)this.CurrentSelection : null;
            }

            set
            {
                if (this.CurrentSelection is DosingPump && value == null)
                {
                    this.CurrentSelection = value;
                }
                else if (value != null)
                {
                    this.CurrentSelection = value;
                }
            }
        }

        /// <summary>
        /// Gets the dosing pumps.
        /// </summary>
        /// <value>The dosing pumps.</value>
        [XmlIgnore]
        public ListCollectionView DosingPumps { get; private set; }

        /// <summary>
        /// Gets the feed command.
        /// </summary>
        /// <value>The feed command.</value>
        [XmlIgnore]
        public ICommand FeedCommand
        {
            get
            {
                return this.feedCommand ?? (this.feedCommand = new DelegateCommand(this.Feed));
            }
        }

        /// <summary>
        /// Gets or sets the id.
        /// </summary>
        /// <value>The id.</value>
        public int Id
        {
            get
            {
                return this.id;
            }

            set
            {
                this.id = value;
                if (string.IsNullOrEmpty(this.Name))
                {
                    this.Name = "Controler" + this.id;
                }
            }
        }

        /// <summary>
        /// Gets or sets the controler info.
        /// </summary>
        /// <value>The info.</value>
        public Info Info { get; set; }

        /// <summary>
        /// Gets or sets the items.
        /// </summary>
        /// <value>The items.</value>
        public SafeObservableCollection<BaseInfo> Items { get; set; }

        /// <summary>
        /// Gets or sets the L port selection.
        /// </summary>
        /// <value>The L port selection.</value>
        [XmlIgnore]
        public BaseInfo LPortSelection
        {
            get
            {
                return this.CurrentSelection is LPort ? (BaseInfo)this.CurrentSelection : null;
            }

            set
            {
                if (this.CurrentSelection is LPort && value == null)
                {
                    this.CurrentSelection = value;
                }
                else if (value != null)
                {
                    this.CurrentSelection = value;
                }
            }
        }

        /// <summary>
        /// Gets the L ports.
        /// </summary>
        /// <value>The L ports.</value>
        [XmlIgnore]
        public ListCollectionView LPorts { get; private set; }

        /// <summary>
        /// Gets or sets CurrentSelection.
        /// </summary>
        [XmlIgnore]
        public object LastSelection
        {
            get
            {
                return this.lastSelection;
            }

            set
            {
                if (this.lastSelection != value)
                {
                    this.lastSelection = value;
                    this.OnPropertyChanged("LastSelection");
                    this.ContentChanged = false;
                    this.ContentChanged = true;
                }
            }
        }

        /// <summary>
        /// Gets a value indicating whether LevelIsAlarmOn.
        /// </summary>
        [XmlIgnore]
        public bool LevelIsAlarmOn
        {
            get
            {
                return this.LevelSensors.Cast<LevelSensor>().Any(sensor => sensor.IsAlarmOn == CurrentState.On);
            }
        }

        /// <summary>
        /// Gets or sets the level selection.
        /// </summary>
        /// <value>The level selection.</value>
        [XmlIgnore]
        public BaseInfo LevelSelection
        {
            get
            {
                return this.CurrentSelection is LevelSensor ? (BaseInfo)this.CurrentSelection : null;
            }

            set
            {
                if (this.CurrentSelection is LevelSensor && value == null)
                {
                    this.CurrentSelection = value;
                }
                else if (value != null)
                {
                    this.CurrentSelection = value;
                }
            }
        }

        /// <summary>
        /// Gets the level sensors.
        /// </summary>
        /// <value>The level sensors.</value>
        [XmlIgnore]
        public ListCollectionView LevelSensors { get; private set; }

        /// <summary>
        /// Gets a value indicating whether LightIsOverMaxOperationHours.
        /// </summary>
        [XmlIgnore]
        public bool LightIsOverMaxOperationHours
        {
            get
            {
                return this.Lights.Cast<Light>().Any(sensor => sensor.IsOverMaxOperationHours);
            }
        }

        /// <summary>
        /// Gets or sets the light selection.
        /// </summary>
        /// <value>The light selection.</value>
        [XmlIgnore]
        public BaseInfo LightSelection
        {
            get
            {
                return this.CurrentSelection is Light ? (BaseInfo)this.CurrentSelection : null;
            }

            set
            {
                if (this.CurrentSelection is Light && value == null)
                {
                    this.CurrentSelection = value;
                }
                else if (value != null)
                {
                    this.CurrentSelection = value;
                }
            }
        }

        /// <summary>
        /// Gets LightTestCommand.
        /// </summary>
        [XmlIgnore]
        public ICommand LightTestCommand
        {
            get
            {
                return this.lightTestCommand ?? (this.lightTestCommand = new DelegateCommand(this.LightTest));
            }
        }

        /// <summary>
        /// Gets the lights.
        /// </summary>
        /// <value>The lights.</value>
        [XmlIgnore]
        public ListCollectionView Lights { get; private set; }

        /// <summary>
        /// Gets the maintenance command.
        /// </summary>
        /// <value>The maintenance command.</value>
        [XmlIgnore]
        public ICommand MaintenanceCommand
        {
            get
            {
                return this.maintenanceCommand ??
                       (this.maintenanceCommand = new DelegateCommand<Maintenance>(this.Maintenance));
            }
        }

        /// <summary>
        /// Gets the manual command.
        /// </summary>
        /// <value>The manual command.</value>
        [XmlIgnore]
        public ICommand ManualCommand
        {
            get
            {
                return this.manualCommand ?? (this.manualCommand = new DelegateCommand(this.ManualSockets));
            }
        }

        /// <summary>
        /// Gets the manual illumination command.
        /// </summary>
        /// <value>The manual illumination command.</value>
        [XmlIgnore]
        public ICommand ManualIlluminationCommand
        {
            get
            {
                return this.manualIlluminationCommand ??
                       (this.manualIlluminationCommand = new DelegateCommand(this.ManualIllumination));
            }
        }

        /// <summary>
        /// Gets or sets the name.
        /// </summary>
        /// <value>The name.</value>
        public string Name
        {
            get
            {
                return this.name;
            }

            set
            {
                if (this.name != value)
                {
                    this.name = value;
                    this.OnPropertyChanged("Name");
                }
            }
        }

        /// <summary>
        /// Gets the new dosing command.
        /// </summary>
        /// <value>The new dosing command.</value>
        [XmlIgnore]
        public ICommand NewDosingCommand
        {
            get
            {
                return this.newDosingCommand ??
                       (this.newDosingCommand =
                        new DelegateCommand(
                            this.NewDosing, () => (this.CurrentSelection != null && this.CurrentSelection is DosingPump)));
            }
        }

        /// <summary>
        /// Gets or sets the new dosing per day.
        /// </summary>
        /// <value>The new dosing per day.</value>
        [XmlIgnore]
        public int NewDosingPerDay
        {
            get
            {
                return this.newDosingPerDay;
            }

            set
            {
                this.newDosingPerDay = value;
                this.OnPropertyChanged("NewDosingPerDay");
            }
        }

        /// <summary>
        /// Gets or sets the new dosing rate.
        /// </summary>
        /// <value>The new dosing rate.</value>
        [XmlIgnore]
        public int NewDosingRate
        {
            get
            {
                return this.newDosingRate;
            }

            set
            {
                this.newDosingRate = value;
                this.OnPropertyChanged("NewDosingRate");
            }
        }

        /// <summary>
        /// Gets or sets the new user value.
        /// </summary>
        /// <value>The new user value.</value>
        [XmlIgnore]
        public double NewUserValue
        {
            get
            {
                return this.newUserValue;
            }

            set
            {
                if (this.newUserValue != value)
                {
                    this.newUserValue = value;
                    this.OnPropertyChanged("NewUserValue");
                }
            }
        }

        /// <summary>
        /// Gets or sets the new user value date.
        /// </summary>
        /// <value>The new user value date.</value>
        [XmlIgnore]
        public DateTime NewUserValueDate
        {
            get
            {
                return this.newUserValueDate;
            }

            set
            {
                if (this.newUserValueDate != value)
                {
                    this.newUserValueDate = value;
                    this.OnPropertyChanged("NewUserValueDate");
                }
            }
        }

        /// <summary>
        /// Gets the sensors.
        /// </summary>
        /// <value>The sensors.</value>
        [XmlIgnore]
        public ReadOnlyCollection<Probe> ProbeCollection
        {
            get
            {
                var tempCollection = new Collection<Probe>();
                foreach (Probe item in this.Probes)
                {
                    tempCollection.Add(item);
                }

                return new ReadOnlyCollection<Probe>(tempCollection);
            }
        }

        /// <summary>
        /// Gets or sets the probe selection.
        /// </summary>
        /// <value>The probe selection.</value>
        [XmlIgnore]
        public BaseInfo ProbeSelection
        {
            get
            {
                return this.CurrentSelection is Probe ? (BaseInfo)this.CurrentSelection : null;
            }

            set
            {
                if (this.CurrentSelection is Probe && value == null)
                {
                    this.CurrentSelection = value;
                }
                else if (value != null)
                {
                    this.CurrentSelection = value;
                }
            }
        }

        /// <summary>
        /// Gets the sensors.
        /// </summary>
        /// <value>The sensors.</value>
        [XmlIgnore]
        public ListCollectionView Probes { get; private set; }

        /// <summary>
        /// Gets a value indicating whether ReminderIsOverdue.
        /// </summary>
        [XmlIgnore]
        public bool ReminderIsOverdue
        {
            get
            {
                return this.Info.Reminders.Any(reminder => reminder.IsOverdue);
            }
        }

        /// <summary>
        /// Gets or sets the reminder selection.
        /// </summary>
        /// <value>
        /// The reminder selection.
        /// </value>
        [XmlIgnore]
        public Reminder ReminderSelection
        {
            get
            {
                return this.CurrentSelection is Reminder ? (Reminder)this.CurrentSelection : null;
            }

            set
            {
                if (this.CurrentSelection is Reminder && value == null)
                {
                    this.CurrentSelection = value;
                }
                else if (value != null)
                {
                    this.CurrentSelection = value;
                }
            }
        }

        /// <summary>
        /// Gets or sets Remote.
        /// </summary> 
        [XmlIgnore]
        public RemoteViewModel Remote { get; set; }

        /// <summary>
        /// Gets the reset operational hours command.
        /// </summary>
        /// <value>The reset operational hours command.</value>
        [XmlIgnore]
        public ICommand ResetOperationalHoursCommand
        {
            get
            {
                return this.resetOperationalHoursCommand ??
                       (this.resetOperationalHoursCommand =
                        new DelegateCommand<BaseInfo>(
                            this.ResetOperationalHours, arg => (arg != null && (arg is Probe || arg is Light))));
            }
        }

        /// <summary>
        /// Gets or sets the S port selection.
        /// </summary>
        /// <value>The S port selection.</value>
        [XmlIgnore]
        public BaseInfo SPortSelection
        {
            get
            {
                return this.CurrentSelection is SPort ? (BaseInfo)this.CurrentSelection : null;
            }

            set
            {
                if (this.CurrentSelection is SPort && value == null)
                {
                    this.CurrentSelection = value;
                }
                else if (value != null)
                {
                    this.CurrentSelection = value;
                }
            }
        }

        /// <summary>
        /// Gets the S ports.
        /// </summary>
        /// <value>The S ports.</value>
        [XmlIgnore]
        public ListCollectionView SPorts { get; private set; }

        /// <summary>
        /// Gets a value indicating whether SensorIsAlarmOn.
        /// </summary>
        [XmlIgnore]
        public bool SensorIsAlarmOn
        {
            get
            {
                return this.Probes.Cast<Probe>().Any(probe => probe.IsAlarmOn == CurrentState.On);
            }
        }

        /// <summary>
        /// Gets a value indicating whether SensorIsOverMaxOperationHours.
        /// </summary>
        [XmlIgnore]
        public bool SensorIsOverMaxOperationHours
        {
            get
            {
                return this.Probes.Cast<Probe>().Any(probe => probe.IsOverMaxOperationHours);
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether [sent alarm email].
        /// </summary>
        /// <value><c>true</c> if [sent alarm email]; otherwise, <c>false</c>.</value>
        [XmlIgnore]
        public bool SentAlarmEmail { get; set; }

        /// <summary>
        /// Gets or sets a value indicating whether ShowConnectionOptions.
        /// </summary>
        [XmlIgnore]
        public bool ShowConnectionOptions
        {
            get
            {
                return this.showConnectionOptions;
            }

            set
            {
                if (this.showConnectionOptions != value)
                {
                    this.showConnectionOptions = value;
                    this.OnPropertyChanged("ShowConnectionOptions");
                }
            }
        }

        /// <summary>
        /// Gets the new dosing command.
        /// </summary>
        /// <value>The new dosing command.</value>
        [XmlIgnore]
        public ICommand StartWaterChangeCommand
        {
            get
            {
                return this.startWaterChangeCommand ??
                       (this.startWaterChangeCommand =
                        new DelegateCommand<BaseInfo>(
                            this.StartWaterChange, 
                            arg => (arg != null && arg is LevelSensor && ((LevelSensor)arg).CanDoWaterChange)));
            }
        }

        /// <summary>
        /// Gets UpdateCommand.
        /// </summary>
        [XmlIgnore]
        public ICommand UpdateCommand { get; private set; }

        /// <summary>
        /// Gets UpdateDosingCommand.
        /// </summary>
        [XmlIgnore]
        public ICommand UpdateDosingCommand
        {
            get
            {
                return this.updateDosingCommand ??
                       (this.updateDosingCommand =
                        new DelegateCommand(
                            this.UpdateDosing, 
                            () => (this.CurrentSelection != null && this.CurrentSelection is DosingPump)));
            }
        }

        /// <summary>
        /// Gets or sets the user value selection.
        /// </summary>
        /// <value>The user value selection.</value>
        [XmlIgnore]
        public BaseInfo UserValueSelection
        {
            get
            {
                return this.CurrentSelection is UserInfo ? (BaseInfo)this.CurrentSelection : null;
            }

            set
            {
                this.CurrentSelection = value;
            }
        }

        /// <summary>
        /// Gets the S ports.
        /// </summary>
        /// <value>The S ports.</value>
        [XmlIgnore]
        public ListCollectionView UserValues { get; private set; }

        #endregion

        #region Public Methods

        /// <summary>
        /// Changes the user values.
        /// </summary>
        public void ChangeUserValues()
        {
            this.OnPropertyChanged("UserValues");
        }

        /// <summary>
        /// Clears the level alarm.
        /// </summary>
        /// <param name="basicProtocol">
        /// The basic protocol.
        /// </param>
        /// <param name="levelSensor">
        /// The level sensor.
        /// </param>
        public void ClearLevelAlarm(IProfilux basicProtocol, LevelSensor levelSensor)
        {
            basicProtocol.ClearLevelAlarm(levelSensor.Index);
            this.UpdateLevelSensorsValue(basicProtocol, levelSensor);
            this.UpdateAlarm(basicProtocol.Alarm, null);
        }

        /// <summary>
        /// Feeds this instance.
        /// </summary>
        public void Feed()
        {
            this.Commands.SendFeed(true);
        }

        /// <summary>
        /// Gets the item.
        /// </summary>
        /// <param name="itemId">
        /// The id.
        /// </param>
        /// <returns>
        /// and item with the given id
        /// </returns>
        public BaseInfo GetItem(string itemId)
        {
            return this.Items.FirstOrDefault(p => p.Id == itemId);
        }

        /// <summary>
        /// Maintenances this instance.
        /// </summary>
        /// <param name="maintenance">
        /// The maintenance.
        /// </param>
        public void Maintenance(Maintenance maintenance)
        {
            this.Commands.SendMaintenance(!maintenance.IsActive, maintenance);
        }

        /// <summary>
        /// Resets the operational hours.
        /// </summary>
        /// <param name="basicProtocol">
        /// The basic protocol.
        /// </param>
        /// <param name="baseInfo">
        /// The base info.
        /// </param>
        public void ResetOperationalHours(IProfilux basicProtocol, BaseInfo baseInfo)
        {
            if (baseInfo is Light)
            {
                var light = (Light)baseInfo;
                basicProtocol.SetLightOperationHours(light.Channel, 0);
                this.UpdateLightOperationHours(basicProtocol, light);
            }
            else if (baseInfo is Probe)
            {
                var probe = (Probe)baseInfo;
                basicProtocol.SetProbeOperationHours(probe.Index, 0);
                UpdateProbeOperationalHours(basicProtocol, probe);
            }
        }

        /// <summary>
        /// Resets the reminder.
        /// </summary>
        /// <param name="basicProtocol">
        /// The basic protocol.
        /// </param>
        /// <param name="reminder">
        /// The reminder.
        /// </param>
        public void ResetReminder(IProfilux basicProtocol, Reminder reminder)
        {
            if (reminder.IsRepeating)
            {
                basicProtocol.ResetReminder(reminder.Index, reminder.Period);
            }
            else
            {
                basicProtocol.ClearReminder(reminder.Index);
            }

            this.UpdateReminders(basicProtocol, null);
        }

        /// <summary>
        /// Sets the state of the light.
        /// </summary>
        /// <param name="basicProtocol">
        /// The basic protocol.
        /// </param>
        /// <param name="light">
        /// The light.
        /// </param>
        /// <param name="value">
        /// The value.
        /// </param>
        public void SetLightState(IProfilux basicProtocol, Light light, double value)
        {
            basicProtocol.SetLightValue(light.Channel, value);
        }

        public void SetLightTestTime(IProfilux basicProtocol, int value)
        {
            basicProtocol.SetLightTestValue(value);
        }

        /// <summary>
        /// Sets the state of the socket.
        /// </summary>
        /// <param name="basicProtocol">
        /// The basic protocol.
        /// </param>
        /// <param name="port">
        /// The port.
        /// </param>
        /// <param name="value">
        /// if set to <c>true</c> [value].
        /// </param>
        public void SetSocketState(IProfilux basicProtocol, SPort port, bool value)
        {
            basicProtocol.SetSocketState(port.PortNumber, value);
        }

        /// <summary>
        /// Updates the alarm.
        /// </summary>
        /// <param name="alarmState">
        /// State of the alarm.
        /// </param>
        /// <param name="progress">
        /// The progress.
        /// </param>
        public void UpdateAlarm(CurrentState alarmState, IUpdateProgress progress)
        {
            if (progress != null)
            {
                progress.ProgressText = "Update Alarm State";
            }

            if (this.Info.Alarm != alarmState)
            {
                this.Info.Alarm = alarmState;
                Logger.Instance.Log(
                    new LogMessage(alarmState == CurrentState.On ? 1 : 0, "System Alarm is now " + alarmState));
            }
        }

        /// <summary>
        /// Updates all.
        /// </summary>
        /// <param name="protocol">
        /// The protocol.
        /// </param>
        /// <param name="progress">
        /// The progress.
        /// </param>
        /// <param name="skipNonEssntals">
        /// if set to <c>true</c> [skip non essntals].
        /// </param>
        /// <returns>
        /// True if all was updated
        /// </returns>
        public bool UpdateAll(IProfilux protocol, IUpdateProgress progress, bool skipNonEssntals)
        {
            if (progress != null)
            {
                progress.StopProcessing = false;
                progress.DisplayProgress = true;
                progress.ProgressText = "Update Started";
                double steps = protocol.ReminderCount + protocol.SensorCount + protocol.LevelSenosrCount +
                               protocol.LPortCount + protocol.SPortCount + protocol.DigitalInputCount +
                               protocol.TimerCount + protocol.LightCount;
                progress.SetProgressSteps(steps);
            }

            try
            {
                if (progress != null && progress.StopProcessing)
                {
                    return false;
                }

                this.UpdateInfo(protocol, progress);
                if (progress != null && progress.StopProcessing)
                {
                    return false;
                }

                this.UpdateProbes(protocol, progress, skipNonEssntals);
                if (progress != null && progress.StopProcessing)
                {
                    return false;
                }

                this.UpdateLevelSensors(protocol, progress);
                if (progress != null && progress.StopProcessing)
                {
                    return false;
                }

                this.UpdateSPorts(protocol, progress);
                if (progress != null && progress.StopProcessing)
                {
                    return false;
                }

                this.UpdateLPorts(protocol, progress);
                if (progress != null && progress.StopProcessing)
                {
                    return false;
                }

                this.UpdateDigitalInputs(protocol, progress);
                if (progress != null && progress.StopProcessing)
                {
                    return false;
                }

                this.UpdateDosingPumps(protocol, progress);
                if (progress != null && progress.StopProcessing)
                {
                    return false;
                }

                this.UpdateLights(protocol, progress, skipNonEssntals);
                if (progress != null && progress.StopProcessing)
                {
                    return false;
                }

                this.UpdateCurrentPumps(protocol, progress);
                if (progress != null && progress.StopProcessing)
                {
                    return false;
                }

                this.Info.LastUpdate = DateTime.Now;
            }
            finally
            {
                if (progress != null)
                {
                    progress.DisplayProgress = false;
                }
            }

            return true;
        }

        /// <summary>
        /// Updates the update lights values.
        /// </summary>
        /// <param name="basicProtocol">
        /// The basic protocol.
        /// </param>
        /// <param name="progress">
        /// The progress.
        /// </param>
        public void UpdateCurrentPumpValues(IProfilux basicProtocol, IUpdateProgress progress)
        {
            if (this.CurrentPumps.Count != 0)
            {
                foreach (CurrentPump pump in this.CurrentPumps)
                {
                    if (progress != null)
                    {
                        progress.IncrementProgress();
                        progress.ProgressText = "Update Current Pump " + pump.DisplayName;
                        if (progress.StopProcessing)
                        {
                            return;
                        }
                    }

                    pump.OldValue = pump.Value;
                    pump.Value = basicProtocol.GetCurrentPumpValue(pump.Index);
                }
            }
            else
            {
                this.UpdateLights(basicProtocol, progress, false);
            }
        }

        /// <summary>
        /// Updates the lights.
        /// </summary>
        /// <param name="basicProtocol">
        /// The basic protocol.
        /// </param>
        /// <param name="progress">
        /// The progress.
        /// </param>
        public void UpdateCurrentPumps(IProfilux basicProtocol, IUpdateProgress progress)
        {
            for (int i = 0; i < basicProtocol.CurrentPumpCount; i++)
            {
                if (progress != null)
                {
                    progress.IncrementProgress();
                    progress.ProgressText = "Getting Current Pump" + (i + 1) + " Information";
                    if (progress.StopProcessing)
                    {
                        return;
                    }
                }

                try
                {
                    int index = i;
                    var pump =
                        (CurrentPump)this.Items.FirstOrDefault(p => p is CurrentPump && ((CurrentPump)p).Index == index);

                    if (basicProtocol.IsCurrentPumpAssinged(i))
                    {
                        if (pump == null)
                        {
                            pump = new CurrentPump { Id = basicProtocol.GetCurrentPumpId(index), Index = index, };
                            this.Items.Add(pump);

                            this.OnPropertyChanged("CurrentPumps");
                        }

                        pump.OldValue = pump.Value;
                        pump.Value = basicProtocol.GetCurrentPumpValue(i);
                    }
                    else
                    {
                        if (pump != null)
                        {
                            this.Items.Remove(pump);
                            this.OnPropertyChanged("CurrentPumps");
                        }
                    }
                }
                catch (ErrorCodeException ex)
                {
                    Logger.Instance.Log(
                        new LogMessage(2002, string.Format("CurrentPump {0} not supported code {1}", i, ex.ErrorCode))
                            {
                               Exception = ex 
                            });
                }
            }
        }

        /// <summary>
        /// Updates the digital input values.
        /// </summary>
        /// <param name="basicProtocol">
        /// The basic protocol.
        /// </param>
        /// <param name="progress">
        /// The progress.
        /// </param>
        public void UpdateDigitalInputValues(IProfilux basicProtocol, IUpdateProgress progress)
        {
            if (this.DigitalInputs.Count == 0)
            {
                foreach (DigitalInput sensor in this.DigitalInputs)
                {
                    if (progress != null)
                    {
                        progress.IncrementProgress();
                        progress.ProgressText = "Update Digital Input " + sensor.DisplayName;
                        if (progress.StopProcessing)
                        {
                            return;
                        }
                    }

                    sensor.OldValue = sensor.Value;
                    sensor.Value = basicProtocol.GetDigitalInputState(sensor.Index);
                }
            }
            else
            {
                this.UpdateDigitalInputs(basicProtocol, progress);
            }
        }

        /// <summary>
        /// Updates the digital inputs.
        /// </summary>
        /// <param name="basicProtocol">
        /// The basic protocol.
        /// </param>
        /// <param name="progress">
        /// The progress.
        /// </param>
        public void UpdateDigitalInputs(IProfilux basicProtocol, IUpdateProgress progress)
        {
            for (int i = 0; i < basicProtocol.DigitalInputCount; i++)
            {
                if (progress != null)
                {
                    progress.IncrementProgress();
                    progress.ProgressText = "Getting Digital Input" + (i + 1) + " Information";
                    if (progress.StopProcessing)
                    {
                        return;
                    }
                }

                DigitalInputFunction mode;
                try
                {
                    mode = basicProtocol.GetDigitalInputFunction(i);
                }
                catch (ErrorCodeException)
                {
                    continue;
                }

                try
                {
                    int i1 = i;
                    var sensor =
                        (DigitalInput)this.Items.FirstOrDefault(p => p is DigitalInput && ((DigitalInput)p).Index == i1);
                    if (mode != DigitalInputFunction.NotUsed || this.Connection.GetAll)
                    {
                        if (sensor == null)
                        {
                            sensor = new DigitalInput { Id = basicProtocol.GetDigtialInputId(i), Index = i };
                            this.Items.Add(sensor);
                            this.OnPropertyChanged("DigitalInputs");
                        }

                        sensor.Function = mode;

                        sensor.OldValue = sensor.Value;
                        sensor.Value = basicProtocol.GetDigitalInputState(i);
                    }
                    else
                    {
                        if (sensor != null)
                        {
                            this.Items.Remove(sensor);
                            this.OnPropertyChanged("DigitalInputs");
                        }
                    }
                }
                catch (ErrorCodeException ex)
                {
                    Logger.Instance.Log(
                        new LogMessage(2001, string.Format("Digital Input {0} not supported code {1}", i, ex.ErrorCode))
                            {
                               Exception = ex 
                            });
                }
            }
        }

        /// <summary>
        /// Updates the lights.
        /// </summary>
        /// <param name="basicProtocol">
        /// The basic protocol.
        /// </param>
        /// <param name="progress">
        /// The progress.
        /// </param>
        public void UpdateDosingPumps(IProfilux basicProtocol, IUpdateProgress progress)
        {
            for (int i = 0; i < basicProtocol.TimerCount; i++)
            {
                if (progress != null)
                {
                    progress.IncrementProgress();
                    progress.ProgressText = "Getting Timer" + (i + 1) + " Information";
                    if (progress.StopProcessing)
                    {
                        return;
                    }
                }

                try
                {
                    int channel = i;
                    var pump =
                        (DosingPump)
                        this.Items.FirstOrDefault(p => p is DosingPump && ((DosingPump)p).Channel == channel);
                    TimerSettings settings = basicProtocol.GetTimerSettings(i);

                    if (settings.Mode == TimerMode.AutoDosing)
                    {
                        if (pump == null)
                        {
                            pump = new DosingPump { Id = basicProtocol.GetDousingPumpId(channel), Channel = channel };
                            this.Items.Add(pump);

                            this.OnPropertyChanged("DosingPumps");
                        }

                        pump.Settings = settings;
                        pump.OldValue = pump.Value;
                        pump.Rate = basicProtocol.GetDosingRate(i);
                        pump.PerDay = settings.SwitchingCount;
                    }
                    else
                    {
                        if (pump != null)
                        {
                            this.Items.Remove(pump);
                            this.OnPropertyChanged("DosingPumpss");
                        }
                    }
                }
                catch (ErrorCodeException ex)
                {
                    Logger.Instance.Log(
                        new LogMessage(2002, string.Format("Timer {0} not supported code {1}", i, ex.ErrorCode))
                            {
                               Exception = ex 
                            });
                }
            }
        }

        /// <summary>
        /// Updates the dosing pumps value.
        /// </summary>
        /// <param name="basicProtocol">
        /// The basic protocol.
        /// </param>
        /// <param name="pump">
        /// The pump.
        /// </param>
        public void UpdateDosingPumpsValue(IProfilux basicProtocol, DosingPump pump)
        {
            pump.OldValue = pump.Value;
            pump.Rate = basicProtocol.GetDosingRate(pump.Channel);

            TimerSettings settings = basicProtocol.GetTimerSettings(pump.Channel);
            pump.Settings = settings;
            pump.PerDay = settings.SwitchingCount;
        }

        /// <summary>
        /// Updates the update lights values.
        /// </summary>
        /// <param name="basicProtocol">
        /// The basic protocol.
        /// </param>
        /// <param name="progress">
        /// The progress.
        /// </param>
        public void UpdateDosingPumpsValues(IProfilux basicProtocol, IUpdateProgress progress)
        {
            if (this.Lights.Count != 0)
            {
                foreach (DosingPump pump in this.DosingPumps)
                {
                    if (progress != null)
                    {
                        progress.IncrementProgress();
                        progress.ProgressText = "Update Dosing Pump " + pump.DisplayName;
                        if (progress.StopProcessing)
                        {
                            return;
                        }
                    }

                    this.UpdateDosingPumpsValue(basicProtocol, pump);
                }
            }
            else
            {
                this.UpdateDosingPumps(basicProtocol, progress);
            }
        }

        /// <summary>
        /// Updates the dosing rate.
        /// </summary>
        /// <param name="basicProtocol">
        /// The basic protocol.
        /// </param>
        /// <param name="pump">
        /// The pump.
        /// </param>
        /// <param name="rate">
        /// The rate.
        /// </param>
        /// <param name="perDay">
        /// The per day.
        /// </param>
        public void UpdateDosingRate(IProfilux basicProtocol, DosingPump pump, int rate, int perDay)
        {
            basicProtocol.UpdateDosingRate(pump.Channel, rate);
            TimerSettings settings = pump.Settings;
            settings.SwitchingCount = perDay;
            basicProtocol.UpdateTimerSettings(pump.Channel, settings);
            this.UpdateDosingPumpsValue(basicProtocol, pump);
        }

        /// <summary>
        /// Updates the info.
        /// </summary>
        /// <param name="basicProtocol">
        /// The basic protocol.
        /// </param>
        /// <param name="progress">
        /// The progress.
        /// </param>
        public void UpdateInfo(IProfilux basicProtocol, IUpdateProgress progress)
        {
            if (progress != null)
            {
                progress.ProgressText = "Getting Controler Information";
            }

            this.Info.SoftwareVersion = basicProtocol.Version;
            this.Info.ProductId = basicProtocol.ProductId;
            this.Info.SerialNumber = basicProtocol.SerialNumber;
            this.Info.SoftwareDate = basicProtocol.SoftwareDate;
            this.Info.DeviceAddress = basicProtocol.DeviceAddress;
            if (this.Info.IsP3)
            {
                this.Connection.WebServerPort = basicProtocol.WebServerPort;
            }

            this.Info.Latitude = basicProtocol.Latitude;
            this.Info.Longitude = basicProtocol.Longitude;
            this.Info.MoonPhase = basicProtocol.MoonPhase;
            this.UpdateAlarm(basicProtocol.Alarm, progress);
            this.UpdateOperationMode(basicProtocol.OpMode, progress);
            this.UpdateMaintianceMode(basicProtocol, progress, 0);
            this.UpdateMaintianceMode(basicProtocol, progress, 1);
            this.UpdateMaintianceMode(basicProtocol, progress, 2);
            this.UpdateMaintianceMode(basicProtocol, progress, 3);

            for (int i = 0; i < basicProtocol.ReminderCount; i++)
            {
                if (progress != null)
                {
                    progress.IncrementProgress();
                    progress.ProgressText = "Getting Reminder" + (i + 1) + " Information";
                    if (progress.StopProcessing)
                    {
                        return;
                    }
                }

                DateTime? nextReminder = basicProtocol.GetNextReminder(i);

                int index = i;
                Reminder reminder = this.Info.Reminders.FirstOrDefault(r => r.Index == index);

                if (nextReminder.HasValue)
                {
                    if (reminder == null)
                    {
                        reminder = new Reminder { Index = i };
                        this.Info.Reminders.Add(reminder);
                    }

                    // to reduce spam from the e-mail check to see if we have already sent the e-mail
                    if (reminder.Next != nextReminder)
                    {
                        reminder.SentMail = false;
                    }

                    reminder.Next = nextReminder.Value;
                    if (DateTime.Now > reminder.Next)
                    {
                        if (!reminder.IsOverdue)
                        {
                            reminder.IsOverdue = true;
                            Logger.Instance.Log(
                                new LogMessage(
                                    2,
                                    Language.GetResource("strReminder") + " " + reminder.Text +
                                    Language.GetResource("strIsOverdue")));
                        }
                    }
                    else
                    {
                        if (reminder.IsOverdue)
                        {
                            reminder.IsOverdue = false;
                            Logger.Instance.Log(
                                new LogMessage(
                                    0, 
                                    Language.GetResource("strReminder") + " " + reminder.Text +
                                    Language.GetResource("strIsNotOverdue")));
                        }
                    }

                    reminder.Text = basicProtocol.GetReminderText(i);
                    reminder.Period = basicProtocol.GetReminderPeriod(i);
                    reminder.IsRepeating = basicProtocol.GetReminderRepeats(i);
                }
                else
                {
                    if (reminder != null)
                    {
                        this.Info.Reminders.Remove(reminder);
                    }
                }

                this.OnPropertyChanged("ReminderIsOverdue");
            }
        }

        /// <summary>
        /// Updates the alarm.
        /// </summary>
        /// <param name="basicProtocol">
        /// The basic protocol.
        /// </param>
        /// <param name="progress">
        /// The progress.
        /// </param>
        public void UpdateInfoValues(IProfilux basicProtocol, IUpdateProgress progress)
        {
            if (this.Info.SoftwareVersion == 0)
            {
                this.UpdateInfo(basicProtocol, progress);
            }
            else
            {
                this.Info.MoonPhase = basicProtocol.MoonPhase;
                this.UpdateAlarm(basicProtocol.Alarm, progress);
                this.UpdateOperationMode(basicProtocol.OpMode, progress);
                this.UpdateMaintianceMode(basicProtocol, progress, 0);
                this.UpdateMaintianceMode(basicProtocol, progress, 1);
                this.UpdateMaintianceMode(basicProtocol, progress, 2);
                this.UpdateMaintianceMode(basicProtocol, progress, 3);
                this.UpdateReminders(basicProtocol, progress);
            }
        }

        /// <summary>
        /// Updates the L port values.
        /// </summary>
        /// <param name="basicProtocol">
        /// The basic protocol.
        /// </param>
        /// <param name="progress">
        /// The progress.
        /// </param>
        public void UpdateLPortValues(IProfilux basicProtocol, IUpdateProgress progress)
        {
            if (this.LPorts.Count != 0)
            {
                foreach (LPort port in this.LPorts)
                {
                    if (progress != null)
                    {
                        progress.IncrementProgress();
                        progress.ProgressText = "Update 1-10V Port " + port.DisplayName;
                        if (progress.StopProcessing)
                        {
                            return;
                        }
                    }

                    port.SetValue(basicProtocol.GetLPortValue(port.PortNumber));
                }
            }
            else
            {
                this.UpdateLPorts(basicProtocol, progress);
            }
        }

        /// <summary>
        /// Gets the Devices.
        /// </summary>
        /// <param name="basicProtocol">
        /// The basic protocol.
        /// </param>
        /// <param name="progress">
        /// The progress.
        /// </param>
        /// <exception cref="ProtocolException">
        /// If there is a failure in the protocol
        /// </exception>
        /// <exception cref="ConnectionException">
        /// There is a failure in the communication
        /// </exception>
        public void UpdateLPorts(IProfilux basicProtocol, IUpdateProgress progress)
        {
            for (int i = 0; i < basicProtocol.LPortCount; i++)
            {
                if (progress != null)
                {
                    progress.IncrementProgress();
                    progress.ProgressText = "Getting L" + (i + 1) + " Port Information";
                    if (progress.StopProcessing)
                    {
                        return;
                    }
                }

                try
                {
                    int number = i;
                    PortMode mode = basicProtocol.GetLPortFunction(i);
                    var port = (LPort)this.Items.FirstOrDefault(p => p is LPort && ((LPort)p).PortNumber == number);
                    if (mode.DeviceMode != DeviceMode.AlwaysOff || this.Connection.GetAll)
                    {
                        if (port == null)
                        {
                            port = new LPort { Id = basicProtocol.GetLPortId(i), PortNumber = i };
                            port.UpdateMode(mode, this.ProbeCollection);
                            this.Items.Add(port);
                            this.OnPropertyChanged("LPorts");
                        }
                        else
                        {
                            port.UpdateMode(mode, this.ProbeCollection);
                        }

                        port.SetValue(basicProtocol.GetLPortValue(i));
                    }
                    else
                    {
                        if (port != null)
                        {
                            this.Items.Remove(port);
                            this.OnPropertyChanged("LPorts");
                        }
                    }
                }
                catch (ErrorCodeException ex)
                {
                    Logger.Instance.Log(
                        new LogMessage(2003, string.Format("LPort {0} not supported code {1}", i, ex.ErrorCode))
                            {
                               Exception = ex 
                            });
                }
            }
        }

        /// <summary>
        /// Gets the sensors.
        /// </summary>
        /// <param name="basicProtocol">
        /// The basic protocol.
        /// </param>
        /// <param name="progress">
        /// The progress.
        /// </param>
        /// <exception cref="ProtocolException">
        /// If there is a failure in the protocol
        /// </exception>
        /// <exception cref="ConnectionException">
        /// There is a failure in the communication
        /// </exception>
        public void UpdateLevelSensors(IProfilux basicProtocol, IUpdateProgress progress)
        {
            for (int i = 0; i < basicProtocol.LevelSenosrCount; i++)
            {
                if (progress != null)
                {
                    progress.IncrementProgress();
                    progress.ProgressText = "Getting Level Sensor" + (i + 1) + " Information";
                    if (progress.StopProcessing)
                    {
                        return;
                    }
                }

                LevelSensorOpertationMode mode;
                try
                {
                    mode = basicProtocol.GetLevelSensorMode(i);
                }
                catch (ErrorCodeException)
                {
                    continue;
                }

                try
                {
                    int i1 = i;
                    var sensor =
                        (LevelSensor)this.Items.FirstOrDefault(p => p is LevelSensor && ((LevelSensor)p).Index == i1);

                    if (mode != LevelSensorOpertationMode.NotEnabled || this.Connection.GetAll)
                    {
                        if (sensor == null)
                        {
                            sensor = new LevelSensor { Id = basicProtocol.GetLevelId(i), Index = i };
                            this.Items.Add(sensor);
                            this.OnPropertyChanged("LevelSensors");
                        }

                        sensor.OpertationMode = mode;
                        LevelState state = basicProtocol.GetLevelSensorState(i);
                        if (state.Alarm != sensor.IsAlarmOn)
                        {
                            sensor.IsAlarmOn = state.Alarm;
                            Logger.Instance.Log(
                                new LogMessage(
                                    state.Alarm == CurrentState.On ? 1 : 0, 
                                    "Level Sensor " + sensor.DisplayName + " Alarm is now " + state.Alarm));
                        }

                        sensor.OldValue = sensor.Value;
                        sensor.Value = state.State;

                        sensor.WaterMode = state.WaterMode;
                    }
                    else
                    {
                        if (sensor != null)
                        {
                            this.Items.Remove(sensor);
                            this.OnPropertyChanged("LevelSensors");
                        }
                    }
                }
                catch (ErrorCodeException ex)
                {
                    Logger.Instance.Log(
                        new LogMessage(2000, string.Format("Level Sensor {0} not supported code {1}", i, ex.ErrorCode))
                            {
                               Exception = ex 
                            });
                }
            }

            this.OnPropertyChanged("LevelIsAlarmOn");
        }

        /// <summary>
        /// Updates the level sensors value.
        /// </summary>
        /// <param name="basicProtocol">
        /// The basic protocol.
        /// </param>
        /// <param name="sensor">
        /// The sensor.
        /// </param>
        public void UpdateLevelSensorsValue(IProfilux basicProtocol, LevelSensor sensor)
        {
            LevelState state = basicProtocol.GetLevelSensorState(sensor.Index);
            if (state.Alarm != sensor.IsAlarmOn)
            {
                sensor.IsAlarmOn = state.Alarm;
                Logger.Instance.Log(
                    new LogMessage(
                        state.Alarm == CurrentState.On ? 1 : 0, 
                        "Level Sensor " + sensor.DisplayName + " Alarm is now " + state.Alarm));
            }

            sensor.OldValue = sensor.Value;
            sensor.Value = state.State;

            sensor.WaterMode = state.WaterMode;
        }

        /// <summary>
        /// Updates the level sensors values.
        /// </summary>
        /// <param name="basicProtocol">
        /// The basic protocol.
        /// </param>
        /// <param name="progress">
        /// The progress.
        /// </param>
        public void UpdateLevelSensorsValues(IProfilux basicProtocol, IUpdateProgress progress)
        {
            if (this.LevelSensors.Count == 0)
            {
                foreach (LevelSensor sensor in this.LevelSensors)
                {
                    if (progress != null)
                    {
                        progress.IncrementProgress();
                        progress.ProgressText = "Update Level Sensor " + sensor.DisplayName;
                        if (progress.StopProcessing)
                        {
                            return;
                        }
                    }

                    this.UpdateLevelSensorsValue(basicProtocol, sensor);
                }

                this.OnPropertyChanged("LevelIsAlarmOn");
            }
            else
            {
                this.UpdateLevelSensors(basicProtocol, progress);
            }
        }

        /// <summary>
        /// Updates the light value.
        /// </summary>
        /// <param name="basicProtocol">
        /// The basic protocol.
        /// </param>
        /// <param name="light">
        /// The light.
        /// </param>
        public void UpdateLightValue(IProfilux basicProtocol, Light light)
        {
            light.OldValue = light.Value;
            light.Value = basicProtocol.GetLightValue(light.Channel);
        }

        /// <summary>
        /// Updates the lights.
        /// </summary>
        /// <param name="basicProtocol">
        /// The basic protocol.
        /// </param>
        /// <param name="progress">
        /// The progress.
        /// </param>
        /// <param name="skipNonEssntals">
        /// if set to <c>true</c> [skip non essntals].
        /// </param>
        public void UpdateLights(IProfilux basicProtocol, IUpdateProgress progress, bool skipNonEssntals)
        {
            for (int i = 0; i < basicProtocol.LightCount; i++)
            {
                if (progress != null)
                {
                    progress.IncrementProgress();
                    progress.ProgressText = "Getting Light" + (i + 1) + " Information";
                    if (progress.StopProcessing)
                    {
                        return;
                    }
                }

                try
                {
                    int channel = i;
                    var light = (Light)this.Items.FirstOrDefault(p => p is Light && ((Light)p).Channel == channel);

                    if (basicProtocol.IsLightActive(i))
                    {
                        if (light == null)
                        {
                            light = new Light { Id = basicProtocol.GetLightId(channel), Channel = channel, };
                            this.Items.Add(light);

                            this.OnPropertyChanged("Lights");
                        }

                        if (!skipNonEssntals)
                        {
                            light.IsDimmable = basicProtocol.IsLightDimmable(i);
                            this.UpdateLightOperationHours(basicProtocol, light);
                        }

                        light.OldValue = light.Value;
                        light.Value = basicProtocol.GetLightValue(i);

                        if (this.Info.IsP3)
                        {
                            string lightName = basicProtocol.GetLightName(i);
                            if (!string.IsNullOrEmpty(lightName))
                            {
                                light.DisplayName = lightName;
                            }
                        }
                    }
                    else
                    {
                        if (light != null)
                        {
                            this.Items.Remove(light);
                            this.OnPropertyChanged("Lights");
                        }
                    }
                }
                catch (ErrorCodeException ex)
                {
                    Logger.Instance.Log(
                        new LogMessage(2002, string.Format("Light {0} not supported code {1}", i, ex.ErrorCode))
                            {
                               Exception = ex 
                            });
                }
            }

            this.OnPropertyChanged("LightIsOverMaxOperationHours");
        }

        /// <summary>
        /// Updates the update lights values.
        /// </summary>
        /// <param name="basicProtocol">
        /// The basic protocol.
        /// </param>
        /// <param name="progress">
        /// The progress.
        /// </param>
        public void UpdateLightsValues(IProfilux basicProtocol, IUpdateProgress progress)
        {
            if (this.Lights.Count != 0)
            {
                foreach (Light light in this.Lights)
                {
                    if (progress != null)
                    {
                        progress.IncrementProgress();
                        progress.ProgressText = "Update Light " + light.DisplayName;
                        if (progress.StopProcessing)
                        {
                            return;
                        }
                    }

                    this.UpdateLightValue(basicProtocol, light);
                    this.UpdateLightOperationHours(basicProtocol, light);
                }

                this.OnPropertyChanged("LightIsOverMaxOperationHours");
            }
            else
            {
                this.UpdateLights(basicProtocol, progress, false);
            }
        }

        /// <summary>
        /// The update maintiance mode.
        /// </summary>
        /// <param name="basicProtocol">
        /// The basic protocol.
        /// </param>
        /// <param name="progress">
        /// The progress.
        /// </param>
        /// <param name="index">
        /// The index.
        /// </param>
        public void UpdateMaintianceMode(IProfilux basicProtocol, IUpdateProgress progress, int index)
        {
            if (progress != null)
            {
                progress.ProgressText = "Update Maintiance Mode " + index;
            }

            this.Info.Maintenance[index].IsActive = basicProtocol.GetMaintenanceIsActive(index);
            this.Info.Maintenance[index].Duration = basicProtocol.GetMaintenanceDuration(index) * 60;
            this.Info.Maintenance[index].TimeLeft = basicProtocol.GetMaintenanceTimeLeft(index) * 60;
        }

        /// <summary>
        /// Updates the operation mode.
        /// </summary>
        /// <param name="operationMode">
        /// The operation mode.
        /// </param>
        /// <param name="progress">
        /// The progress.
        /// </param>
        public void UpdateOperationMode(OperationMode operationMode, IUpdateProgress progress)
        {
            if (progress != null)
            {
                progress.ProgressText = "Update Operation Mode";
            }

            if (this.Info.OperationMode != operationMode)
            {
                this.Info.OperationMode = operationMode;
                Logger.Instance.Log(
                    new LogMessage(
                        0, Language.GetResource("strOperationModechanged") + Language.GetFrendyName(operationMode)));
            }
        }

        /// <summary>
        /// Updates the probe values.
        /// </summary>
        /// <param name="basicProtocol">
        /// The basic protocol.
        /// </param>
        /// <param name="progress">
        /// The progress.
        /// </param>
        public void UpdateProbeValues(IProfilux basicProtocol, IUpdateProgress progress)
        {
            if (this.Probes.Count != 0)
            {
                foreach (Probe probe in this.Probes)
                {
                    if (progress != null)
                    {
                        progress.IncrementProgress();
                        progress.ProgressText = "Update Sensor " + probe.DisplayName;
                        if (progress.StopProcessing)
                        {
                            return;
                        }
                    }

                    probe.SetValue(basicProtocol.GetSensorValue(probe.Index));
                    CurrentState alarm = basicProtocol.GetSensorAlarm(probe.Index);
                    if (alarm != probe.IsAlarmOn)
                    {
                        probe.IsAlarmOn = alarm;
                        Logger.Instance.Log(
                            new LogMessage(
                                alarm == CurrentState.On ? 1 : 0, 
                                "Sensor " + probe.DisplayName + " Alarm is now " + alarm));
                    }

                    UpdateProbeOperationalHours(basicProtocol, probe);
                }

                this.OnPropertyChanged("SensorIsOverMaxOperationHours");
                this.OnPropertyChanged("SensorIsAlarmOn");
            }
            else
            {
                this.UpdateProbes(basicProtocol, progress, false);
            }
        }

        /// <summary>
        /// Updates the probes.
        /// </summary>
        /// <param name="basicProtocol">
        /// The basic protocol.
        /// </param>
        /// <param name="progress">
        /// The progress.
        /// </param>
        /// <param name="skipNonEssntals">
        /// if set to <c>true</c> [skip non essntals].
        /// </param>
        public void UpdateProbes(IProfilux basicProtocol, IUpdateProgress progress, bool skipNonEssntals)
        {
            for (int i = 0; i < basicProtocol.SensorCount; i++)
            {
                if (progress != null)
                {
                    progress.IncrementProgress();
                    progress.ProgressText = "Getting Sensor" + (i + 1) + " Information";
                    if (progress.StopProcessing)
                    {
                        return;
                    }
                }

                SensorType type;
                SensorMode mode = SensorMode.Normal;
                bool active = false;
                try
                {
                    type = basicProtocol.GetSensorType(i);
                    if (type != SensorType.None && type != SensorType.Free)
                    {
                        mode = basicProtocol.GetSensorMode(i);
                        active = basicProtocol.GetSensorActive(i);
                    }
                }
                catch (ErrorCodeException)
                {
                    continue;
                }

                int i1 = i;
                var probe = (Probe)this.Items.FirstOrDefault(p => p is Probe && ((Probe)p).Index == i1);

                string sensorId = basicProtocol.GetSensorId(i, type);

                if (type != SensorType.None && type != SensorType.Free && (active || this.Connection.GetAll) &&
                    (mode == SensorMode.Normal || this.Connection.GetAll))
                {
                    if (probe == null)
                    {
                        probe = new Probe
                            {
                                Format = basicProtocol.GetSensorFormat(i), 
                                Id = sensorId, 
                                Index = i, 
                                SensorType = type, 
                                SensorMode = mode
                            };
                        this.Items.Add(probe);
                        this.OnPropertyChanged("Probes");
                    }
                    else
                    {
                        probe.Format = basicProtocol.GetSensorFormat(i);
                        probe.SensorMode = mode;
                        probe.Id = sensorId;
                        probe.Index = i;
                        probe.SensorType = type;
                    }

                    if (!skipNonEssntals)
                    {
                        probe.NominalValue = probe.ConvertFromInt(basicProtocol.GetSensorNominalValue(i));
                        probe.AlarmDeviation = probe.ConvertFromInt(basicProtocol.GetSensorAlarmDeviation(i));
                        probe.AlarmEnable = basicProtocol.GetSensorAlarmEnable(i);
                        UpdateProbeOperationalHours(basicProtocol, probe);
                    }

                    if (this.Info.IsP3)
                    {
                        string probeName = basicProtocol.GetProbeName(i);
                        if (!string.IsNullOrEmpty(probeName))
                        {
                            probe.DisplayName = probeName;
                        }
                    }

                    probe.SetValue(basicProtocol.GetSensorValue(i));
                    CurrentState alarm = basicProtocol.GetSensorAlarm(i);
                    if (alarm != probe.IsAlarmOn)
                    {
                        probe.IsAlarmOn = alarm;
                        Logger.Instance.Log(
                            new LogMessage(
                                alarm == CurrentState.On ? 1 : 0, 
                                "Sensor " + probe.DisplayName + " Alarm is now " + alarm));
                    }
                }
                else
                {
                    if (probe != null)
                    {
                        this.Items.Remove(probe);
                        this.OnPropertyChanged("Probes");
                    }
                }
            }

            this.OnPropertyChanged("SensorIsOverMaxOperationHours");
            this.OnPropertyChanged("SensorIsAlarmOn");
        }

        /// <summary>
        /// Updates the reminders.
        /// </summary>
        /// <param name="basicProtocol">
        /// The basic protocol.
        /// </param>
        /// <param name="progress">
        /// The progress.
        /// </param>
        public void UpdateReminders(IProfilux basicProtocol, IUpdateProgress progress)
        {
            foreach (Reminder reminder in this.Info.Reminders)
            {
                if (progress != null)
                {
                    progress.IncrementProgress();
                    progress.ProgressText = "Update Reminder " + reminder.Text;
                    if (progress.StopProcessing)
                    {
                        return;
                    }
                }

                DateTime? nextReminder = basicProtocol.GetNextReminder(reminder.Index);
                if (nextReminder.HasValue)
                {
                    reminder.Next = nextReminder.Value;
                    if (DateTime.Now > reminder.Next)
                    {
                        if (!reminder.IsOverdue)
                        {
                            reminder.IsOverdue = true;
                            Logger.Instance.Log(
                                new LogMessage(
                                    2, 
                                    Language.GetResource("strReminder") + " " + reminder.Text + " " +
                                    Language.GetResource("strIsOverdue")));
                        }
                    }
                    else
                    {
                        if (reminder.IsOverdue)
                        {
                            reminder.IsOverdue = false;
                            Logger.Instance.Log(
                                new LogMessage(
                                    0, 
                                    Language.GetResource("strReminder") + " " + reminder.Text + " " +
                                    Language.GetResource("strIsNotOverdue")));
                        }
                    }
                }
            }

            this.OnPropertyChanged("ReminderIsOverdue");
        }

        /// <summary>
        /// Updates the S port value.
        /// </summary>
        /// <param name="basicProtocol">
        /// The basic protocol.
        /// </param>
        /// <param name="port">
        /// The port.
        /// </param>
        public void UpdateSPortValue(IProfilux basicProtocol, SPort port)
        {
            port.SetValue(basicProtocol.GetSPortValue(port.PortNumber));
        }

        /// <summary>
        /// Gets the Devices.
        /// </summary>
        /// <param name="basicProtocol">
        /// The basic protocol.
        /// </param>
        /// <param name="progress">
        /// The progress.
        /// </param>
        /// <exception cref="ProtocolException">
        /// If there is a failure in the protocol
        /// </exception>
        /// <exception cref="ConnectionException">
        /// There is a failure in the commuincation
        /// </exception>
        public void UpdateSPorts(IProfilux basicProtocol, IUpdateProgress progress)
        {
            for (int i = 0; i < basicProtocol.SPortCount; i++)
            {
                if (progress != null)
                {
                    progress.IncrementProgress();
                    progress.ProgressText = "Getting S" + (i + 1) + " Port Information";
                    if (progress.StopProcessing)
                    {
                        return;
                    }
                }

                try
                {
                    PortMode mode = basicProtocol.GetSPortFunction(i);
                    int portNumber = i;
                    var port = (SPort)this.Items.FirstOrDefault(p => p is SPort && ((SPort)p).PortNumber == portNumber);

                    if (mode.DeviceMode != DeviceMode.AlwaysOff || this.Connection.GetAll)
                    {
                        if (port == null)
                        {
                            port = new SPort { Id = basicProtocol.GetSPortId(portNumber), PortNumber = portNumber };
                            port.UpdateMode(mode, this.ProbeCollection);
                            this.Items.Add(port);
                            this.OnPropertyChanged("SPorts");
                        }
                        else
                        {
                            port.UpdateMode(mode, this.ProbeCollection);
                        }

                        port.SetValue(basicProtocol.GetSPortValue(i));
                        port.OldCurrentValue = port.Current;
                        port.Current = basicProtocol.GetSPortCurrent(i);

                        if (this.Info.IsP3)
                        {
                            string portName = basicProtocol.GetSPortName(i);
                            if (!string.IsNullOrEmpty(portName))
                            {
                                port.DisplayName = portName;
                            }
                        }
                    }
                    else
                    {
                        if (port != null)
                        {
                            this.Items.Remove(port);
                            this.OnPropertyChanged("SPorts");
                        }
                    }
                }
                catch (ErrorCodeException ex)
                {
                    Logger.Instance.Log(
                        new LogMessage(2002, string.Format("SPort {0} not supported code {1}", i, ex.ErrorCode))
                            {
                               Exception = ex 
                            });
                }
            }
        }

        /// <summary>
        /// Updates the S ports value.
        /// </summary>
        /// <param name="basicProtocol">
        /// The basic protocol.
        /// </param>
        /// <param name="progress">
        /// The progress.
        /// </param>
        public void UpdateSPortsValues(IProfilux basicProtocol, IUpdateProgress progress)
        {
            if (this.SPorts.Count != 0)
            {
                foreach (SPort port in this.SPorts)
                {
                    if (progress != null)
                    {
                        progress.IncrementProgress();
                        progress.ProgressText = "Update Socket " + port.DisplayName;
                        if (progress.StopProcessing)
                        {
                            return;
                        }
                    }

                    this.UpdateSPortValue(basicProtocol, port);
                    port.OldCurrentValue = port.Current;
                    port.Current = basicProtocol.GetSPortCurrent(port.PortNumber);
                }
            }
            else
            {
                this.UpdateSPorts(basicProtocol, progress);
            }
        }

        /// <summary>
        /// Updates the values.
        /// </summary>
        /// <param name="protocol">
        /// The protocol.
        /// </param>
        /// <param name="progress">
        /// The progress.
        /// </param>
        /// <returns>
        /// retuns True if the values were updated
        /// </returns>
        public bool UpdateValues(IProfilux protocol, IUpdateProgress progress)
        {
            if (progress != null)
            {
                progress.StopProcessing = false;
                progress.DisplayProgress = true;
                progress.ProgressText = "Log Data Started";
                double steps = this.Probes.Count + this.LevelSensors.Count + this.SPorts.Count + this.LPorts.Count +
                               this.Lights.Count + this.DosingPumps.Count + this.Info.Reminders.Count;
                progress.SetProgressSteps(steps);
            }

            try
            {
                if (progress != null && progress.StopProcessing)
                {
                    return false;
                }

                this.UpdateInfoValues(protocol, progress);
                if (progress != null && progress.StopProcessing)
                {
                    return false;
                }

                this.UpdateProbeValues(protocol, progress);
                if (progress != null && progress.StopProcessing)
                {
                    return false;
                }

                this.UpdateLevelSensorsValues(protocol, progress);
                if (progress != null && progress.StopProcessing)
                {
                    return false;
                }

                this.UpdateSPortsValues(protocol, progress);
                if (progress != null && progress.StopProcessing)
                {
                    return false;
                }

                this.UpdateLPortValues(protocol, progress);
                if (progress != null && progress.StopProcessing)
                {
                    return false;
                }

                this.UpdateDigitalInputValues(protocol, progress);
                if (progress != null && progress.StopProcessing)
                {
                    return false;
                }

                this.UpdateDosingPumpsValues(protocol, progress);
                if (progress != null && progress.StopProcessing)
                {
                    return false;
                }

                this.UpdateLightsValues(protocol, progress);
                if (progress != null && progress.StopProcessing)
                {
                    return false;
                }

                this.UpdateCurrentPumps(protocol, progress);
                if (progress != null && progress.StopProcessing)
                {
                    return false;
                }

                this.Info.LastUpdate = DateTime.Now;
            }
            finally
            {
                if (progress != null)
                {
                    progress.DisplayProgress = false;
                }
            }

            return true;
        }

        /// <summary>
        /// Waters the change.
        /// </summary>
        /// <param name="basicProtocol">
        /// The basic protocol.
        /// </param>
        /// <param name="levelSensor">
        /// The level sensor.
        /// </param>
        public void WaterChange(IProfilux basicProtocol, LevelSensor levelSensor)
        {
            basicProtocol.WaterChange(levelSensor.Index);
        }

        #endregion

        #region Methods

        /// <summary>
        /// The add data item.
        /// </summary>
        private void AddDataItem()
        {
            int userIndex = this.UserValues.Cast<UserInfo>().Aggregate(
                0, (current, item) => Math.Max(int.Parse(item.Id.Substring(3), CultureInfo.InvariantCulture), current));

            userIndex += 1;

            BaseInfo settings = new UserInfo
                {
                   Id = "USR" + userIndex, DisplayName = "USR" + userIndex, Units = string.Empty 
                };

            this.Items.Add(settings);
            this.ChangeUserValues();
            ReefStatusSettings.Instance.Save();

            this.CurrentSelection = settings;
        }

        /// <summary>
        /// Adds the user value.
        /// </summary>
        private void AddUserValue()
        {
            var userInfo = this.CurrentSelection as UserInfo;
            if (userInfo != null)
            {
                // only display the newest value
                if (userInfo.Time < this.NewUserValueDate)
                {
                    userInfo.Value = this.NewUserValue;
                    userInfo.Time = this.NewUserValueDate;
                }

                try
                {
                    using (IDataAccess access = ReefStatusSettings.Instance.Logging.Connection.Create())
                    {
                        access.InsertItem(
                            this.NewUserValue, this.NewUserValueDate, userInfo.Id, false, userInfo.Controler.Id, null);
                    }
                }
                catch (ReefStatusException ex)
                {
                    Logger.Instance.LogError(ex);
                }

                ReefStatusSettings.Instance.Save();

                this.dispatcher.BeginInvoke(
                    new Action(
                        () =>
                        {
                            if (userInfo.HasGraph)
                            {
                                userInfo.Graph.Refresh();
                            }

                            if (userInfo.HasDataPoints)
                            {
                                userInfo.DataPoints.Refresh();
                            }

                            if (userInfo.HasReport)
                            {
                                userInfo.Report.UpdateReport();
                            }
                        }));
            }
        }

        /// <summary>
        /// Clears the level alarm.
        /// </summary>
        /// <param name="item">
        /// The item.
        /// </param>
        private void ClearLevelAlarm(BaseInfo item)
        {
            if (item != null)
            {
                this.Commands.SendClearLevelAlarm((LevelSensor)item);
            }
        }

        /// <summary>
        /// Deletes the item.
        /// </summary>
        /// <param name="item">
        /// The item.
        /// </param>
        private void DeleteItem(BaseInfo item)
        {
            this.Items.Remove(item);
            this.ChangeUserValues();
            ReefStatusSettings.Instance.Save();

            if (this.CurrentSelection == item)
            {
                this.CurrentSelection = null;
            }
        }

        /// <summary>
        /// The light test.
        /// </summary>
        private void LightTest()
        {
            this.Commands.SendOperationMode(
                this.Info.OperationMode == OperationMode.LightTest ? OperationMode.Normal : OperationMode.LightTest);
        }

        /// <summary>
        /// Manuals the sockets.
        /// </summary>
        private void ManualIllumination()
        {
            this.Commands.SendOperationMode(
                this.Info.OperationMode == OperationMode.ManualIllumination
                    ? OperationMode.Normal
                    : OperationMode.ManualIllumination);
        }

        /// <summary>
        /// Manuals the sockets.
        /// </summary>
        private void ManualSockets()
        {
            this.Commands.SendOperationMode(
                this.Info.OperationMode == OperationMode.ManualSockets
                    ? OperationMode.Normal
                    : OperationMode.ManualSockets);
        }

        /// <summary>
        /// News the dosing.
        /// </summary>
        private void NewDosing()
        {
            var pump = this.CurrentSelection as DosingPump;
            if (pump != null)
            {
                pump.Commands.SendUpdateDosingRate(pump, this.NewDosingRate, this.NewDosingPerDay);
                this.dispatcher.BeginInvoke(
                    new Action(
                        () =>
                            {
                                if (pump.HasGraph)
                                {
                                    pump.Graph.Refresh();
                                }

                                if (pump.HasDataPoints)
                                {
                                    pump.DataPoints.Refresh();
                                }

                                if (pump.HasReport)
                                {
                                    pump.Report.UpdateReport();
                                }
                            }));
            }
        }

        /// <summary>
        /// Resets the operational hours.
        /// </summary>
        /// <param name="item">
        /// The item.
        /// </param>
        private void ResetOperationalHours(BaseInfo item)
        {
            this.Commands.SendResetOperationalHours(item);
        }

        /// <summary>
        /// The set all lights.
        /// </summary>
        /// <param name="isOn">
        /// The is on.
        /// </param>
        private void SetAllLights(bool isOn)
        {
            foreach (Light light in this.Lights)
            {
                this.Commands.SendLightState(light, isOn ? 100 : 0, true);
            }
        }

        /// <summary>
        /// The start water change.
        /// </summary>
        /// <param name="item">
        /// The item.
        /// </param>
        private void StartWaterChange(BaseInfo item)
        {
            this.Commands.SendStartWaterChange((LevelSensor)item);
        }

        /// <summary>
        /// The update dosing.
        /// </summary>
        private void UpdateDosing()
        {
            var pump = this.CurrentSelection as DosingPump;
            if (pump != null)
            {
                this.NewDosingPerDay = pump.PerDay;
                this.NewDosingRate = pump.Rate;
            }
        }

        /// <summary>
        /// Updates the item.
        /// </summary>
        private void UpdateItem()
        {
            this.NewUserValueDate = DateTime.Now;
            var item = this.CurrentSelection as BaseInfo;
            if (item != null && item.Value != null && item.Value is double)
            {
                this.NewUserValue = (double)item.Value;
            }
            else
            {
                this.NewUserValue = 0;
            }
        }

        /////////////////////////////////////////////////

        /// <summary>
        /// Updates the light operation hours.
        /// </summary>
        /// <param name="basicProtocol">
        /// The basic protocol.
        /// </param>
        /// <param name="light">
        /// The light.
        /// </param>
        private void UpdateLightOperationHours(IProfilux basicProtocol, Light light)
        {
            int hours = basicProtocol.GetLightOperationHours(light.Channel);

            if (light.EnableMaxOperationHours &&
                (!light.IsOverMaxOperationHours && light.MaxOperationHours < (hours / 60.0)))
            {
                Logger.Instance.Log(
                    new LogMessage(2, string.Format(Language.GetResource("strOpHoursOver"), light.DisplayName)));
            }

            light.OperationHours = hours;
        }

        /// <summary>
        /// Updates the probe operational hours.
        /// </summary>
        /// <param name="basicProtocol">
        /// The basic protocol.
        /// </param>
        /// <param name="probe">
        /// The probe.
        /// </param>
        private static void UpdateProbeOperationalHours(IProfilux basicProtocol, Probe probe)
        {
            int hours = basicProtocol.GetProbeOperationHours(probe.Index);

            if (probe.EnableMaxOperationHours &&
                (!probe.IsOverMaxOperationHours && probe.MaxOperationHours < (hours / 60.0)))
            {
                Logger.Instance.Log(
                    new LogMessage(2, string.Format(Language.GetResource("strOpHoursOver"), probe.DisplayName)));
            }

            probe.OperationHours = hours;
        }

        /// <summary>
        /// The timer tick.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        private void TimerTick(object sender, EventArgs e)
        {
            foreach (var maintenance in this.Info.Maintenance)
            {
                if (maintenance.IsActive)
                {
                    if (maintenance.TimeLeft == 0)
                    {
                        this.Commands.UpdateMaintenance(maintenance);
                    }
                    else
                    {
                        maintenance.TimeLeft--;
                    }
                }
            }
        }

        #endregion
    }
}