﻿// --------------------------------------------------------------------------------------------------------------------
// <copyright file="MainWindowViewModel.cs" company="Redpoint">
//   2009
// </copyright>
// <summary>
//   The main window view model.
// </summary>
// --------------------------------------------------------------------------------------------------------------------

namespace Redpoint.ReefStatus.ScreenSaver
{
    using System;
    using System.Windows;
    using System.Windows.Input;
    using System.Windows.Media;
    using System.Windows.Media.Imaging;
    using System.Windows.Threading;

    using RedPoint.ReefStatus.Common;
    using RedPoint.ReefStatus.Common.ProfiLux;
    using RedPoint.ReefStatus.Common.Settings;
    using RedPoint.ReefStatus.Common.UI.ViewModel;
    using RedPoint.ReefStatus.Common.WebServer;

    /// <summary>
    /// The main window view model.
    /// </summary>
    public class MainWindowViewModel : ViewModelBase
    {
        /// <summary>
        /// The timer display text.
        /// </summary>
        private readonly DispatcherTimer viewDisplayTimer = new DispatcherTimer();

        private Controler controler;

        private string viewText;

        private bool showViewText;

        private string displayText;

        private int currentControler;

        private DateTime lastUpdate;

        private static MainWindowViewModel instance;

        ImageSource graph;

        private int currentGraph;

        private bool showLeftPane;

        /// <summary>
        /// Prevents a default instance of the <see cref="MainWindowViewModel"/> class from being created.
        /// </summary>
        private MainWindowViewModel()
        {
            this.WindowClosingCommand = new DelegateCommand(this.CloseWindow, () => true);
            this.viewDisplayTimer.Interval = new TimeSpan(0, 0, 0, 2);
            this.viewDisplayTimer.Tick += this.TimerDisplayTextTick;
            this.viewDisplayTimer.Start();
            this.Update();
        }

        /// <summary>
        /// Gets the instance.
        /// </summary>
        /// <value>The instance.</value>
        public static MainWindowViewModel Instance
        {
            get
            {
                return instance ?? (instance = new MainWindowViewModel());
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether [show view text].
        /// </summary>
        /// <value><c>true</c> if [show view text]; otherwise, <c>false</c>.</value>
        public bool ShowViewText
        {
            get
            {
                return this.showViewText;
            }

            set
            {
                if (this.showViewText != value)
                {
                    this.showViewText = value;
                    this.OnPropertyChanged("ShowViewText");
                }
            }
        }

        /// <summary>
        /// Gets or sets the controler.
        /// </summary>
        /// <value>The controler.</value>
        public Controler Controler
        {
            get
            {
                return this.controler;
            }

            set
            {
                if (this.controler != value)
                {
                    this.controler = value;
                    this.OnPropertyChanged("Controler");
                }
            }
        }

        /// <summary>
        /// Gets or sets the view text.
        /// </summary>
        /// <value>The view text.</value>
        public string ViewText
        {
            get
            {
                return this.viewText;
            }

            set
            {
                if (this.viewText != value)
                {
                    this.viewText = value;
                    this.OnPropertyChanged("ViewText");
                }
            }
        }

        /// <summary>
        /// Gets or sets the display text.
        /// </summary>
        /// <value>The display text.</value>
        public string DisplayText
        {
            get
            {
                return this.displayText;
            }

            set
            {
                if (this.displayText != value)
                {
                    this.displayText = value;
                    this.OnPropertyChanged("DisplayText");
                }
            }
        }

        /// <summary>
        /// Gets the settings.
        /// </summary>
        /// <value>The settings.</value>
        public ReefStatusSettings Settings
        {
            get
            {
                return ReefStatusSettings.Instance;
            }
        }

        /// <summary>
        /// Gets WindowClosingCommand.
        /// </summary>
        public ICommand WindowClosingCommand { get; private set; }

        /// <summary>
        /// Gets or sets the last update.
        /// </summary>
        /// <value>The last update.</value>
        public DateTime LastUpdate
        {
            get
            {
                return this.lastUpdate;
            }

            set
            {
                this.lastUpdate = value;
                this.OnPropertyChanged("LastUpdate");
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether [show left pane].
        /// </summary>
        /// <value><c>true</c> if [show left pane]; otherwise, <c>false</c>.</value>
        public bool ShowLeftPane
        {
            get
            {
                return this.showLeftPane;
            }

            set
            {
                if (this.showLeftPane != value)
                {
                    this.showLeftPane = value;
                    this.OnPropertyChanged("ShowLeftPane");
                }
            }
        }

        /// <summary>
        /// Gets or sets the graph.
        /// </summary>
        /// <value>The graph.</value>
        public ImageSource Graph
        {
            get
            { 
                return this.graph; 
            }

            set
            {
                if (this.graph != value)
                {
                    this.graph = value;
                    this.OnPropertyChanged("Graph");
                }
            }
        }

        /// <summary>
        /// Loads the bitmap.
        /// </summary>
        /// <param name="source">The source.</param>
        /// <returns>An image source</returns>
        private static BitmapSource LoadBitmap(System.Drawing.Bitmap source)
        {
            return System.Windows.Interop.Imaging.CreateBitmapSourceFromHBitmap(
                source.GetHbitmap(),
                IntPtr.Zero,
                Int32Rect.Empty,
                BitmapSizeOptions.FromEmptyOptions());
        }

        /// <summary>
        /// Closes the window.
        /// </summary>
        private void CloseWindow()
        {
            foreach (Controler item in this.Settings.Controlers)
            {
                item.Commands.Stop();
            }
        }

        /// <summary>
        /// Handles the Tick event of the timerDisplayText control.
        /// </summary>
        /// <param name="sender">
        /// The source of the event.
        /// </param>
        /// <param name="e">
        /// The <see cref="System.EventArgs"/> instance containing the event data.
        /// </param>
        private void TimerDisplayTextTick(object sender, EventArgs e)
        {
            this.Update();
        }

        /// <summary>
        /// Updates this instance.
        /// </summary>
        private void Update()
        {
            this.currentControler++;
            if (this.currentControler >= this.Settings.Controlers.Count)
            {
                this.currentControler = 0;
            }

            this.Controler = this.Settings.Controlers[this.currentControler];

            this.currentGraph++;
            if (this.currentGraph >= this.Controler.Probes.Count)
            {
                this.currentGraph = 0;
            }

            try
            {
                var client = new WebClient { Address = "localhost", Port = ReefStatusSettings.Instance.Web.Port };

                try
                {
                    foreach (var webData in client.GetItemData("probes", this.Controler.Id))
                    {
                        var item = this.Controler.GetItem(webData.Name) as Probe;
                        if (item != null)
                        {
                            item.Value = double.Parse(webData.Value);
                            item.IsAlarmOn = webData.IsAlarmOn;
                        }
                    }

                    //string[] viewTexts = client.GetViewString(this.Controler.Id);
                    //string tempViewText = string.Empty;
                    //this.ViewText =
                    //    viewTexts.Aggregate(tempViewText, (current, line) => current + (line + "\n")).TrimEnd();

                    //string[] displayTexts = client.GetDisplayString(this.Controler.Id);
                    //string tempDisplayText = string.Empty;
                    //this.DisplayText =
                    //    displayTexts.Aggregate(tempDisplayText, (current, line) => current + (line + "\n")).TrimEnd();

                    var probe = (Probe)this.Controler.Probes.GetItemAt(this.currentGraph);
                    this.Graph = LoadBitmap(client.Graph(this.Controler.Id, probe.Id, "day"));

                    ControlerInfo info = client.GetControlerInfo(this.Controler.Id);
                    this.LastUpdate = info.LastUpdate;
                    this.ShowLeftPane = true;
                    this.ShowViewText = false;
                }
                catch (ReefStatusException e)
                {
                    if (e.Code == 6145)
                    {
                        this.Controler.Commands.UpdateProbes(false);
                        //this.ViewText = this.Controler.Commands.GetViewString();
                        //this.DisplayText = this.Controler.Commands.GetDisplayString();
                        this.ShowViewText = false;
                        this.ShowLeftPane = false;
                        this.LastUpdate = DateTime.Now;
                    }
                    else
                    {
                        throw;
                    }
                }
            }
            catch (ReefStatusException ex)
            {
                Logger.Instance.LogError(ex);
            }
        }
    }
}