﻿// <copyright file="OptionsDialog.cs" company="Redpoint Apps">
// Copyright (c) Redpoint Apps. All rights reserved.
// </copyright>

namespace RedPoint.ReefStatus
{
    using System;
    using System.Collections.Generic;
    using System.Globalization;
    using System.Windows.Forms;
    using Microsoft.Win32;
    using RedPoint.ReefStatus.Common;
    using RedPoint.ReefStatus.Common.Database;
    using RedPoint.ReefStatus.Common.ProfiLux;
    using RedPoint.ReefStatus.Common.Settings;
    using RedPoint.ReefStatus.Common.UI;

    /// <summary>
    /// Options dialog
    /// </summary>
    public partial class OptionsDialog : Form
    {
        private readonly ReefStatusSettings settings;

        /// <summary>
        /// Initializes a new instance of the <see cref="OptionsDialog"/> class.
        /// </summary>
        public OptionsDialog()
        {
            this.settings = ReefStatusSettings.Instance;
            InitializeComponent();
            InitializeLanguage();

            logtime.Text = settings.Logging.LogInterval.ToString(CultureInfo.CurrentCulture);
            databaseConnection.Text = settings.Logging.DatabaseConnection;
            databaseType.SelectedIndex = (int)settings.Logging.DatabaseType;

            alarmEnabled.Checked = settings.Mail.SendOnAlarm;
            emalStatus.Checked = settings.Mail.SendStatus;
            emailStatusMode.SelectedIndex = (int)settings.Mail.SendStatusMode;

            emailStatusDuration.Value = settings.Mail.SendStatusDuration;

            this.MailServer.Text = settings.Mail.Server;
            this.mailPort.Value = settings.Mail.Port;
            this.mailTo.Text = settings.Mail.To;
            this.mailFrom.Text = settings.Mail.From;
            this.mailAutentication.Checked = settings.Mail.UsePassword;
            this.userName.Text = settings.Mail.UserName;
            this.password.Text = settings.Mail.Password;
            this.shortMessage.Checked = settings.Mail.SendShortMessage;

            this.addresss.Text = settings.FirstControler.Connection.Address;
            this.port.Text = settings.FirstControler.Connection.Port.ToString(CultureInfo.CurrentCulture);
            this.deviceAddress.Text = settings.FirstControler.Connection.ControllerAddress.ToString(CultureInfo.CurrentCulture);
            this.comPort.Text = settings.FirstControler.Connection.ComPort.ToString(CultureInfo.CurrentCulture);
            this.connectionType.SelectedIndex = (int)settings.FirstControler.Connection.ConnectionType;
            this.pin.Text = settings.FirstControler.Connection.Password;

            this.settingsFile.Text = settings.SettingsFileLocation;

            this.webInterfacePort.Value = settings.Web.Port;
            this.enableWebInterface.Checked = settings.Web.Enable;
            this.webPassword.Text = settings.Web.Password;
            this.enableWebProtection.Checked = settings.Web.Protection;

            this.archiveDuration.Value = settings.Logging.ArchiveDatabaseDuration;
            this.enableArchive.Checked = settings.Logging.ArchiveDatabase;
            this.archiveMode.SelectedIndex = (int)settings.Logging.ArchiveDatabaseMode;

            this.limitDuration.Value = settings.Logging.LimitDatabaseDuration;
            this.enableLimit.Checked = settings.Logging.LimitDatabase;
            this.limitMode.SelectedIndex = (int)settings.Logging.LimitDatabaseMode;

            try
            {
                using (IDataAccess database = settings.Logging.Connection.Create())
                {
                    this.totalRecords.Text = database.RecordCount().ToString(CultureInfo.CurrentCulture);
                    this.recordsWrite.Text = database.WriteCount.ToString(CultureInfo.CurrentCulture);
                    this.recordsDelete.Text = database.DeleteCount.ToString(CultureInfo.CurrentCulture);
                }
            }
            catch (DataAccessException ex)
            {
                Logger.Instance.LogError(ex);
            }

            this.autoStarupService.Checked = settings.AutoServiceStarup;
        }

        /// <summary>
        /// Updates the settings.
        /// </summary>
        public void UpdateSettings()
        {
            settings.Logging.LogInterval = (int)this.logtime.Value;
            settings.Logging.DatabaseConnection = this.databaseConnection.Text;
            settings.Logging.DatabaseType = (DatabaseType)this.databaseType.SelectedIndex;

            settings.Logging.ArchiveDatabaseDuration = (int)this.archiveDuration.Value;
            settings.Logging.ArchiveDatabase = this.enableArchive.Checked;
            settings.Logging.ArchiveDatabaseMode = (DateRangeMode)this.archiveMode.SelectedIndex;

            settings.Logging.LimitDatabaseDuration = (int)this.limitDuration.Value;
            settings.Logging.LimitDatabase = this.enableLimit.Checked;
            settings.Logging.LimitDatabaseMode = (DateRangeMode)this.limitMode.SelectedIndex;

            settings.Mail.SendOnAlarm = this.alarmEnabled.Checked;
            settings.Mail.Server = this.MailServer.Text;
            settings.Mail.Port = (int)this.mailPort.Value;
            settings.Mail.To = this.mailTo.Text;
            settings.Mail.From = this.mailFrom.Text;
            settings.Mail.UsePassword = this.mailAutentication.Checked;
            settings.Mail.UserName = this.userName.Text;
            settings.Mail.Password = this.password.Text;

            settings.Mail.SendStatus = this.emalStatus.Checked;
            settings.Mail.SendStatusDuration = (int)this.emailStatusDuration.Value;
            settings.Mail.SendStatusMode = (DateRangeMode)this.emailStatusMode.SelectedIndex;
            settings.Mail.SendShortMessage = this.shortMessage.Checked;

            settings.FirstControler.Connection.Address = this.addresss.Text;
            settings.FirstControler.Connection.Port = (int)this.port.Value;
            settings.FirstControler.Connection.ControllerAddress = (int)this.deviceAddress.Value;

            settings.SettingsFileLocation = this.settingsFile.Text;

            settings.FirstControler.Connection.ComPort = (int)this.comPort.Value;
            settings.FirstControler.Connection.ConnectionType = (ConnectionType)this.connectionType.SelectedIndex;

            settings.Web.Port = (int)this.webInterfacePort.Value;
            settings.Web.Enable = this.enableWebInterface.Checked;
            settings.Web.Password = this.webPassword.Text;
            settings.Web.Protection = this.enableWebProtection.Checked;

            settings.AutoServiceStarup = this.autoStarupService.Checked;

            settings.FirstControler.Connection.Password = this.pin.Text;

            settings.Save();
        }

        private void InitializeLanguage()
        {
            this.label21.Text = "Type";
            this.connectionTypeLabel.Text = "Type";
            this.Connection.Text = "Connection";
            this.portGroup.Text = "Port";
            this.label16.Text = "COM";
            this.networkGroup.Text = "Network";
            this.label17.Text = "Host Address";
            this.label18.Text = "Port";
            this.label19.Text = "Device Address";
            this.connectionType.Items.AddRange(new object[] 
            {
            "Network",
            "RS232/USB",
            "Web"
            });

            this.label20.Text = "Pin";
            this.tabPage1.Text = "Logging";
            this.enableLimit.Text = "Limit Data Points";

            this.limitMode.Items.AddRange(new object[] 
            {
            "Days",
            "Weeks",
            "Months",
            });

            this.loggingGroup.Text = "Database";
            this.enableArchive.Text = "Arcive Database Every";

            this.archiveMode.Items.AddRange(new object[] 
            {
            "Days",
            "Weeks",
            "Months",
            });

            this.emailStatusMode.Items.AddRange(new object[] 
            {
            "Days",
            "Weeks",
            "Months",
            "Hours",
            });

            this.databaseType.Items.AddRange(new object[] 
            {
            "Access DB",
            "ODBC",
            "SQL Server",
            "Sql lite",
            "Memory"
            });

            this.label1.Text = "Connection";
            this.label2.Text = "Log Every";
            this.label3.Text = "Min";
            this.tabPage3.Text = "E-Mail";
            this.emailSettingsGroup.Text = "E-Mail Settings";
            this.authenticationGroup.Text = "Mail Server Authentication";
            this.label7.Text = "Username";
            this.label9.Text = "Password";
            this.mailAutentication.Text = "Mail Server Requires Authentication";
            this.label8.Text = "From";
            this.label6.Text = "To";
            this.testMail.Text = "Test";
            this.label5.Text = "Port";
            this.label4.Text = "Mail Server (SNTP)";
            this.alarmEnabled.Text = "E-Mail on Alarm and Reminders";
            this.tabPage6.Text = "Web Interface";
            this.webInterfaceGroup.Text = "Settings";
            this.label15.Text = "Port";
            this.enableWebInterface.Text = "Enable";
            this.tabPage5.Text = "Advanced";
            this.settingsFilebutton.Text = "...";
            this.label13.Text = "Settings";
            this.cancel.Text = "Cancel";
            this.okButton.Text = "OK";
            this.langFileButton.Text = "...";
            this.label22.Text = "Language File";
            this.Text = "Options";
        }

        /// <summary>
        /// Handles the CheckedChanged event of the mailAutentication control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void MailAutentication_CheckedChanged(object sender, EventArgs e)
        {
            authenticationGroup.Enabled = mailAutentication.Checked;
        }

        /// <summary>
        /// Handles the Click event of the testMail control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void TestMail_Click(object sender, EventArgs e)
        {
            try
            {
                MailSettings mailSettings = new MailSettings
                                                {
                                                    Server = MailServer.Text,
                                                    Port = (int) mailPort.Value,
                                                    UsePassword = mailAutentication.Checked,
                                                    From = mailFrom.Text,
                                                    To = mailTo.Text,
                                                    UserName = userName.Text,
                                                    Password = password.Text
                                                };

                string body = "This is a test of the Reef status e-mail";
                mailSettings.SendMailMessage("Test Reef Status E-mail", body, body, null);
            }
            catch (ReefStatusException ex)
            {
                Logger.Instance.LogError(ex);
                MessageBox.Show(ex.ToString(), "Error Testing E-Mail");
            }
        }

        /// <summary>
        /// Handles the Load event of the OptionsDialog control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void OptionsDialog_Load(object sender, EventArgs e)
        {
            authenticationGroup.Enabled = mailAutentication.Checked;
            webInterfaceGroup.Enabled = enableWebInterface.Checked;

            limitDuration.Enabled = enableLimit.Checked;
            limitMode.Enabled = enableLimit.Checked;

            archiveDuration.Enabled = enableArchive.Checked;
            archiveMode.Enabled = enableArchive.Checked;

            UpdateArchiveDuration();

            UpdateLimitDuration();

            UpdateEmailStatusDuration();
        }

        /// <summary>
        /// Updates the duration of the limit.
        /// </summary>
        private void UpdateLimitDuration()
        {
            if (limitMode.Text == "Days")
            {
                limitDuration.Minimum = 1;
                limitDuration.Maximum = 365;
            }
            else if (limitMode.Text == "Weeks")
            {
                limitDuration.Minimum = 1;
                limitDuration.Maximum = 52;
            }
            else if (limitMode.Text == "Months")
            {
                limitDuration.Minimum = 1;
                limitDuration.Maximum = 12;
            }
        }

        /// <summary>
        /// Handles the SelectedIndexChanged event of the connectionType control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void ConnectionType_SelectedIndexChanged(object sender, EventArgs e)
        {
            networkGroup.Enabled = connectionType.SelectedIndex == 0 || connectionType.SelectedIndex == 2;
            pin.Enabled = connectionType.SelectedIndex == 2;
            portGroup.Enabled = connectionType.SelectedIndex != 0;
        }

        /// <summary>
        /// Handles the Click event of the settingsFilebutton control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void SettingsFilebutton_Click(object sender, EventArgs e)
        {
            System.Windows.Forms.OpenFileDialog dialog = new System.Windows.Forms.OpenFileDialog
                                                             {
                                                                 FileName = settingsFile.Text,
                                                                 Filter = "All files (*.*)|*.*"
                                                             };
            if (dialog.ShowDialog(this) == DialogResult.OK)
            {
                settingsFile.Text = dialog.FileName;
            }
        }

        /// <summary>
        /// Handles the CheckedChanged event of the enableWebInterface control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void EnableWebInterface_CheckedChanged(object sender, EventArgs e)
        {
            webInterfaceGroup.Enabled = enableWebInterface.Checked;
        }

        /// <summary>
        /// Handles the SelectedIndexChanged event of the limitMode control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void LimitMode_SelectedIndexChanged(object sender, EventArgs e)
        {
            UpdateLimitDuration();
        }

        /// <summary>
        /// Handles the SelectedIndexChanged event of the archiveMode control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void ArchiveMode_SelectedIndexChanged(object sender, EventArgs e)
        {
            UpdateArchiveDuration();
        }

        /// <summary>
        /// Updates the duration of the archive.
        /// </summary>
        private void UpdateArchiveDuration()
        {
            if (archiveMode.Text == "Days")
            {
                archiveDuration.Minimum = 1;
                archiveDuration.Maximum = 365;
            }
            else if (archiveMode.Text == "Weeks")
            {
                archiveDuration.Minimum = 1;
                archiveDuration.Maximum = 52;
            }
            else if (archiveMode.Text == "Months")
            {
                archiveDuration.Minimum = 1;
                archiveDuration.Maximum = 12;
            }
        }

        /// <summary>
        /// Handles the CheckedChanged event of the enableLimit control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void EnableLimit_CheckedChanged(object sender, EventArgs e)
        {
            limitDuration.Enabled = enableLimit.Checked;
            limitMode.Enabled = enableLimit.Checked;
        }

        /// <summary>
        /// Handles the CheckedChanged event of the enableArchive control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void EnableArchive_CheckedChanged(object sender, EventArgs e)
        {
            archiveDuration.Enabled = enableArchive.Checked;
            archiveMode.Enabled = enableArchive.Checked;
        }

        /// <summary>
        /// Handles the Click event of the langFileButton control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void LangFileButton_Click(object sender, EventArgs e)
        {
            System.Windows.Forms.OpenFileDialog dialog = new System.Windows.Forms.OpenFileDialog
                                                             {
                                                                 FileName = langFile.Text,
                                                                 Filter = "All files (*.*)|*.*"
                                                             };
            if (dialog.ShowDialog(this) == DialogResult.OK)
            {
                langFile.Text = dialog.FileName;
            }
        }

        /// <summary>
        /// Handles the SelectedIndexChanged event of the emailStatusMode control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void EmailStatusMode_SelectedIndexChanged(object sender, EventArgs e)
        {
            UpdateEmailStatusDuration();
        }

        /// <summary>
        /// Updates the duration of the email status.
        /// </summary>
        private void UpdateEmailStatusDuration()
        {
            if (emailStatusMode.Text == "Days")
            {
                emailStatusDuration.Minimum = 1;
                emailStatusDuration.Maximum = 365;
            }
            else if (emailStatusMode.Text == "Weeks")
            {
                emailStatusDuration.Minimum = 1;
                emailStatusDuration.Maximum = 52;
            }
            else if (emailStatusMode.Text == "Months")
            {
                emailStatusDuration.Minimum = 1;
                emailStatusDuration.Maximum = 12;
            }
            else if (emailStatusMode.Text == "Hours")
            {
                emailStatusDuration.Minimum = 1;
                emailStatusDuration.Maximum = 24;
            }
        }

        /// <summary>
        /// Handles the Click event of the cleanUpButton control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void CleanUpButton_Click(object sender, EventArgs e)
        {
            ProgressWindow progress = new ProgressWindow { Text = "Clean Up Database" };
            System.Threading.ThreadPool.QueueUserWorkItem(CleanUpDatabaseThread, progress);
            progress.ShowDialog();

            using (IDataAccess database = settings.Logging.Connection.Create())
            {
                totalRecords.Text = database.RecordCount().ToString(CultureInfo.CurrentCulture);
                recordsWrite.Text = database.WriteCount.ToString(CultureInfo.CurrentCulture);
                recordsDelete.Text = database.DeleteCount.ToString(CultureInfo.CurrentCulture);
            }
        }

        /// <summary>
        /// Updates the graphs thread.
        /// </summary>
        /// <param name="status">The status.</param>
        private void CleanUpDatabaseThread(object status)
        {
            IProgressCallback callback = status as IProgressCallback;
            if (callback != null)
            {
                try
                {
                    using (IDataAccess database = settings.Logging.Connection.Create())
                    {
                        Dictionary<int, string> types = database.GetTypes();
                        callback.Begin(0, types.Count, string.Empty);
                        foreach (string type in types.Values)
                        {
                            callback.SetText(Language.GetResource("strCleaningUp") + " " + type);
                            if (callback.IsAborting)
                            {
                                break;
                            }

                            foreach (Controler controler in ReefStatusSettings.Instance.Controlers)
                            {
                                database.CleanupDataSet(type, controler.Id);
                            }
                            
                            callback.Increment(1);
                        }

                        callback.StepTo(types.Count);
                    }
                }
                catch (ReefStatusException ex)
                {
                    Logger.Instance.LogError(ex);
                }
                catch (System.Threading.ThreadAbortException)
                {
                    // We want to exit gracefully here (if we're lucky)
                }
                catch (System.Threading.ThreadInterruptedException)
                {
                    // And here, if we can
                }
                finally
                {
                    callback.End();
                }
            }
        }

        /// <summary>
        /// Handles the LinkClicked event of the LinkLabel1 control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.Forms.LinkLabelLinkClickedEventArgs"/> instance containing the event data.</param>
        private void LinkLabel1_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            System.Diagnostics.Process.Start("http://localhost:" + settings.Web.Port);
        }
    }
}
